@@ -79,6 +79,7 @@ FGAIAircraft::FGAIAircraft(FGAISchedule *ref) :
     tgt_speed = 0;
     speed = 0;
     groundTargetSpeed = 0;
+    spinCounter = 0;
 
     // set heading and altitude locks
     hdg_lock = false;
@@ -125,6 +126,7 @@ void FGAIAircraft::readFromScenario(SGPropertyNode* scFileNode) {
 
     FGAIBase::readFromScenario(scFileNode);
 
+    //acwakecategory = scFileNode->getStringValue("class", "jet_transport");
     setPerformance("", scFileNode->getStringValue("class", "jet_transport"));
     setFlightPlan(scFileNode->getStringValue("flightplan"),
                   scFileNode->getBoolValue("repeat", false));
@@ -194,7 +196,7 @@ void FGAIAircraft::setPerformance(const std::string& acType, const std::string&
     // enough
      if (manager){
         UpdateRadar(manager);
-	invisible = !manager->isVisible(pos);
+    invisible = !manager->isVisible(pos);
      }
   }
 
@@ -500,7 +502,7 @@ bool FGAIAircraft::loadNextLeg(double distance) {
     } else {
         double cruiseAlt = trafficRef->getCruiseAlt() * 100;
 
-        fp->create (this,
+        bool ok = fp->create (this,
                     dep,
                     arr,
                     leg+1,
@@ -514,6 +516,10 @@ bool FGAIAircraft::loadNextLeg(double distance) {
                     acType,
                     company,
                     distance);
+
+        if (!ok) {
+            SG_LOG(SG_AI, SG_WARN, "Failed to create waypoints for leg:" << leg+1);
+        }
        //cerr << "created  leg " << leg << " for " << trafficRef->getCallSign() << endl;
     }
     return true;
@@ -664,7 +670,7 @@ void FGAIAircraft::processATC(const FGATCInstruction& instruction) {
         // if circular waits are resolved.
         // For now, just take the offending aircraft 
         // out of the scene
-	setDie(true);
+    setDie(true);
         // a more proper way should be - of course - to
         // let an offending aircraft take an evasive action
         // for instance taxi back a little bit.
@@ -681,7 +687,7 @@ void FGAIAircraft::processATC(const FGATCInstruction& instruction) {
     } else {
         if (holdPos) {
             //if (trafficRef)
-            //	cerr << trafficRef->getCallSign() << " Resuming Taxi." << endl;
+            //  cerr << trafficRef->getCallSign() << " Resuming Taxi." << endl;
             holdPos = false;
         }
         // Change speed Instruction. This can only be excecuted when there is no
@@ -993,7 +999,7 @@ void FGAIAircraft::controlSpeed(FGAIWaypoint* curr, FGAIWaypoint* next) {
  * Update target values (heading, alt, speed) depending on flight plan or control properties
  */
 void FGAIAircraft::updatePrimaryTargetValues(double dt, bool& flightplanActive, bool& aiOutOfSight) {
-    if (fp)                      // AI object has a flightplan
+    if (fp && fp->isValidPlan()) // AI object has a flightplan
     {
         //TODO make this a function of AIBase
         time_t now = globals->get_time_params()->get_cur_time();
@@ -1094,7 +1100,7 @@ void FGAIAircraft::updateHeading(double dt) {
 //                bank_sense = 1.0;
             }
             //if (trafficRef)
-            //	cerr << trafficRef->getCallSign() << " Heading " 
+            //  cerr << trafficRef->getCallSign() << " Heading " 
             //         << hdg << ". Target " << tgt_heading <<  ". Diff " << fabs(sum - tgt_heading) << ". Speed " << speed << endl;
             //if (headingDiff > 60) {
             groundTargetSpeed = tgt_speed; // * cos(headingDiff * SG_DEGREES_TO_RADIANS);
@@ -1202,7 +1208,7 @@ void FGAIAircraft::updateVerticalSpeedTarget(double dt) {
             } else {
                 tgt_vs = std::max(tgt_altitude_ft - altitude_ft, -_performance->descentRate());
             }
-        } else {
+        } else if (fp->getCurrentWaypoint()) {
             double vert_dist_ft = fp->getCurrentWaypoint()->getCrossat() - altitude_ft;
             double err_dist     = 0; //prev->getCrossat() - altitude_ft;
             double dist_m       = fp->getDistanceToGo(pos.getLatitudeDeg(), pos.getLongitudeDeg(), fp->getCurrentWaypoint());
@@ -1219,9 +1225,13 @@ void FGAIAircraft::updateVerticalSpeedTarget(double dt) {
             if (fabs(tgt_vs) < fabs(min_vs))
                 tgt_vs = min_vs;*/
             //cerr << "target vs : after " << tgt_vs << endl;
+        } else {
+            // avoid crashes when fp has no current waypoint
+            // eg see FLIGHTGEAR-68 on sentry; we crashed in getCrossat()
+            tgt_vs = 0.0;
         }
-    } //else 
-    //    tgt_vs = 0.0;
+    }
+
     checkTcas();
 }
 
@@ -1312,7 +1322,9 @@ bool FGAIAircraft::reachedEndOfCruise(double &distance) {
     FGAIWaypoint* curr = fp->getCurrentWaypoint();
     if (!curr) {
         SG_LOG(SG_AI, SG_WARN, "FGAIAircraft::reachedEndOfCruise: no current waypoint");
-        return false;
+
+        // return true (=done) here, so we don't just get stuck on this forever
+        return true;
     }
     
     if (curr->getName() == string("BOD")) {
