@@ -252,9 +252,6 @@ void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
         setSMPath(submodels->getStringValue("path", ""));
     }
 
-    // By default, scenarios include the full path to any aircraft.
-    _searchOrder = DATA_ONLY;
-
     string searchOrder = scFileNode->getStringValue("search-order", "");
     if (!searchOrder.empty()) {
         if (searchOrder == "DATA_ONLY") {
@@ -619,7 +616,11 @@ bool FGAIBase::init(ModelSearchOrder searchOrder)
         aip.init( _model.get() );
         aip.setVisible(true);
         invisible = false;
-        globals->get_scenery()->get_models_branch()->addChild(aip.getSceneGraph());
+        
+        auto scenery = globals->get_scenery();
+        if (scenery) {
+            scenery->get_models_branch()->addChild(aip.getSceneGraph());
+        }
         _initialized = true;
 
         SG_LOG(SG_AI, SG_DEBUG, "AIBase: Loaded model " << model_path);
