@@ -558,7 +558,7 @@ void FGRouteMgr::update( double dt )
 void FGRouteMgr::clearRoute()
 {
   if (_plan) {
-    _plan->clear();
+      _plan->clearLegs();
   }
 }
 
@@ -805,6 +805,14 @@ void FGRouteMgr::jumpToIndex(int index)
   if (!_plan) {
     return;
   }
+
+  // this method is tied() to current-wp property, but FlightPlan::setCurrentIndex
+  // will throw on invalid input, so guard against invalid values here.
+  // See Sentry FLIGHTGEAR-71
+  if ((index < -1) || (index >= _plan->numLegs())) {
+    SG_LOG(SG_AUTOPILOT, SG_WARN, "FGRouteMgr::jumpToIndex: ignoring invalid index:" << index);
+    return;
+  }
   
   _plan->setCurrentIndex(index);
 }
@@ -1123,7 +1131,7 @@ void FGRouteMgr::setApproach(const std::string& aIdent)
   }
   
   if (!apt || aIdent.empty()) {
-    _plan->setApproach(NULL);
+      _plan->setApproach(static_cast<Approach*>(nullptr));
   } else {
     _plan->setApproach(apt->findApproachWithIdent(aIdent));
   }
