@@ -52,7 +52,7 @@ namespace canvas
       "canvas::Text: using font file " << path
     );
 
-    simgear::canvas::FontPtr font = osgText::readFontFile(path.local8BitStr());
+    simgear::canvas::FontPtr font = osgText::readFontFile(path.utf8Str());
     if( !font )
       SG_LOG
       (
@@ -86,9 +86,9 @@ namespace canvas
       SGPath valid_path = fgValidatePath(p, false);
       if( !valid_path.isNull() )
 #if OSG_VERSION_LESS_THAN(3,4,0)
-          return osgDB::readRefImageFile(valid_path.local8BitStr());
+          return osgDB::readRefImageFile(valid_path.utf8Str());
 #else
-          return osgDB::readRefImageFile(valid_path.local8BitStr());
+          return osgDB::readRefImageFile(valid_path.utf8Str());
 #endif
 
       SG_LOG(SG_IO, SG_ALERT, "canvas::Image: reading '" << path << "' denied");
@@ -98,9 +98,9 @@ namespace canvas
       SGPath tpath = globals->resolve_resource_path(path);
       if( !tpath.isNull() )
 #if OSG_VERSION_LESS_THAN(3,4,0)
-          return osgDB::readImageFile(tpath.local8BitStr());
+          return osgDB::readImageFile(tpath.utf8Str());
 #else
-          return osgDB::readRefImageFile(tpath.local8BitStr());
+          return osgDB::readRefImageFile(tpath.utf8Str());
 #endif
 
       SG_LOG(SG_IO, SG_ALERT, "canvas::Image: No such image: '" << path << "'");
@@ -127,7 +127,7 @@ namespace canvas
     SG_LOG( SG_IO,
             SG_ALERT,
             "FGCanvasSystemAdapter: Failed to get HTTP subsystem" );
-    return 0;
+    return nullptr;
   }
 
 }
