@@ -100,9 +100,9 @@ public:
   }
 
   /** Compute the altitude above ground. */
-  virtual double GetAGLevel(double t, const FGLocation& l,
-                            FGLocation& cont, FGColumnVector3& n,
-                            FGColumnVector3& v, FGColumnVector3& w) const {
+  double GetAGLevel(double t, const FGLocation& l, FGLocation& cont,
+                    FGColumnVector3& n, FGColumnVector3& v, FGColumnVector3& w)
+      const override {
     double contact[3], normal[3], vel[3], angularVel[3];
     double agl = mInterface->get_agl_ft(t, l, SG_METER_TO_FEET*2, contact,
                                         normal, vel, angularVel);
@@ -113,14 +113,14 @@ public:
     return agl;
   }
 
-  virtual double GetTerrainGeoCentRadius(double t, const FGLocation& l) const {
+  double GetTerrainGeoCentRadius(double t, const FGLocation& l) const override {
     double contact[3], normal[3], vel[3], angularVel[3];
     mInterface->get_agl_ft(t, l, SG_METER_TO_FEET*2, contact,
                            normal, vel, angularVel);
     return sqrt(contact[0]*contact[0]+contact[1]*contact[1]+contact[2]*contact[2]);
   }
 
-  virtual double GetSeaLevelRadius(const FGLocation& l) const {
+  double GetSeaLevelRadius(const FGLocation& l) const override {
     double seaLevelRadius, latGeoc;
 
     sgGeodToGeoc(l.GetGeodLatitudeRad(), l.GetGeodAltitude(),
@@ -129,8 +129,7 @@ public:
     return seaLevelRadius * SG_METER_TO_FEET;
   }
 
-  virtual void SetTerrainGeoCentRadius(double radius) {}
-  virtual void SetSeaLevelRadius(double radius) {}
+  void SetTerrainGeoCentRadius(double radius) override {}
 private:
   FGJSBsim* mInterface;
 };
@@ -199,6 +198,7 @@ FGJSBsim::FGJSBsim( double dt )
 
     PropertyManager = new FGPropertyManager( (FGPropertyNode*)globals->get_props() );
     fdmex = new FGFDMExec( PropertyManager );
+    fdmex->Hold();
 
     // Register ground callback.
     fdmex->SetGroundCallback( new FGFSGroundCallback(this) );
@@ -273,19 +273,29 @@ FGJSBsim::FGJSBsim( double dt )
       double d;
       SGPropertyNode * node = fgGetNode("/consumables/fuel/tank", i, true);
       FGTank* tank = Propulsion->GetTank(i);
+      SGPropertyNode * prop = node->getNode( "density-ppg", true );
 
-      d = node->getNode( "density-ppg", true )->getDoubleValue();
+      d = prop->getDoubleValue();
       if( d > 0.0 ) {
         tank->SetDensity( d );
       } else {
-        node->getNode( "density-ppg", true )->setDoubleValue( SG_MAX2<double>(tank->GetDensity(), 0.1) );
+        prop->setDoubleValue( SG_MAX2<double>(tank->GetDensity(), 0.1) );
       }
 
-      d = node->getNode( "level-lbs", true )->getDoubleValue();
+      prop = node->getNode( "level-lbs", true );
+      d = prop->getDoubleValue();
       if( d > 0.0 ) {
         tank->SetContents( d );
       } else {
-        node->getNode( "level-lbs", true )->setDoubleValue( tank->GetContents() );
+        prop->setDoubleValue( tank->GetContents() );
+      }
+
+      prop = node->getNode( "unusable-gal_us", true );
+      d = prop->getDoubleValue();
+      if ( d > 0.0 ) {
+        tank->SetUnusableVolume( d );
+      } else {
+        prop->setDoubleValue(tank->GetUnusableVolume());
       }
       /* Capacity is read-only in FGTank and can't be overwritten from FlightGear */
       node->getNode("capacity-gal_us", true )->setDoubleValue( tank->GetCapacityGallons() );
@@ -374,6 +384,15 @@ FGJSBsim::FGJSBsim( double dt )
 
     mesh = new AircraftMesh(fgGetDouble("/fdm/jsbsim/metrics/bw-ft"),
                             fgGetDouble("/fdm/jsbsim/metrics/cbarw-ft"));
+
+    // Trim once to initialize all output parameters
+    FGTrim *fgtrim = new FGTrim(fdmex,tFull);
+    fgtrim->DoTrim();
+    delete fgtrim;
+
+    string directive_file = fgGetString("/sim/jsbsim/output-directive-file");
+    if (!directive_file.empty())
+      fdmex->SetOutputDirectives(directive_file);
 }
 
 /******************************************************************************/
@@ -418,6 +437,7 @@ void FGJSBsim::init()
     common_init();
 
     copy_to_JSBsim();
+    fdmex->Resume();
     fdmex->RunIC();     //loop JSBSim once w/o integrating
     if (fgGetBool("/sim/presets/running")) {
       Propulsion->InitRunning(-1);
@@ -697,8 +717,8 @@ bool FGJSBsim::copy_to_JSBsim()
 
     Atmosphere->SetTemperature(temperature->getDoubleValue(), get_Altitude(), FGAtmosphere::eCelsius);
     Atmosphere->SetPressureSL(FGAtmosphere::eInchesHg, pressureSL->getDoubleValue());
-    // static_cast<FGStandardAtmosphere*>(Atmosphere)->SetDewPoint(FGAtmosphere::eCelsius,
-    //                                                             dew_point->getDoubleValue());
+    static_cast<FGStandardAtmosphere*>(Atmosphere)->SetDewPoint(FGAtmosphere::eCelsius,
+                                                                dew_point->getDoubleValue());
 
     Winds->SetTurbType((FGWinds::tType)TURBULENCE_TYPE_NAMES[turbulence_model->getStringValue()]);
     switch( Winds->GetTurbType() ) {
@@ -1334,20 +1354,20 @@ bool FGJSBsim::update_ground_cache(const FGLocation& cart, double dt)
   bool cache_ok = prepare_ground_cache_ft( t0, t0 + dt, cart_pos,
                                            groundCacheRadius );
   if (!cache_ok) {
-    SG_LOG(SG_FLIGHT, SG_WARN,
-           "FGInterface is being called without scenery below the aircraft!");
+    //SG_LOG(SG_FLIGHT, SG_WARN,
+    //       "FGInterface is being called without scenery below the aircraft!");
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "altitude         = "
-                      << fgic->GetAltitudeASLFtIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "altitude         = "
+    //                  << fgic->GetAltitudeASLFtIC());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "sea level radius = "
-                      << get_Sea_level_radius());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "sea level radius = "
+    //                  << get_Sea_level_radius());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "latitude         = "
-                      << fgic->GetLatitudeRadIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "latitude         = "
+    //                  << fgic->GetLatitudeRadIC());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "longitude        = "
-                      << fgic->GetLongitudeRadIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "longitude        = "
+    //                  << fgic->GetLongitudeRadIC());
   }
 
   return cache_ok;
@@ -1358,14 +1378,13 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
                      double contact[3], double normal[3], double vel[3],
                      double angularVel[3])
 {
-  const simgear::BVHMaterial* material;
+  const simgear::BVHMaterial* material = nullptr;
   simgear::BVHNode::Id id;
   double pt[3] {loc(1), loc(2), loc(3)};
 
-  // don't check the return value and continue above scenery discontinuity
-  // see http://osdir.com/ml/flightgear-sim/2014-04/msg00145.html
-  FGInterface::get_agl_ft(t, pt, alt_off, contact, normal, vel,
-                          angularVel, material, id);
+  if (!FGInterface::get_agl_ft(t, pt, alt_off, contact, normal, vel,
+                          angularVel, material, id))
+    material = nullptr; // Discard the material data when FGInterface reports an problem.
 
   SGGeod geodPt = SGGeod::fromCart(SG_FEET_TO_METER*SGVec3d(pt));
   SGQuatd hlToEc = SGQuatd::fromLonLat(geodPt);
@@ -1619,3 +1638,9 @@ void FGJSBsim::update_external_forces(double t_off)
       if (_mmag) _mmag->setDoubleValue(0.0);
     }
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<FGJSBsim> registrantFGJSBsim;
+#endif
