@@ -11,21 +11,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -34,16 +34,13 @@ HISTORY
 --------------------------------------------------------------------------------
 11/18/99   JSB   Created
 01/30/01   NHP   Extended gear model to properly simulate steering and braking
-07/08/09   BC    Modified gear model to support large angles between aircraft and ground
+07/08/09   BC    Modified gear model to support large angles between aircraft
+                 and ground
 
-/%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+/%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <cstdlib>
-#include <cstring>
-
-#include "math/FGFunction.h"
 #include "FGLGear.h"
 #include "models/FGGroundReactions.h"
 #include "math/FGTable.h"
@@ -61,8 +58,8 @@ DEFINITIONS
 GLOBAL DATA
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-// Body To Structural (body frame is rotated 180 deg about Y and lengths are given in
-// ft instead of inches)
+// Body To Structural (body frame is rotated 180 deg about Y and lengths are
+// given in ft instead of inches)
 const FGMatrix33 FGLGear::Tb2s(-1./inchtoft, 0., 0., 0., 1./inchtoft, 0., 0., 0., -1./inchtoft);
 const FGMatrix33 FGLGear::Ts2b(-inchtoft, 0., 0., 0., inchtoft, 0., 0., 0., -inchtoft);
 
@@ -111,7 +108,7 @@ FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs&
   Element* strutForce = el->FindElement("strut_force");
   if (strutForce) {
     Element* springFunc = strutForce->FindElement("function");
-    fStrutForce = new FGFunction(PropertyManager, springFunc);
+    fStrutForce = new FGFunction(fdmex, springFunc);
   }
   else {
     if (el->FindElement("spring_coeff"))
@@ -311,9 +308,9 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
       vGroundNormal = in.Tec2b * normal;
 
       // The height returned by GetGroundCallback() is the AGL and is expressed
-      // in the Z direction of the local coordinate frame. We now need to transform
-      // this height in actual compression of the strut (BOGEY) or in the normal
-      // direction to the ground (STRUCTURE)
+      // in the Z direction of the local coordinate frame. We now need to
+      // transform this height in actual compression of the strut (BOGEY) or in
+      // the normal direction to the ground (STRUCTURE)
       double normalZ = (in.Tec2l*normal)(eZ);
       LGearProj = -(mTGear.Transposed() * vGroundNormal)(eZ);
 
@@ -351,12 +348,19 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
 
       vGroundWhlVel = mT.Transposed() * vBodyWhlVel;
 
-      if (fdmex->GetTrimStatus())
+      if (fdmex->GetTrimStatus() || in.TotalDeltaT == 0.0)
         compressSpeed = 0.0; // Steady state is sought during trimming
       else {
         compressSpeed = -vGroundWhlVel(eZ);
         if (eContactType == ctBOGEY)
           compressSpeed /= LGearProj;
+
+        // If the gear is entering in contact with the ground during the current
+        // time step, the compression speed might actually be lower than the
+        // aircraft velocity projected along the gear leg (compressSpeed).
+        double maxCompressSpeed = compressLength/in.TotalDeltaT;
+        if (fabs(compressSpeed) > maxCompressSpeed)
+          compressSpeed = sign(compressSpeed)*maxCompressSpeed;
       }
 
       ComputeVerticalStrutForce();
