@@ -36,10 +36,7 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGPID.h"
-#include "input_output/FGXMLElement.h"
 #include "math/FGParameterValue.h"
-#include <string>
-#include <iostream>
 
 using namespace std;
 
@@ -64,14 +61,12 @@ FGPID::FGPID(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
 
   if (pid_type == "standard") IsStandard = true;
 
-  string kp_string = "0.0";
   el = element->FindElement("kp");
   if (el)
-    kp_string = el->GetDataLine();
+    Kp = new FGParameterValue(el, PropertyManager);
+  else
+    Kp = new FGRealValue(0.0);
 
-  Kp = new FGParameterValue(kp_string, PropertyManager);
-
-  string ki_string = "0.0";
   el = element->FindElement("ki");
   if (el) {
     string integ_type = el->GetAttributeValue("type");
@@ -87,17 +82,17 @@ FGPID::FGPID(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
       IntType = eAdamsBashforth2;
     }
 
-    ki_string = el->GetDataLine();
+    Ki = new FGParameterValue(el, PropertyManager);
   }
+  else
+    Ki = new FGRealValue(0.0);
 
-  Ki = new FGParameterValue(ki_string, PropertyManager);
 
-  string kd_string = "0.0";
   el = element->FindElement("kd");
   if (el)
-    kd_string = el->GetDataLine();
-
-  Kd = new FGParameterValue(kd_string, PropertyManager);
+    Kd = new FGParameterValue(el, PropertyManager);
+  else
+    Kd = new FGRealValue(0.0);
 
   el = element->FindElement("pvdot");
   if (el)
@@ -107,7 +102,8 @@ FGPID::FGPID(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
   if (el)
     Trigger = PropertyManager->GetNode(el->GetDataLine());
 
-  FGFCSComponent::bind();
+  bind(element);
+
   string tmp;
   if (Name.find("/") == string::npos) {
     tmp = "fcs/" + PropertyManager->mkPropertyName(Name, true);
@@ -115,7 +111,8 @@ FGPID::FGPID(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
     tmp = Name;
   }
   typedef double (FGPID::*PMF)(void) const;
-  PropertyManager->Tie(tmp+"/initial-integrator-value", this, (PMF)0, &FGPID::SetInitialOutput);
+  PropertyManager->Tie(tmp+"/initial-integrator-value", this, (PMF)0,
+                       &FGPID::SetInitialOutput);
 
   Debug(0);
 }
@@ -196,7 +193,7 @@ bool FGPID::Run(void )
   Input_prev = Input;
 
   Clip();
-  if (IsOutput) SetOutput();
+  SetOutput();
 
   return true;
 }
