@@ -196,8 +196,10 @@ void YASim::init()
 
     // Are we at ground level?  If so, lift the plane up so the gear
     // clear the ground.
+    bool respect_external_gear_state = fgGetBool("/fdm/yasim/respect-external-gear-state");
     double runway_altitude = get_Runway_altitude();
     if(get_Altitude() - runway_altitude < 50) {
+        bool gear_state = fgGetBool("/controls/gear/gear-down");
         fgSetBool("/controls/gear/gear-down", false);
         float minGearZ = 1e18;
         for(int i=0; i<airplane->numGear(); i++) {
@@ -209,9 +211,9 @@ void YASim::init()
         }
         _set_Altitude(runway_altitude - minGearZ*M2FT);
         // ground start-up: gear down
-        fgSetBool("/controls/gear/gear-down", true);
+        fgSetBool("/controls/gear/gear-down", respect_external_gear_state ? gear_state : true);
     }
-    else
+    else if (! respect_external_gear_state)
     {
         // airborne start-up: gear up
         fgSetBool("/controls/gear/gear-down", false);
@@ -300,7 +302,7 @@ void YASim::copyToYASim(bool copyState)
     atmo.setDensity(dens);
     atmo.setTemperature(temp);
     atmo.setPressure(pressure);
-                            
+
     // Convert and set:
     Model* model = _fdm->getAirplane()->getModel();
     yasim::State s;
@@ -335,14 +337,14 @@ void YASim::copyToYASim(bool copyState)
         Math::tmul33(s.orient, v, v);
         break;
     case KNOTS:
-        v[0] = atmo.spdFromVCAS(get_V_calibrated_kts()/MPS2KTS);
+        v[0] = atmo.speedFromVCAS(get_V_calibrated_kts()/MPS2KTS);
         v[1] = 0;
         v[2] = 0;
         Math::tmul33(s.orient, v, v);
         needCopy = true;
         break;
     case MACH:
-        v[0] = atmo.spdFromMach(get_Mach_number());
+        v[0] = atmo.speedFromMach(get_Mach_number());
         v[1] = 0;
         v[2] = 0;
         Math::tmul33(s.orient, v, v);
@@ -380,9 +382,9 @@ void YASim::copyToYASim(bool copyState)
 // These are set below:
 // _set_Accels_Local
 // _set_Accels_Body
-// _set_Accels_CG_Body 
+// _set_Accels_CG_Body
 // _set_Accels_Pilot_Body
-// _set_Accels_CG_Body_N 
+// _set_Accels_CG_Body_N
 // _set_Velocities_Local
 // _set_Velocities_Ground
 // _set_Velocities_Body
@@ -574,3 +576,9 @@ void YASim::reinit()
     // get current FDM values from the property tree
     copyToYASim(true);
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<YASim> registrantYASim;
+#endif
