@@ -12,6 +12,7 @@
 #include <simgear/package/Catalog.hxx>
 
 #include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
 #include <Network/HTTPClient.hxx>
 
 #include "Add-ons/Addon.hxx"
@@ -72,9 +73,13 @@ AddOnsController::AddOnsController(LauncherMainWindow *parent, LaunchConfig* con
         settings.setArrayIndex(i);
 
         QString path = settings.value("path").toString();
-        m_addonModulePaths.push_back( path );
         const SGPath addonPath(path.toStdString());
+        if (!addonPath.exists()) {
+            // drop non-existing paths on load
+            continue;
+        }
 
+        m_addonModulePaths.push_back( path );
         bool enable = settings.value("enable").toBool();
 
         try {
@@ -146,6 +151,8 @@ QString AddOnsController::addAircraftPath() const
         if (mb.result() == QMessageBox::No) {
             return {};
         }
+
+        flightgear::addSentryBreadcrumb("User continued adding add-on with invalid path", "info");
     }
 
     m_aircraftPaths->appendPath(path);
@@ -295,7 +302,8 @@ void AddOnsController::officialCatalogAction(QString s)
         QSettings settings;
         settings.setValue("hide-official-catalog-message", true);
     } else if (s == "add-official") {
-        m_catalogs->installDefaultCatalog();
+        flightgear::addSentryBreadcrumb("user requested to add the default catalog", "info");
+        m_catalogs->installDefaultCatalog(false);
     }
 
     emit showNoOfficialHangarChanged();
@@ -363,6 +371,12 @@ void AddOnsController::collectArgs()
         settings.setArrayIndex(i);
 
         QString path = settings.value("path").toString();
+        const SGPath addonPath(path.toStdString());
+        if (!addonPath.exists()) {
+            // don't pass missing paths
+            continue;
+        }
+
         bool enable = settings.value("enable").toBool();
         if (enable) {
             m_config->setArg("addon", path);
