@@ -84,7 +84,17 @@ QVariant AddonsModel::get(int idx, int role) const
 {
     if (idx >= 0 && idx < m_addonsList.size()) {
         auto path = m_addonsList[idx];
+        if (!m_addonsMap.contains(path)) {
+            if ((role == PathRole) || (role == Qt::DisplayRole)) {
+                return path;
+            }
+
+            return {};
+        }
+
         auto addon = m_addonsMap[path].addon;
+        if (!addon)
+            return {};
 
         if (role == Qt::DisplayRole) {
             QString name = QString::fromStdString(addon->getName());
@@ -101,7 +111,8 @@ QVariant AddonsModel::get(int idx, int role) const
             return path;
         }
         else if (role == VersionRole) {
-            return QString::fromStdString(addon->getVersion()->str());
+            const auto v = addon->getVersion()->str();
+            return QString::fromStdString(v);
         }
         else if (role == AuthorsRole) {
             QStringList authors;
@@ -137,10 +148,16 @@ QVariant AddonsModel::get(int idx, int role) const
             return tags;
         }
         else if (role == MinFGVersionRole) {
-            return QString::fromStdString(addon->getMinFGVersionRequired());
+            const auto v = addon->getMinFGVersionRequired();
+            if (v == "none")
+                return QStringLiteral("-");
+            return QString::fromStdString(v);
         }
         else if (role == MaxFGVersionRole) {
-            return QString::fromStdString(addon->getMaxFGVersionRequired());
+            const auto v = addon->getMaxFGVersionRequired();
+            if (v == "none")
+                return QStringLiteral("-");
+            return QString::fromStdString(v);
         }
         else if (role == HomePageRole) {
             return QUrl(QString::fromStdString(addon->getHomePage()));
@@ -191,15 +208,24 @@ QHash<int, QByteArray> AddonsModel::roleNames() const
     return m_roleToName;
 }
 
-void AddonsModel::enable(int index, bool enable)
+void AddonsModel::enable(int row, bool enable)
 {
-    if ((index < 0) || (index >= m_addonsList.size())) {
+    if ((row < 0) || (row >= m_addonsList.size())) {
         return;
     }
 
-    auto path = m_addonsList[index];
-    m_addonsMap[path].enable = enable && checkVersion(path);
+    auto path = m_addonsList[row];
+    if (!m_addonsMap.contains(path))
+        return;
 
+    const bool wasEnabled = m_addonsMap[path].enable;
+    const bool nowEnabled = enable && checkVersion(path);
+    if (wasEnabled == nowEnabled)
+        return;
+
+    m_addonsMap[path].enable = nowEnabled;
+    const auto mindex = index(row, 0);
+    emit dataChanged(mindex, mindex, {EnableRole});
     emit modulesChanged();
 }
 
@@ -207,6 +233,10 @@ bool AddonsModel::checkVersion(QString path) const
 {
     using namespace simgear;
 
+    if (!m_addonsMap.contains(path)) {
+        return false;
+    }
+
     // Check that the FlightGear version satisfies the add-on requirements
     std::string minFGversion = m_addonsMap[path].addon->getMinFGVersionRequired();
     if (strutils::compare_versions(FLIGHTGEAR_VERSION, minFGversion) < 0) {
@@ -220,4 +250,4 @@ bool AddonsModel::checkVersion(QString path) const
     }
 
     return true;
-}
\ No newline at end of file
+}
