@@ -97,6 +97,8 @@ protected:
         if (aReason == STATUS_SUCCESS) {
             m_model->installSucceeded(mi);
         }
+
+        m_model->installedAircraftCountChanged();
     }
 
     void availablePackagesChanged() override
@@ -115,6 +117,7 @@ protected:
     {
         QModelIndex mi(indexForPackage(pkg));
         m_model->dataChanged(mi, mi);
+        m_model->installedAircraftCountChanged();
     }
 
 private:
@@ -141,7 +144,7 @@ void PackageDelegate::catalogRefreshed(CatalogRef aCatalog, StatusCode aReason)
         m_model->refreshPackages();
     } else if (aReason ==  FAIL_VERSION) {
         // silent about this
-    } else {
+    } else if (aCatalog) {
         qWarning() << "failed refresh of"
             << QString::fromStdString(aCatalog->url()) << ":" << aReason << endl;
     }
@@ -244,7 +247,7 @@ QVariant AircraftItemModel::data(const QModelIndex& index, int role) const
 
     if (role == AircraftIsFavouriteRole) {
         // recursive call here, hope that's okay
-        const auto uri = data(index, AircraftURIRole).toUrl();
+        const auto uri = data(index, AircraftPrimaryURIRole).toUrl();
         return FavouriteAircraftData::instance()->isFavourite(uri);
     }
 
@@ -277,11 +280,15 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
     if (role >= AircraftVariantDescriptionRole) {
         int variantIndex = role - AircraftVariantDescriptionRole;
         if (variantIndex == 0) {
-            return item->description;
+            return item->name();
         }
 
         Q_ASSERT(variantIndex < item->variants.size());
-        return item->variants.at(variantIndex)->description;
+        return item->variants.at(variantIndex)->name();
+    }
+
+    if (role == AircraftPrimaryURIRole) {
+        return QUrl::fromLocalFile(item->path);
     }
 
     if (state.variant) {
@@ -292,11 +299,11 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
     }
 
     if (role == Qt::DisplayRole) {
-        if (item->description.isEmpty()) {
+        if (item->name().isEmpty()) {
             return tr("Missing description for: %1").arg(item->baseName());
         }
 
-        return item->description;
+        return item->name();
     } else if (role == AircraftPathRole) {
         return item->path;
     } else if (role == AircraftAuthorsRole) {
@@ -318,7 +325,7 @@ QVariant AircraftItemModel::dataFromItem(AircraftItemPtr item, const DelegateSta
         }
         return have;
     } else if (role == AircraftLongDescriptionRole) {
-        return item->longDescription;
+        return item->description();
     } else if (role == AircraftIsHelicopterRole) {
         return item->usesHeliports;
     } else if (role == AircraftIsSeaplaneRole) {
@@ -397,6 +404,8 @@ QVariant AircraftItemModel::dataFromPackage(const PackageRef& item, const Delega
         return static_cast<int>(item->fileSizeBytes());
     } else if (role == AircraftURIRole) {
         return QUrl("package:" + QString::fromStdString(item->qualifiedVariantId(state.variant)));
+    } else if (role == AircraftPrimaryURIRole) {
+        return QUrl("package:" + QString::fromStdString(item->qualifiedId()));
     } else if (role == AircraftHasRatingsRole) {
         return item->properties()->hasChild("rating");
     } else if ((role >= AircraftRatingRole) && (role < AircraftVariantDescriptionRole)) {
@@ -408,6 +417,10 @@ QVariant AircraftItemModel::dataFromPackage(const PackageRef& item, const Delega
         if (!v.empty()) {
             return QString::fromStdString(v);
         }
+    } else if (role == AircraftIsHelicopterRole) {
+        return item->hasTag("helicopter");
+    } else if (role == AircraftIsSeaplaneRole) {
+        return item->hasTag("seaplane") || item->hasTag("floats");
     }
 
     return QVariant();
@@ -432,7 +445,7 @@ bool AircraftItemModel::setData(const QModelIndex &index, const QVariant &value,
           emit dataChanged(index, index);
           return true;
       } else if (role == AircraftIsFavouriteRole) {
-          const auto uri = data(index, AircraftURIRole).toUrl();
+          const auto uri = data(index, AircraftPrimaryURIRole).toUrl();
           bool changed = FavouriteAircraftData::instance()->setFavourite(uri, value.toBool());
           if (changed) {
               emit dataChanged(index, index);
@@ -564,14 +577,14 @@ QString AircraftItemModel::nameForAircraftURI(QUrl uri) const
 
         const QString path = uri.toLocalFile();
         if (item->path == path) {
-            return item->description;
+            return item->name();
         }
 
         // check variants too
         for (int vr=0; vr < item->variants.size(); ++vr) {
             auto variant = item->variants.at(vr);
             if (variant->path == path) {
-                return variant->description;
+                return variant->name();
             }
         }
     } else if (uri.scheme() == "package") {
@@ -588,6 +601,17 @@ QString AircraftItemModel::nameForAircraftURI(QUrl uri) const
     return {};
 }
 
+int AircraftItemModel::installedAircraftCount() const
+{
+    int c = m_cachedLocalAircraftCount;
+
+    for (const auto& cat : m_packageRoot->catalogs()) {
+        c += static_cast<int>(cat->installedPackages().size());
+    }
+
+    return c;
+}
+
 void AircraftItemModel::onScanAddedItems(int addedCount)
 {
     Q_UNUSED(addedCount)
@@ -601,6 +625,7 @@ void AircraftItemModel::onScanAddedItems(int addedCount)
     m_cachedLocalAircraftCount += newItemCount;
     endInsertRows();
     emit contentsChanged();
+    emit installedAircraftCountChanged();
 }
 
 void AircraftItemModel::onLocalCacheCleared()
@@ -614,6 +639,8 @@ void AircraftItemModel::onLocalCacheCleared()
         m_cachedLocalAircraftCount = 0;
         endRemoveRows();
     }
+
+    emit installedAircraftCountChanged();
 }
 
 void AircraftItemModel::installFailed(QModelIndex index, simgear::pkg::Delegate::StatusCode reason)
