@@ -107,6 +107,9 @@ AirportDiagram::AirportDiagram(QQuickItem* pr) :
     m_parkingIconLeftPath.lineTo(16, 16);
     m_parkingIconLeftPath.lineTo(0, 0);
 
+    m_helipadBoundsPath.moveTo(0, 0);
+    m_helipadBoundsPath.addEllipse(QPointF(0, 0), 16.0, 16.0);
+
     m_helipadIconPath.moveTo(0,0);
     m_helipadIconPath.addEllipse(QPointF(0, 0), 16.0, 16.0);
     m_helipadIconPath.addEllipse(QPointF(0, 0), 13.0, 13.0);
@@ -147,6 +150,11 @@ void AirportDiagram::setAirport(FGAirportRef apt)
                 addHelipad(pad);
             }
         } else {
+            for (unsigned int r = 0; r < apt->numHelipads(); ++r) {
+                FGHelipadRef pad = apt->getHelipadByIndex(r);
+                addHelipad(pad);
+            }
+
             for (unsigned int r=0; r<apt->numRunways(); ++r) {
                 addRunway(apt->getRunwayByIndex(r));
             }
@@ -561,11 +569,15 @@ void AirportDiagram::mouseReleaseEvent(QMouseEvent* me)
     me->accept();
     QTransform t(transform());
     double minWidth = 8.0 * unitLengthAfterMapping(t.inverted());
-#if 0
-    QImage img(width(), height(), QImage::Format_ARGB32);
-    QPainter imgPaint(&img);
-    imgPaint.setPen(QPen(Qt::cyan, 1));
-#endif
+
+    Q_FOREACH (const HelipadData& pad, m_helipads) {
+        QPainterPath pp = pathForHelipad(pad, t);
+        //imgPaint.drawPath(pp);
+        if (pp.contains(me->pos())) {
+            emit clicked(new QmlPositioned{pad.helipad});
+            return;
+        }
+    }
 
     Q_FOREACH(const RunwayData& r, m_runways) {
         QPainterPath pp = pathForRunway(r, t, minWidth);
@@ -588,14 +600,7 @@ void AirportDiagram::mouseReleaseEvent(QMouseEvent* me)
         }
     }
 
-    Q_FOREACH(const HelipadData& pad, m_helipads) {
-        QPainterPath pp = pathForHelipad(pad, t);
-        //imgPaint.drawPath(pp);
-        if (pp.contains(me->pos())) {
-            emit clicked(new QmlPositioned{pad.helipad});
-            return;
-        }
-    }
+
 #if 0
     img.save("/Users/jmt/Desktop/img.png");
 #endif
@@ -642,7 +647,7 @@ QPainterPath AirportDiagram::pathForHelipad(const HelipadData& h, const QTransfo
 {
     QTransform x = t;
     x.translate(h.pt.x(), h.pt.y());
-    return x.map(m_helipadIconPath);
+    return x.map(m_helipadBoundsPath);
 }
 
 void AirportDiagram::buildTaxiways()
