@@ -35,6 +35,7 @@
 // FlightGear
 #include <Main/globals.hxx>
 #include <Network/HTTPClient.hxx>
+#include <Main/sentryIntegration.hxx>
 
 using namespace simgear::pkg;
 
@@ -198,12 +199,16 @@ void CatalogListModel::refreshCatalog(int index)
     m_catalogs.at(static_cast<size_t>(index))->refresh();
 }
 
-void CatalogListModel::installDefaultCatalog()
+void CatalogListModel::installDefaultCatalog(bool showAddFeedback)
 {
     FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
-    m_newlyAddedCatalog = Catalog::createFromUrl(m_packageRoot, http->getDefaultCatalogUrl());
-    emit isAddingCatalogChanged();
-    emit statusOfAddingCatalogChanged();
+    CatalogRef cat = Catalog::createFromUrl(m_packageRoot, http->getDefaultCatalogUrl());
+    if (showAddFeedback) {
+      m_newlyAddedCatalog = cat;
+      emit isAddingCatalogChanged();
+      emit statusOfAddingCatalogChanged();
+    }
+
     resetData();
 }
 
@@ -215,6 +220,7 @@ void CatalogListModel::addCatalogByUrl(QUrl url)
     }
 
     m_newlyAddedCatalog = Catalog::createFromUrl(m_packageRoot, url.toString().toStdString());
+    flightgear::addSentryBreadcrumb("CatalogListModel: Adding catalog " + url.toString().toStdString(), "info");
     resetData();
     emit isAddingCatalogChanged();
 }
@@ -243,12 +249,13 @@ void CatalogListModel::finalizeAddCatalog()
         return;
     }
 
+    flightgear::addSentryBreadcrumb("CatalogListModel: finalziing add of:" + m_newlyAddedCatalog->id(), "info");
+
     const int row = static_cast<int>(std::distance(m_catalogs.begin(), it));
     m_newlyAddedCatalog.clear();
     emit isAddingCatalogChanged();
     emit statusOfAddingCatalogChanged();
     emit dataChanged(index(row), index(row));
-
 }
 
 void CatalogListModel::abandonAddCatalog()
@@ -316,7 +323,7 @@ CatalogListModel::CatalogStatus CatalogListModel::translateStatusForCatalog(Cata
     case Delegate::FAIL_DOWNLOAD:       return NetworkError;
     case Delegate::STATUS_IN_PROGRESS:  return Refreshing;
     case Delegate::FAIL_NOT_FOUND:      return NotFoundOnServer;
-    case Delegate::FAIL_VERSION:        return IncomaptbleVersion;
+    case Delegate::FAIL_VERSION:        return IncompatibleVersion;
     case Delegate::FAIL_HTTP_FORBIDDEN: return HTTPForbidden;
     case Delegate::FAIL_VALIDATION:     return InvalidData;
     default:
