@@ -2,16 +2,16 @@
 
 // Qt headers
 #include <QDebug>
-#include <QSettings>
+#include <QDesktopServices>
+#include <QFileDialog>
+#include <QJSEngine>
+#include <QMessageBox>
 #include <QNetworkAccessManager>
 #include <QNetworkDiskCache>
-#include <QDesktopServices>
-#include <QMessageBox>
-#include <QSettings>
-#include <QQuickWindow>
-#include <QQmlComponent>
 #include <QPushButton>
-#include <QFileDialog>
+#include <QQmlComponent>
+#include <QQuickWindow>
+#include <QSettings>
 
 // simgear headers
 #include <simgear/package/Install.hxx>
@@ -26,36 +26,43 @@
 #include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
 #include "version.h"
+#include <Main/sentryIntegration.hxx>
 
-#include "QtLauncher.hxx"
-#include "QmlAircraftInfo.hxx"
-#include "LauncherArgumentTokenizer.hxx"
-#include "PathUrlHelper.hxx"
-#include "PopupWindowTracker.hxx"
-#include "RecentAircraftModel.hxx"
-#include "RecentLocationsModel.hxx"
-#include "ThumbnailImageItem.hxx"
-#include "PreviewImageItem.hxx"
-#include "MPServersModel.h"
-#include "AircraftSearchFilterModel.hxx"
-#include "DefaultAircraftLocator.hxx"
-#include "LaunchConfig.hxx"
 #include "AircraftModel.hxx"
-#include "LocationController.hxx"
-#include "QmlPositioned.hxx"
-#include "PixmapImageItem.hxx"
+#include "AircraftSearchFilterModel.hxx"
 #include "AirportDiagram.hxx"
 #include "CarrierDiagram.hxx"
-#include "NavaidDiagram.hxx"
-#include "RouteDiagram.hxx"
-#include "QmlRadioButtonHelper.hxx"
-#include "UnitsModel.hxx"
-#include "NavaidSearchModel.hxx"
-#include "FlightPlanController.hxx"
-#include "ModelDataExtractor.hxx"
 #include "CarriersLocationModel.hxx"
-#include "SetupRootDialog.hxx"
+#include "DefaultAircraftLocator.hxx"
+#include "FlightPlanController.hxx"
 #include "HoverArea.hxx"
+#include "LaunchConfig.hxx"
+#include "LauncherArgumentTokenizer.hxx"
+#include "LauncherNotificationsController.hxx"
+#include "LocationController.hxx"
+#include "MPServersModel.h"
+#include "ModelDataExtractor.hxx"
+#include "NavaidDiagram.hxx"
+#include "NavaidSearchModel.hxx"
+#include "PathUrlHelper.hxx"
+#include "PixmapImageItem.hxx"
+#include "PreviewImageItem.hxx"
+#include "QmlAircraftInfo.hxx"
+#include "QmlPositioned.hxx"
+#include "QmlRadioButtonHelper.hxx"
+#include "QtLauncher.hxx"
+#include "RecentAircraftModel.hxx"
+#include "RecentLocationsModel.hxx"
+#include "RouteDiagram.hxx"
+#include "SetupRootDialog.hxx"
+#include "StackController.hxx"
+#include "ThumbnailImageItem.hxx"
+#include "UnitsModel.hxx"
+#include "UpdateChecker.hxx"
+#include "GettingStartedTipsController.hxx"
+#include "GettingStartedTip.hxx"
+#include "TipBackgroundBox.hxx"
+#include "GettingStartedScope.hxx"
 
 using namespace simgear::pkg;
 
@@ -132,6 +139,22 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
         const auto ws = static_cast<Qt::WindowState>(settings.value("window-state").toInt());
         m_window->setWindowState(ws);
     }
+
+    // count launches; we use this to trigger first-run and periodic notices
+    // in the UI.
+    m_launchCount = settings.value("launch-count", 0).toInt();
+    settings.setValue("launch-count", m_launchCount + 1);
+
+    std::ostringstream os;
+    string_list versionParts = simgear::strutils::split(FLIGHTGEAR_VERSION, ".");
+    if (versionParts.size() >= 2) {
+        // build a setting key like launch-count-2020-2
+        QString versionedCountKey = QString::fromStdString("launch-count-" + versionParts.at(0) + "-" + versionParts.at(1));
+        m_versionLaunchCount = settings.value(versionedCountKey, 0).toInt();
+        settings.setValue(versionedCountKey, m_versionLaunchCount + 1);
+    }
+
+    QTimer::singleShot(2000, this, &LauncherController::checkForOldDownloadDir);
 }
 
 void LauncherController::initQML()
@@ -139,6 +162,7 @@ void LauncherController::initQML()
     qmlRegisterUncreatableType<LauncherController>("FlightGear.Launcher", 1, 0, "LauncherController", "no");
     qmlRegisterUncreatableType<LocationController>("FlightGear.Launcher", 1, 0, "LocationController", "no");
     qmlRegisterUncreatableType<FlightPlanController>("FlightGear.Launcher", 1, 0, "FlightPlanController", "no");
+    qmlRegisterUncreatableType<UpdateChecker>("FlightGear.Launcher", 1, 0, "UpdateChecker", "for enums");
 
     qmlRegisterType<LauncherArgumentTokenizer>("FlightGear.Launcher", 1, 0, "ArgumentTokenizer");
     qmlRegisterUncreatableType<QAbstractItemModel>("FlightGear.Launcher", 1, 0, "QAIM", "no");
@@ -154,9 +178,8 @@ void LauncherController::initQML()
     qmlRegisterUncreatableType<Units>("FlightGear", 1, 0, "Units", "Only for enum");
     qmlRegisterType<UnitsModel>("FlightGear", 1, 0, "UnitsModel");
 
-    qmlRegisterType<FileDialogWrapper>("FlightGear.Launcher", 1, 0, "FileDialog");
+    qmlRegisterType<FileDialogWrapper>("FlightGear", 1, 0, "FileDialog");
     qmlRegisterType<QmlAircraftInfo>("FlightGear.Launcher", 1, 0, "AircraftInfo");
-    qmlRegisterType<PopupWindowTracker>("FlightGear.Launcher", 1, 0, "PopupWindowTracker");
 
     qmlRegisterUncreatableType<LocalAircraftCache>("FlightGear.Launcher", 1, 0, "LocalAircraftCache", "Aircraft cache");
     qmlRegisterUncreatableType<AircraftItemModel>("FlightGear.Launcher", 1, 0, "AircraftModel", "Built-in model");
@@ -174,11 +197,17 @@ void LauncherController::initQML()
     qmlRegisterType<RouteDiagram>("FlightGear", 1, 0, "RouteDiagram");
     qmlRegisterType<QmlRadioButtonGroup>("FlightGear", 1, 0, "RadioButtonGroup");
     qmlRegisterType<HoverArea>("FlightGear", 1, 0, "HoverArea");
+    qmlRegisterType<StackController>("FlightGear", 1, 0, "StackController");
 
     qmlRegisterType<ModelDataExtractor>("FlightGear", 1, 0, "ModelDataExtractor");
 
     qmlRegisterSingletonType(QUrl("qrc:/qml/OverlayShared.qml"), "FlightGear", 1, 0, "OverlayShared");
 
+    qmlRegisterType<GettingStartedScope>("FlightGear", 1, 0, "GettingStartedScope");
+    qmlRegisterType<GettingStartedTipsController>("FlightGear", 1, 0, "GettingStartedController");
+    qmlRegisterType<GettingStartedTip>("FlightGear", 1, 0, "GettingStartedTip");
+    qmlRegisterType<TipBackgroundBox>("FlightGear", 1, 0, "TipBackgroundBox");
+
     QNetworkDiskCache* diskCache = new QNetworkDiskCache(this);
     SGPath cachePath = globals->get_fg_home() / "PreviewsCache";
     diskCache->setCacheDirectory(QString::fromStdString(cachePath.utf8Str()));
@@ -310,6 +339,7 @@ void LauncherController::restoreAircraft()
 
 void LauncherController::doRun()
 {
+    flightgear::addSentryBreadcrumb("Launcher: fly!", "info");
     flightgear::Options* opt = flightgear::Options::sharedInstance();
     m_config->reset();
     m_config->collect();
@@ -443,6 +473,13 @@ void LauncherController::updateSelectedAircraft()
         }
     }
 
+    if (!m_aircraftState.isEmpty()) {
+      if (!m_selectedAircraftInfo->hasState(m_aircraftState)) {
+        m_aircraftState.clear();
+        emit selectedAircraftStateChanged();
+      }
+    }
+
     emit canFlyChanged();
 }
 
@@ -845,6 +882,18 @@ void LauncherController::setAircraftGridMode(bool aircraftGridMode)
     emit aircraftGridModeChanged(m_aircraftGridMode);
 }
 
+void LauncherController::resetGettingStartedTips()
+{
+    {
+        QSettings settings;
+        settings.beginGroup("GettingStarted-DontShow");
+        settings.remove(""); // remove all keys in the current group
+        settings.endGroup();
+    } // ensure settings are written, before we emit the signal
+
+    emit didResetGettingStartedTips();
+}
+
 void LauncherController::setMinWindowSize(QSize sz)
 {
     if (sz == m_minWindowSize)
@@ -866,3 +915,66 @@ QUrl LauncherController::flyIconUrl() const
 
     return QUrl{"qrc:///svg/toolbox-fly"};
 }
+
+QString LauncherController::flyButtonLabel() const
+{
+    if (m_aircraftType == Helicopter) {
+        return tr("Fly!", "For a helicopter");
+    } else if (m_selectedAircraftInfo) {
+        if (m_selectedAircraftInfo->hasTag("spaceship")) {
+            return tr("Fly!", "For a spaceship");
+        }
+    }
+
+    return tr("Fly!");
+}
+
+QUrl LauncherController::urlToDataPath(QString relPath) const
+{
+    QString absFilePath = QString::fromStdString(globals->get_fg_root().utf8Str());
+    if (!relPath.startsWith("/")) {
+        relPath.prepend("/");
+    }
+    return QUrl::fromLocalFile(absFilePath + relPath);
+}
+
+void LauncherController::checkForOldDownloadDir()
+{
+#if defined(Q_OS_WIN)
+    auto options = flightgear::Options::sharedInstance();
+    if (options->valueForOption("download-dir") != std::string{}) {
+        return; // if we're using a custom value, nothing to do
+    }
+
+    if (haveOldWindowsDownloadDir()) {
+        // the notifications logic handles 'don't show again' lgoic internally,
+        // so we can always trigger this check
+        auto nc = LauncherNotificationsController::instance();
+        QJSValue args = nc->jsEngine()->newObject();
+
+        const auto oldPath = SGPath::documents() / "FlightGear";
+        const auto newPath = flightgear::defaultDownloadDir();
+
+        const QUrl oldLocURI = QUrl::fromLocalFile(QString::fromStdString(oldPath.utf8Str()));
+        const QUrl newLocURI = QUrl::fromLocalFile(QString::fromStdString(newPath.utf8Str()));
+
+        args.setProperty("oldLocation", oldLocURI.toString());
+        args.setProperty("newLocation", newLocURI.toString());
+        args.setProperty("persistent-dismiss", true);
+
+        nc->postNotification("have-old-downloads-location", QUrl{"qrc:///qml/DownloadsInDocumentsWarning.qml"}, args);
+    }
+#endif
+}
+
+bool LauncherController::haveOldWindowsDownloadDir() const
+{
+    const SGPath p = SGPath::documents() / "FlightGear";
+    if ((p / "TerraSync").exists() || (p / "Aircraft").exists()) {
+        return true;
+    }
+
+    // tex-cache dir is created by default, so check if it's populated
+    simgear::Dir texCacheDir(p / "TextureCache");
+    return (texCacheDir.exists() && !texCacheDir.isEmpty());
+}
