@@ -31,25 +31,39 @@
 #include <QSettings>
 #include <QDebug>
 
-#include <Main/globals.hxx>
 #include <Include/version.h>
+#include <Main/globals.hxx>
+#include <Main/locale.hxx>
+#include <Main/sentryIntegration.hxx>
 
+#include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/structure/exception.hxx>
 
-static quint32 CACHE_VERSION = 12;
+static quint32 CACHE_VERSION = 13;
 
-AircraftItem::AircraftItem()
+const std::vector<QByteArray> static_localizedStringTags = {"name", "desc"};
+
+QDataStream& operator<<(QDataStream& ds, const AircraftItem::LocalizedStrings& ls)
 {
+    ds << ls.locale << ls.strings;
+    return ds;
 }
 
-AircraftItem::AircraftItem(QDir dir, QString filePath)
+QDataStream& operator>>(QDataStream& ds, AircraftItem::LocalizedStrings& ls)
+{
+    ds >> ls.locale >> ls.strings;
+    return ds;
+}
+
+bool AircraftItem::initFromFile(QDir dir, QString filePath)
 {
     SGPropertyNode root;
     readProperties(filePath.toStdString(), &root);
 
     if (!root.hasChild("sim")) {
-        throw sg_io_exception(std::string("Malformed -set.xml file"), filePath.toStdString());
+        qWarning() << "-set.xml has no <sim> element" << filePath;
+        return false;
     }
 
     SGPropertyNode_ptr sim = root.getNode("sim");
@@ -58,13 +72,12 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
     pathModTime = QFileInfo(path).lastModified();
     if (sim->getBoolValue("exclude-from-gui", false)) {
         excluded = true;
-        return;
+        return false;
     }
 
-    description = sim->getStringValue("description");
-	// trim the description to ensure the alphabetical sort
-	// doesn't get thrown off by leading whitespace
-	description = description.trimmed();
+    LocalizedStrings ls;
+    ls.locale = "en";
+    ls.strings["name"] = QString::fromStdString(sim->getStringValue("description")).trimmed();
     authors = sim->getStringValue("author");
 
     if (sim->hasChild("rating")) {
@@ -76,7 +89,7 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
 
     if (sim->hasChild("long-description")) {
         // clean up any XML whitspace in the text.
-        longDescription = QString(sim->getStringValue("long-description")).simplified();
+        ls.strings["desc"] = QString(sim->getStringValue("long-description")).simplified();
     }
 
     if (sim->hasChild("variant-of")) {
@@ -131,6 +144,67 @@ AircraftItem::AircraftItem(QDir dir, QString filePath)
     homepageUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/home-page")));
     supportUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/support")));
     wikipediaUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/wikipedia")));
+
+    _localized.push_front(ls);
+    readLocalizedStrings(sim);
+    doLocalizeStrings();
+
+    return true;
+}
+
+void AircraftItem::readLocalizedStrings(SGPropertyNode_ptr simNode)
+{
+    if (!simNode->hasChild("localized"))
+        return;
+
+    auto localeNode = simNode->getChild("localized");
+    const auto num = localeNode->nChildren();
+    for (int i = 0; i < num; i++) {
+        const SGPropertyNode* c = localeNode->getChild(i);
+
+        LocalizedStrings ls;
+        ls.locale = QString::fromStdString(c->getNameString());
+        if (c->hasChild("description")) {
+            ls.strings["name"] = QString::fromStdString(c->getStringValue("description"));
+        }
+        if (c->hasChild("long-description")) {
+            ls.strings["desc"] = QString::fromStdString(c->getStringValue("long-description")).simplified();
+        }
+
+        _localized.push_back(ls);
+    }
+}
+
+void AircraftItem::doLocalizeStrings()
+{
+    // default strings are always at the front
+    _currentStrings = _localized.front().strings;
+
+    const auto lang = QString::fromStdString(globals->get_locale()->getPreferredLanguage());
+    // find the matching locale
+    auto it = std::find_if(_localized.begin(), _localized.end(), [lang](const LocalizedStrings& ls) {
+        return ls.locale == lang;
+    });
+
+    if (it == _localized.end())
+        return; // nothing else to do
+
+    for (auto t : static_localizedStringTags) {
+        if (it->strings.contains(t)) {
+            // copy the value we found
+            _currentStrings[t] = it->strings.value(t);
+        }
+    } // of strings iteration
+}
+
+QString AircraftItem::name() const
+{
+    return _currentStrings.value("name");
+}
+
+QString AircraftItem::description() const
+{
+    return _currentStrings.value("desc");
 }
 
 QString AircraftItem::baseName() const
@@ -147,7 +221,7 @@ void AircraftItem::fromDataStream(QDataStream& ds)
         return;
     }
 
-    ds >> description >> longDescription >> authors >> variantOf >> isPrimary;
+    ds >> authors >> variantOf >> isPrimary;
     for (int i=0; i<4; ++i) ds >> ratings[i];
     ds >> previews;
     ds >> thumbnailPath;
@@ -155,6 +229,9 @@ void AircraftItem::fromDataStream(QDataStream& ds)
     ds >> needsMaintenance >> usesHeliports >> usesSeaports;
     ds >> homepageUrl >> supportUrl >> wikipediaUrl;
     ds >> tags;
+    ds >> _localized;
+
+    doLocalizeStrings();
 }
 
 void AircraftItem::toDataStream(QDataStream& ds) const
@@ -164,7 +241,7 @@ void AircraftItem::toDataStream(QDataStream& ds) const
         return;
     }
 
-    ds << description << longDescription << authors << variantOf << isPrimary;
+    ds << authors << variantOf << isPrimary;
     for (int i=0; i<4; ++i) ds << ratings[i];
     ds << previews;
     ds << thumbnailPath;
@@ -172,6 +249,7 @@ void AircraftItem::toDataStream(QDataStream& ds) const
     ds << needsMaintenance << usesHeliports << usesSeaports;
     ds << homepageUrl << supportUrl << wikipediaUrl;
     ds << tags;
+    ds << _localized;
 }
 
 int AircraftItem::indexOfVariant(QUrl uri) const
@@ -204,6 +282,84 @@ QVariant AircraftItem::status(int variant)
 
 }
 
+namespace {
+
+static std::unique_ptr<LocalAircraftCache> static_cacheInstance;
+
+
+// ensure references to Aircraft/foo and <my-aircraft-dir>/foo are resolved. This happens when
+// aircraft reference a path (probably to themselves) in their -set.xml
+class ScanDirProvider : public simgear::ResourceProvider
+{
+public:
+    ScanDirProvider() : simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {}
+
+    ~ScanDirProvider() = default;
+
+    SGPath resolve(const std::string& aResource, SGPath& aContext) const override
+    {
+        Q_UNUSED(aContext)
+
+        SGPath ap = _currentAircraftPath / aResource;
+        if (ap.exists())
+            return ap;
+
+        string_list pieces(sgPathBranchSplit(aResource));
+        if ((pieces.size() < 3) || (pieces.front() != "Aircraft")) {
+            return SGPath{}; // not an Aircraft path
+        }
+
+        const std::string res(aResource, 9); // resource path with 'Aircraft/' removed
+        SGPath p = _currentScanPath / res;
+        if (p.exists())
+            return p;
+
+        return SGPath{};
+    }
+
+    void setCurrentPath(const SGPath& p)
+    {
+        _currentScanPath = p;
+    }
+
+    void setCurrentAircraftPath(const SGPath& p)
+    {
+        _currentAircraftPath = p;
+    }
+
+private:
+    SGPath _currentScanPath;
+    SGPath _currentAircraftPath;
+};
+
+class OtherAircraftDirsProvider : public simgear::ResourceProvider
+{
+public:
+    OtherAircraftDirsProvider() : simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {}
+
+    ~OtherAircraftDirsProvider() = default;
+
+    SGPath resolve(const std::string& aResource, SGPath& aContext) const override
+    {
+        if (aResource.find("Aircraft/") != 0) {
+            return SGPath{}; // not an aircraft path
+        }
+
+        const std::string res(aResource, 9); // resource path with 'Aircraft/' removed
+
+        Q_UNUSED(aContext)
+        QStringList paths = LocalAircraftCache::instance()->paths();
+        Q_FOREACH(auto p, paths) {
+            const auto sp = SGPath::fromUtf8(p.toUtf8().toStdString()) / res;
+           // qWarning() << "OADP: trying:" << QString::fromStdString(sp.utf8Str());
+            if (sp.exists())
+                return sp;
+        }
+
+        return SGPath{};
+    }
+};
+
 class AircraftScanThread : public QThread
 {
     Q_OBJECT
@@ -212,10 +368,14 @@ public:
         m_dirs(dirsToScan),
         m_done(false)
     {
+        auto rm = simgear::ResourceManager::instance();
+        m_currentScanDir.reset(new ScanDirProvider);
+        rm->addProvider(m_currentScanDir.get());
     }
 
     ~AircraftScanThread()
     {
+        simgear::ResourceManager::instance()->removeProvider(m_currentScanDir.get());
     }
 
     /** thread-safe access to items already scanned */
@@ -238,11 +398,19 @@ Q_SIGNALS:
     void addedItems();
 
 protected:
-    virtual void run()
+    void run() override
     {
+        flightgear::addSentryBreadcrumb("AircraftScan started", "info");
         readCache();
 
+        // avoid filling up Sentry with many reports
+        // from unmaintained aircraft. We'll still fail if soemeone tries
+        // to use the aircraft, but that's 100x less common.
+        flightgear::sentryThreadReportXMLErrors(false);
+
         Q_FOREACH(QString d, m_dirs) {
+            const auto p = SGPath::fromUtf8(d.toUtf8().toStdString());
+            m_currentScanDir->setCurrentPath(p);
             scanAircraftDir(QDir(d));
             if (m_done) {
                 return;
@@ -250,6 +418,7 @@ protected:
         }
 
         writeCache();
+        flightgear::addSentryBreadcrumb("AircraftScan finished", "info");
     }
 
 private:
@@ -309,6 +478,11 @@ private:
             QMap<QString, AircraftItemPtr> baseAircraft;
             QList<AircraftItemPtr> variants;
 
+            // ensure aircraft dir is available to simgear::ResourceProvider
+            // otherwise some aircraft -set.xml includes fail
+            const auto p = SGPath::fromUtf8(child.absoluteFilePath().toUtf8().toStdString());
+            m_currentScanDir->setCurrentAircraftPath(p);
+
             Q_FOREACH(QFileInfo xmlChild, childDir.entryInfoList(filters, QDir::Files)) {
                 try {
                     QString absolutePath = xmlChild.absoluteFilePath();
@@ -320,7 +494,11 @@ private:
                         item = m_cachedItems.value(absolutePath);
                     } else {
                         // scan the cached item
-                        item = AircraftItemPtr(new AircraftItem(childDir, absolutePath));
+                        item = AircraftItemPtr(new AircraftItem);
+                        bool ok = item->initFromFile(childDir, absolutePath);
+                        if (!ok) {
+                            continue;
+                        }
                     }
 
                     m_nextCache[absolutePath] = item;
@@ -334,7 +512,9 @@ private:
                     } else {
                         variants.append(item);
                     }
-                } catch (sg_exception&) {
+                } catch (sg_exception& e) {
+                    qWarning() << "Problems occurred while parsing" << xmlChild.absoluteFilePath() << "(skipping)"
+                               << "\n\t" << QString::fromStdString(e.what());
                     continue;
                 }
 
@@ -377,9 +557,20 @@ private:
     QMap<QString, AircraftItemPtr > m_nextCache;
 
     bool m_done;
+    std::unique_ptr<ScanDirProvider> m_currentScanDir;
 };
 
-static std::unique_ptr<LocalAircraftCache> static_cacheInstance;
+} // of anonymous namespace
+
+
+class LocalAircraftCache::AircraftCachePrivate
+{
+public:
+    QStringList m_paths;
+    std::unique_ptr<AircraftScanThread> m_scanThread;
+    QVector<AircraftItemPtr> m_items;
+    std::unique_ptr<OtherAircraftDirsProvider> m_otherDirsProvider;
+};
 
 LocalAircraftCache* LocalAircraftCache::instance()
 {
@@ -390,33 +581,53 @@ LocalAircraftCache* LocalAircraftCache::instance()
     return static_cacheInstance.get();
 }
 
-LocalAircraftCache::LocalAircraftCache()
+void LocalAircraftCache::reset()
 {
+    static_cacheInstance.reset();
+}
 
+LocalAircraftCache::LocalAircraftCache() :
+    d(new AircraftCachePrivate)
+{
+    d->m_otherDirsProvider.reset(new OtherAircraftDirsProvider);
+    auto rm = simgear::ResourceManager::instance();
+    rm->addProvider(d->m_otherDirsProvider.get());
 }
 
 LocalAircraftCache::~LocalAircraftCache()
 {
     abandonCurrentScan();
+    if (simgear::ResourceManager::haveInstance()) {
+        simgear::ResourceManager::instance()->removeProvider(d->m_otherDirsProvider.get());
+    } else {
+        // resource manager will already have destroyed the provider. Awkward
+        // ownership model :(
+        d->m_otherDirsProvider.release();
+    }
 }
 
 void LocalAircraftCache::setPaths(QStringList paths)
 {
-    if (paths == m_paths) {
+    if (paths == d->m_paths) {
         return;
     }
 
-    m_items.clear();
+    d->m_items.clear();
     emit cleared();
-    m_paths = paths;
+    d->m_paths = paths;
+}
+
+QStringList LocalAircraftCache::paths() const
+{
+    return d->m_paths;
 }
 
 void LocalAircraftCache::scanDirs()
 {
     abandonCurrentScan();
-    m_items.clear();
+    d->m_items.clear();
 
-    QStringList dirs = m_paths;
+    QStringList dirs = d->m_paths;
 
     for (SGPath ap : globals->get_aircraft_paths()) {
         dirs << QString::fromStdString(ap.utf8Str());
@@ -425,40 +636,40 @@ void LocalAircraftCache::scanDirs()
     SGPath rootAircraft = globals->get_fg_root() / "Aircraft";
     dirs << QString::fromStdString(rootAircraft.utf8Str());
 
-    m_scanThread.reset(new AircraftScanThread(dirs));
-    connect(m_scanThread.get(), &AircraftScanThread::finished, this,
+    d->m_scanThread.reset(new AircraftScanThread(dirs));
+    connect(d->m_scanThread.get(), &AircraftScanThread::finished, this,
             &LocalAircraftCache::onScanFinished);
     // force a queued connection here since we the scan thread object still
     // belongs to the same thread as us, and hence this would otherwise be
     // a direct connection
-    connect(m_scanThread.get(), &AircraftScanThread::addedItems,
+    connect(d->m_scanThread.get(), &AircraftScanThread::addedItems,
             this, &LocalAircraftCache::onScanResults,
             Qt::QueuedConnection);
-    m_scanThread->start();
+    d->m_scanThread->start();
 
     emit scanStarted();
 }
 
 int LocalAircraftCache::itemCount() const
 {
-    return m_items.size();
+    return d->m_items.size();
 }
 
 QVector<AircraftItemPtr> LocalAircraftCache::allItems() const
 {
-    return m_items;
+    return d->m_items;
 }
 
 AircraftItemPtr LocalAircraftCache::itemAt(int index) const
 {
-    return m_items.at(index);
+    return d->m_items.at(index);
 }
 
 int LocalAircraftCache::findIndexWithUri(QUrl aircraftUri) const
 {
     QString path = aircraftUri.toLocalFile();
-    for (int row=0; row < m_items.size(); ++row) {
-        const AircraftItemPtr item(m_items.at(row));
+    for (int row=0; row < d->m_items.size(); ++row) {
+        const AircraftItemPtr item(d->m_items.at(row));
         if (item->path == path) {
             return row;
         }
@@ -479,8 +690,8 @@ AircraftItemPtr LocalAircraftCache::primaryItemFor(AircraftItemPtr item) const
     if (!item || item->variantOf.isEmpty())
         return item;
 
-    for (int row=0; row < m_items.size(); ++row) {
-        const AircraftItemPtr p(m_items.at(row));
+    for (int row=0; row < d->m_items.size(); ++row) {
+        const AircraftItemPtr p(d->m_items.at(row));
         if (p->baseName() == item->variantOf) {
             return p;
         }
@@ -493,7 +704,7 @@ AircraftItemPtr LocalAircraftCache::findItemWithUri(QUrl aircraftUri) const
 {
     int index = findIndexWithUri(aircraftUri);
     if (index >= 0) {
-        return m_items.at(index);
+        return d->m_items.at(index);
     }
 
     return {};
@@ -501,37 +712,37 @@ AircraftItemPtr LocalAircraftCache::findItemWithUri(QUrl aircraftUri) const
 
 void LocalAircraftCache::abandonCurrentScan()
 {
-    if (m_scanThread) {
+    if (d->m_scanThread) {
         // don't fire onScanFinished when the thread ends
-        disconnect(m_scanThread.get(), &AircraftScanThread::finished, this,
+        disconnect(d->m_scanThread.get(), &AircraftScanThread::finished, this,
                 &LocalAircraftCache::onScanFinished);
 
-        m_scanThread->setDone();
-        if (!m_scanThread->wait(2000)) {
+        d->m_scanThread->setDone();
+        if (!d->m_scanThread->wait(2000)) {
             qWarning() << Q_FUNC_INFO << "scan thread failed to terminate";
         }
-        m_scanThread.reset();
+        d->m_scanThread.reset();
     }
 }
 
 
 void LocalAircraftCache::onScanResults()
 {
-    if (!m_scanThread) {
+    if (!d->m_scanThread) {
         return;
     }
 
-    QVector<AircraftItemPtr> newItems = m_scanThread->items();
+    QVector<AircraftItemPtr> newItems = d->m_scanThread->items();
     if (newItems.isEmpty())
         return;
 
-    m_items+=newItems;
+    d->m_items+=newItems;
     emit addedItems(newItems.size());
 }
 
 void LocalAircraftCache::onScanFinished()
 {
-    m_scanThread.reset();
+    d->m_scanThread.reset();
     emit scanCompleted();
 }
 
@@ -568,4 +779,49 @@ int LocalAircraftCache::ratingFromProperties(SGPropertyNode* node, int ratingInd
     return node->getIntValue(names[ratingIndex]);
 }
 
+LocalAircraftCache::ParseSetXMLResult
+LocalAircraftCache::readAircraftProperties(const SGPath &setPath, SGPropertyNode_ptr props)
+{
+    // it woudld be race-y to touch the reosurce provider while the scan thread is running
+    // and our provider would confuse current-aircraft-dir lookups as well. Since we
+    // can't do thread-specific reosurce providers, we just bail here.
+    if (d->m_scanThread) {
+        return ParseSetXMLResult::Retry;
+    }
+
+    auto rm = simgear::ResourceManager::instance();
+
+    // ensure aircraft relative paths in the -set.xml parsing work
+
+    std::unique_ptr<ScanDirProvider> dp{new ScanDirProvider};
+    rm->addProvider(dp.get());
+
+    // we want to know the aircraft directory the aircraft lives in. This might
+    // be a manually added path (for local aircraft) or the install dir for the
+    // hangar (for packaged aircraft). Becuase -set.xml files are always found at
+    // /some/path/foobarAircraft/<aircraft-name>/some-set.xml, and the path we
+    // we want here is /some/path/foodbarAircraft, we use dirPath twice, and this
+    // works any kind of installed aircraft
+    const SGPath aircraftDirPath = setPath.dirPath().dirPath();
+    dp->setCurrentPath(aircraftDirPath);
+
+    dp->setCurrentAircraftPath(setPath);
+
+    ParseSetXMLResult result = ParseSetXMLResult::Failed;
+    flightgear::sentryThreadReportXMLErrors(false);
+
+    try {
+        readProperties(setPath, props);
+        result = ParseSetXMLResult::Ok;
+    } catch (sg_exception& e) {
+        // malformed include or XML
+        qWarning() << "Problems occurred while parsing" << QString::fromStdString(setPath.utf8Str())
+                   << "\n\t" << QString::fromStdString(e.getFormattedMessage());
+    }
+
+    rm->removeProvider(dp.get());
+    flightgear::sentryThreadReportXMLErrors(true);
+    return result;
+}
+
 #include "LocalAircraftCache.moc"
