@@ -50,7 +50,7 @@
 
 using namespace flightgear;
 
-const unsigned int MAX_RECENT_LOCATIONS = 64;
+const unsigned int MAX_RECENT_LOCATIONS = 20;
 
 QVariant savePositionList(const FGPositionedList& posList)
 {
@@ -258,6 +258,7 @@ void LocationController::setBaseLocation(QmlPositioned* pos)
         // disable offset when selecting a heliport
         if (m_airportLocation->isHeliport()) {
             m_onFinal = false;
+            m_useActiveRunway = false;
         }
     } else {
         m_airportLocation.clear();
@@ -325,20 +326,25 @@ QObjectList LocationController::airportRunways() const
         return {};
 
     QObjectList result;
-    if (m_airportLocation->isHeliport()) {
-        // helipads
-        for (unsigned int r=0; r<m_airportLocation->numHelipads(); ++r) {
-            auto p = new QmlPositioned(m_airportLocation->getHelipadByIndex(r).ptr());
-            QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
-            result.push_back(p);
-        }
-    } else {
-        // regular runways
-        for (unsigned int r=0; r<m_airportLocation->numRunways(); ++r) {
-            auto p = new QmlPositioned(m_airportLocation->getRunwayByIndex(r).ptr());
-            QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
-            result.push_back(p);
-        }
+    for (unsigned int r = 0; r < m_airportLocation->numRunways(); ++r) {
+        auto p = new QmlPositioned(m_airportLocation->getRunwayByIndex(r).ptr());
+        QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
+        result.push_back(p);
+    }
+
+    return result;
+}
+
+QObjectList LocationController::airportHelipads() const
+{
+    if (!m_airportLocation)
+        return {};
+
+    QObjectList result;
+    for (unsigned int r = 0; r < m_airportLocation->numHelipads(); ++r) {
+        auto p = new QmlPositioned(m_airportLocation->getHelipadByIndex(r).ptr());
+        QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
+        result.push_back(p);
     }
 
     return result;
@@ -375,12 +381,22 @@ void LocationController::showHistoryInSearchModel()
 	locs.erase(it, locs.end());
 
 	// prepend them
-	FGAirportRef apt = FGAirport::findByIdent(tutorialICAO);
-	locs.insert(locs.begin(), apt);
+    FGAirportRef apt = FGAirport::findByIdent(tutorialICAO);
+    if (apt) {
+        locs.insert(locs.begin(), apt);
+    } else {
+        qWarning() << Q_FUNC_INFO << "couldn't find tutorial airport for:" << QString::fromStdString(tutorialICAO);
+    }
 
-	apt = FGAirport::findByIdent(defaultICAO);
-	locs.insert(locs.begin(), apt);
+    apt = FGAirport::findByIdent(defaultICAO);
+    if (apt) {
+        locs.insert(locs.begin(), apt);
+    } else {
+        qWarning() << Q_FUNC_INFO << "couldn't find default airport for:" << QString::fromStdString(defaultICAO);
+    }
 
+    // Sentry FLIGHTGEAR-1BM indicates we are somtimes passing null pointers
+    // in this list; checks above are an attempt to diagnose this.
     m_searchModel->setItems(locs);
 }
 
@@ -564,12 +580,18 @@ void LocationController::restoreLocation(QVariantMap l)
 
             if (l.contains("location-apt-runway")) {
                 QString runway = l.value("location-apt-runway").toString().toUpper();
+                const auto runwayStr = runway.toStdString();
                 if (runway == QStringLiteral("ACTIVE")) {
                     m_useActiveRunway = true;
                 } else if (m_airportLocation->isHeliport()) {
-                    m_detailLocation = m_airportLocation->getHelipadByIdent(runway.toStdString());
+                    m_detailLocation = m_airportLocation->getHelipadByIdent(runwayStr);
                 } else {
-                    m_detailLocation = m_airportLocation->getRunwayByIdent(runway.toStdString());
+                    // could be a helipad at a regular airport
+                    if (m_airportLocation->hasHelipadWithIdent(runwayStr)) {
+                        m_detailLocation = m_airportLocation->getHelipadByIdent(runwayStr);
+                    } else {
+                        m_detailLocation = m_airportLocation->getRunwayByIdent(runwayStr);
+                    }
                 }
             } else if (l.contains("location-apt-parking")) {
                 QString parking = l.value("location-apt-parking").toString();
@@ -646,7 +668,7 @@ QVariantMap LocationController::saveLocation() const
                 locationSet.insert("location-apt-parking", "AVAILABLE");
             } else if (m_detailLocation) {
                 const auto detailType = m_detailLocation->type();
-                if (detailType == FGPositioned::RUNWAY) {
+                if ((detailType == FGPositioned::RUNWAY) || (detailType == FGPositioned::HELIPAD)) {
                     locationSet.insert("location-apt-runway", QString::fromStdString(m_detailLocation->ident()));
                 } else if (detailType == FGPositioned::PARKING) {
                     locationSet.insert("location-apt-parking", QString::fromStdString(m_detailLocation->ident()));
@@ -755,7 +777,7 @@ void LocationController::setLocationProperties()
             fgSetString("/sim/presets/parkpos", "AVAILABLE");
         } else if (onRunway) {
             if (m_airportLocation->type() == FGPositioned::AIRPORT) {
-                // explicit runway choice
+                // explicit runway choice (this also works for helipads)
                 fgSetString("/sim/presets/runway", m_detailLocation->ident() );
                 fgSetBool("/sim/presets/runway-requested", true );
 
@@ -948,8 +970,9 @@ void LocationController::onCollectConfig()
 
     if (m_airportLocation) {
         m_config->setArg("airport", QString::fromStdString(m_airportLocation->ident()));
-        const bool onRunway = (m_detailLocation && (m_detailLocation->type() == FGPositioned::RUNWAY));
-        const bool atParking = (m_detailLocation && (m_detailLocation->type() == FGPositioned::PARKING));
+        const auto ty = m_detailLocation ? m_detailLocation->type() : FGPositioned::INVALID;
+        const bool onRunway = (ty == FGPositioned::RUNWAY) || (ty == FGPositioned::HELIPAD);
+        const bool atParking = ty == FGPositioned::PARKING;
 
         if (m_useActiveRunway) {
             // pick by default
@@ -960,14 +983,17 @@ void LocationController::onCollectConfig()
             if (m_airportLocation->type() == FGPositioned::AIRPORT) {
                 m_config->setArg("runway", QString::fromStdString(m_detailLocation->ident()));
 
+                if (ty == FGPositioned::RUNWAY) {
                     // set nav-radio 1 based on selected runway
-                FGRunway* runway = static_cast<FGRunway*>(m_detailLocation.ptr());
-                if (runway->ILS()) {
-                    double mhz = runway->ILS()->get_freq() / 100.0;
-                    m_config->setArg("nav1", QString("%1:%2").arg(runway->headingDeg()).arg(mhz));
+                    auto runway = fgpositioned_cast<FGRunway>(m_detailLocation);
+                    if (runway->ILS()) {
+                        double mhz = runway->ILS()->get_freq() / 100.0;
+                        m_config->setArg("nav1", QString("%1:%2").arg(runway->headingDeg()).arg(mhz));
+                    }
+
+                    applyOnFinal();
                 }
 
-                applyOnFinal();
             } else if (m_airportLocation->type() == FGPositioned::HELIPORT) {
                 m_config->setArg("runway", QString::fromStdString(m_detailLocation->ident()));
             }
@@ -1028,24 +1054,25 @@ void LocationController::setNavRadioOption()
     }
 }
 
-QString compassPointFromHeading(int heading)
+QString LocationController::compassPointFromHeading(int heading) const
 {
     const int labelArc = 360 / 8;
     heading += (labelArc >> 1);
     SG_NORMALIZE_RANGE(heading, 0, 359);
 
+    //
     switch (heading / labelArc) {
-    case 0: return "N";
-    case 1: return "NE";
-    case 2: return "E";
-    case 3: return "SE";
-    case 4: return "S";
-    case 5: return "SW";
-    case 6: return "W";
-    case 7: return "NW";
+    case 0: return tr("N");
+    case 1: return tr("NE");
+    case 2: return tr("E");
+    case 3: return tr("SE");
+    case 4: return tr("S");
+    case 5: return tr("SW");
+    case 6: return tr("W");
+    case 7: return tr("NW");
     }
 
-    return QString();
+    return {};
 }
 
 QString LocationController::description() const
@@ -1082,8 +1109,11 @@ QString LocationController::description() const
     name = fixNavaidName(name);
 
     if (m_airportLocation) {
-        const bool onRunway = (m_detailLocation && (m_detailLocation->type() == FGPositioned::RUNWAY));
-        const bool atParking = (m_detailLocation && (m_detailLocation->type() == FGPositioned::PARKING));
+        const auto ty = m_detailLocation ? m_detailLocation->type() : FGPositioned::INVALID;
+        const bool onRunway = ty == FGPositioned::RUNWAY;
+        const bool onPad = ty == FGPositioned::HELIPAD;
+        const bool atParking = ty == FGPositioned::PARKING;
+
         QString locationOnAirport;
 
         if (m_useActiveRunway) {
@@ -1094,14 +1124,16 @@ QString LocationController::description() const
             }
         } else if (m_useAvailableParking) {
             locationOnAirport = tr("at an available parking position");
-        } if (onRunway) {
-            QString runwayName = QString("runway %1").arg(QString::fromStdString(m_detailLocation->ident()));
+        } else if (onRunway) {
+            QString runwayName = tr("runway %1").arg(QString::fromStdString(m_detailLocation->ident()));
 
             if (m_onFinal) {
                 locationOnAirport = tr("on %2-mile final to %1").arg(runwayName).arg(offsetNm);
             } else {
                 locationOnAirport = tr("on %1").arg(runwayName);
             }
+        } else if (onPad) {
+            locationOnAirport = tr("on pad %1").arg(QString::fromStdString(m_detailLocation->ident()));
         } else if (atParking) {
             locationOnAirport = tr("at parking position %1").arg(QString::fromStdString(m_detailLocation->ident()));
         }
@@ -1137,6 +1169,12 @@ QString LocationController::description() const
 
 void LocationController::addToRecent(FGPositionedRef pos)
 {
+    if (!pos) {
+        // ensure null positiones don't get added, could cause
+        // Sentry FLIGHTGEAR-1BM
+        return;
+    }
+
     auto it = std::find(m_recentLocations.begin(),
                         m_recentLocations.end(), pos);
     if (it != m_recentLocations.end()) {
