@@ -4,6 +4,7 @@
 #include <QDebug>
 #include <QQmlEngine>
 #include <QQmlComponent>
+#include <QTimer>
 
 #include <simgear/package/Install.hxx>
 #include <simgear/package/Root.hxx>
@@ -78,16 +79,6 @@ private:
     QmlAircraftInfo* p;
 };
 
-void QmlAircraftInfo::Delegate::finishInstall(InstallRef aInstall, StatusCode aReason)
-{
-    Q_UNUSED(aReason)
-    if (aInstall->package() == p->packageRef()) {
-        p->_cachedProps.reset();
-        p->checkForStates();
-        p->infoChanged();
-    }
-}
-
 ////////////////////////////////////////////////////////////////////////////
 
 
@@ -103,22 +94,14 @@ struct StateInfo
 
 using AircraftStateVec = std::vector<StateInfo>;
 
-static AircraftStateVec readAircraftStates(const SGPath& setXMLPath)
+static bool readAircraftStates(const SGPropertyNode_ptr root, AircraftStateVec& result)
 {
-    SGPropertyNode_ptr root(new SGPropertyNode);
-    try {
-        readProperties(setXMLPath, root);
-    } catch (sg_exception&) {
-        // malformed include or XML, just bail
-        return {};
-    }
-
+    result.clear();
     if (!root->getNode("sim/state")) {
-        return {};
+        return false; // no states
     }
 
     auto nodes = root->getNode("sim")->getChildren("state");
-    AircraftStateVec result;
     result.reserve(nodes.size());
     for (auto cn : nodes) {
         string_list stateNames;
@@ -137,7 +120,7 @@ static AircraftStateVec readAircraftStates(const SGPath& setXMLPath)
                          });
     }
 
-    return result;
+    return true;
 }
 
 QString humanNameFromStateTag(const std::string& tag)
@@ -168,15 +151,27 @@ class StatesModel : public QAbstractListModel
     Q_OBJECT
 
 public:
-    StatesModel()
+    StatesModel(QObject* pr) : QAbstractListModel(pr)
     {
     }
 
-    StatesModel(const AircraftStateVec& states) :
-        _data(states)
+    void clear()
     {
+        beginResetModel();
+        _data.clear();
+        _explicitAutoState = false;
+        endResetModel();
+    }
+
+    void initWithStates(const AircraftStateVec& states)
+    {
+        beginResetModel();
+        _data = states;
+        _explicitAutoState = false;
+
         // we use an empty model for aircraft with no states defined
         if (states.empty()) {
+            endResetModel();
             return;
         }
 
@@ -194,6 +189,8 @@ public:
         } else {
             _data.insert(_data.begin(), {{"auto"}, {}, tr("Select state based on startup position.")});
         }
+
+        endResetModel();
     }
 
     Q_INVOKABLE int indexForTag(QString s) const
@@ -299,6 +296,22 @@ private:
     bool _explicitAutoState = false;
 };
 
+////////////////////////////////////////////////////////////////////////////
+
+void QmlAircraftInfo::Delegate::finishInstall(InstallRef aInstall, StatusCode aReason)
+{
+    Q_UNUSED(aReason)
+    if (aInstall->package() == p->packageRef()) {
+        p->_cachedProps.reset();
+        if (p->_statesModel) {
+            p->_statesModel->clear();
+        }
+        p->_statesChecked = false;
+        p->infoChanged();
+    }
+}
+
+
 ////////////////////////////////////////////////////////////////////////////
 
 QmlAircraftInfo::QmlAircraftInfo(QObject *parent)
@@ -342,7 +355,7 @@ quint32 QmlAircraftInfo::numVariants() const
 QString QmlAircraftInfo::name() const
 {
     if (_item) {
-        return resolveItem()->description;
+        return resolveItem()->name();
     } else if (_package) {
         return QString::fromStdString(_package->nameForVariant(_variant));
     }
@@ -353,7 +366,7 @@ QString QmlAircraftInfo::name() const
 QString QmlAircraftInfo::description() const
 {
     if (_item) {
-        return resolveItem()->longDescription;
+        return resolveItem()->description();
     } else if (_package) {
         std::string longDesc = _package->getLocalisedProp("description", _variant);
         return QString::fromStdString(longDesc).simplified();
@@ -591,46 +604,49 @@ AircraftItemPtr QmlAircraftInfo::resolveItem() const
     return _item;
 }
 
-void QmlAircraftInfo::checkForStates()
+void QmlAircraftInfo::validateStates() const
 {
-    QString path = pathOnDisk();
-    if (path.isEmpty()) {
-        _statesModel.reset(new StatesModel);
-        emit infoChanged();
-        return;
+    // after calling this, StatesModel must exist, but can be empty
+    if (!_statesModel) {
+        _statesModel = new StatesModel(const_cast<QmlAircraftInfo*>(this));
     }
 
-    auto states = readAircraftStates(SGPath::fromUtf8(path.toUtf8().toStdString()));
-    if (states.empty()) {
-        _statesModel.reset(new StatesModel);
-        emit infoChanged();
+    if (_statesChecked)
         return;
-    }
 
-    _statesModel.reset(new StatesModel(states));
-    emit infoChanged();
+    validateLocalProps();
+    if (!_cachedProps)
+        return;
+
+    AircraftStateVec statesData;
+    readAircraftStates(_cachedProps, statesData);
+    _statesModel->initWithStates(statesData);
+    _statesChecked = true;
 }
 
 void QmlAircraftInfo::validateLocalProps() const
 {
     if (!_cachedProps) {
-        SGPath path;
-        if (_item) {
-            path = resolveItem()->path.toUtf8().toStdString();
-        } else if (_package) {
-            auto install = _package->existingInstall();
-            if (!install.valid())
-                return;
+        // clear any previous states if we're reusing this object
+        if (_statesModel)
+            _statesModel->clear();
 
-            path = install->path();
-        }
+        _statesChecked = false;
 
+        SGPath path = SGPath::fromUtf8(pathOnDisk().toStdString());
         if (!path.exists())
             return;
         _cachedProps = new SGPropertyNode;
-        try {
-            readProperties(path, _cachedProps);
-        } catch (sg_exception&) {
+        auto r = LocalAircraftCache::instance()->readAircraftProperties(path, _cachedProps);
+        if (r == LocalAircraftCache::ParseSetXMLResult::Retry) {
+            _cachedProps.reset();
+
+            // give the AircraftScsn threa d abit more time
+            QTimer::singleShot(2000, this, &QmlAircraftInfo::retryValidateLocalProps);
+        } else if (r == LocalAircraftCache::ParseSetXMLResult::Ok) {
+            // we're good
+        } else {
+            // failure
             _cachedProps.reset();
         }
     }
@@ -643,8 +659,10 @@ void QmlAircraftInfo::setUri(QUrl u)
 
     _item.clear();
     _package.clear();
-    _statesModel.reset(new StatesModel);
     _cachedProps.clear();
+    _statesChecked = false;
+    if (_statesModel)
+        _statesModel->clear();
 
     if (u.isLocalFile()) {
         _item = LocalAircraftCache::instance()->findItemWithUri(u);
@@ -671,8 +689,6 @@ void QmlAircraftInfo::setUri(QUrl u)
         }
     }
 
-    checkForStates();
-
     emit uriChanged();
     emit infoChanged();
     emit downloadChanged();
@@ -694,8 +710,9 @@ void QmlAircraftInfo::setVariant(quint32 variant)
 
     _variant = variant;
     _cachedProps.clear();
-
-    checkForStates();
+    _statesChecked = false;
+    if (_statesModel)
+        _statesModel->clear();
 
     emit infoChanged();
     emit variantChanged(_variant);
@@ -714,6 +731,12 @@ void QmlAircraftInfo::onFavouriteChanged(QUrl u)
     emit favouriteChanged();
 }
 
+void QmlAircraftInfo::retryValidateLocalProps()
+{
+    qInfo() << Q_FUNC_INFO << "Retrying validation of local props for:" << uri();
+    validateLocalProps();
+}
+
 QVariant QmlAircraftInfo::packageAircraftStatus(simgear::pkg::PackageRef p)
 {
     if (p->hasTag("needs-maintenance")) {
@@ -773,12 +796,12 @@ QStringList QmlAircraftInfo::variantNames() const
 {
     QStringList result;
     if (_item) {
-        result.append(_item->description);
+        result.append(_item->name());
         Q_FOREACH(auto v, _item->variants) {
-            if (v->description.isEmpty()) {
+            if (v->name().isEmpty()) {
                 qWarning() << Q_FUNC_INFO << "missing description for " << v->path;
             }
-            result.append(v->description);
+            result.append(v->name());
         }
     } else if (_package) {
         for (quint32 vindex = 0; vindex < _package->variants().size(); ++vindex) {
@@ -798,22 +821,26 @@ bool QmlAircraftInfo::isPackaged() const
 
 bool QmlAircraftInfo::hasStates() const
 {
+    validateStates();
     return !_statesModel->isEmpty();
 }
 
 bool QmlAircraftInfo::hasState(QString name) const
 {
+    validateStates();
     return _statesModel->hasState(name);
 }
 
 bool QmlAircraftInfo::haveExplicitAutoState() const
 {
+    validateStates();
     return _statesModel->hasExplicitAuto();
 }
 
 StatesModel *QmlAircraftInfo::statesModel()
 {
-    return _statesModel.data();
+    validateStates();
+    return _statesModel;
 }
 
 QuantityValue QmlAircraftInfo::cruiseSpeed() const
@@ -878,11 +905,13 @@ QString QmlAircraftInfo::icaoType() const
 
 bool QmlAircraftInfo::isSpeedBelowLimits(QuantityValue speed) const
 {
+    Q_UNUSED(speed)
     return true;
 }
 
 bool QmlAircraftInfo::isAltitudeBelowLimits(QuantityValue speed) const
 {
+    Q_UNUSED(speed)
     return true;
 }
 
