@@ -25,26 +25,23 @@
 #include <locale.h>
 
 // Qt
-#include <QtGlobal>
-#include <QString>
-#include <QProgressDialog>
+#include <QApplication>
+#include <QDebug>
 #include <QDir>
 #include <QFileInfo>
-#include <QPixmap>
-#include <QTimer>
-#include <QDebug>
-#include <QCompleter>
-#include <QListView>
-#include <QSettings>
-#include <QUrl>
-#include <QAction>
-#include <QDateTime>
-#include <QApplication>
-#include <QSpinBox>
-#include <QDoubleSpinBox>
-#include <QThread>
+#include <QMessageBox>
+#include <QOffscreenSurface>
+#include <QOpenGLContext>
+#include <QPointer>
 #include <QProcess>
+#include <QProgressDialog>
+#include <QSettings>
+#include <QString>
+#include <QThread>
+#include <QTimer>
 #include <QTranslator>
+#include <QUrl>
+#include <QtGlobal>
 
 // Simgear
 #include <simgear/timing/timestamp.hxx>
@@ -52,30 +49,39 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/strutils.hxx>
 #include <simgear/package/Root.hxx>
 #include <simgear/package/Catalog.hxx>
 #include <simgear/package/Package.hxx>
 #include <simgear/package/Install.hxx>
 #include <simgear/debug/logstream.hxx>
 
-#include <Main/globals.hxx>
-#include <Main/fg_props.hxx>
-#include <Navaids/NavDataCache.hxx>
-#include <Navaids/navrecord.hxx>
-#include <Navaids/SHPParser.hxx>
-#include <Airports/airport.hxx>
 #include <Add-ons/AddonManager.hxx>
+#include <Airports/airport.hxx>
+#include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Navaids/NavDataCache.hxx>
+#include <Navaids/SHPParser.hxx>
+#include <Navaids/navrecord.hxx>
 
 
-#include <Main/options.hxx>
 #include <Main/fg_init.hxx>
-#include <Viewer/WindowBuilder.hxx>
+#include <Main/locale.hxx>
+#include <Main/options.hxx>
 #include <Network/HTTPClient.hxx>
+#include <Viewer/WindowBuilder.hxx>
 
-#include "LauncherMainWindow.hxx"
 #include "LaunchConfig.hxx"
-#include "UnitsModel.hxx"
+#include "LauncherMainWindow.hxx"
+#include "LocalAircraftCache.hxx"
 #include "PathListModel.hxx"
+#include "UnitsModel.hxx"
+#include "GettingStartedTip.hxx"
+
+#if defined(SG_MAC)
+#include <GUI/CocoaHelpers.h>
+#endif
 
 using namespace flightgear;
 using namespace simgear::pkg;
@@ -101,24 +107,54 @@ void initNavCache()
     const char* baseLabelKey = QT_TRANSLATE_NOOP("initNavCache", "Initialising navigation data, this may take several minutes");
     QString baseLabel= qApp->translate("initNavCache", baseLabelKey);
 
+    const auto wflags = Qt::Dialog | Qt::CustomizeWindowHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::MSWindowsFixedSizeDialogHint;
+
+    if (NavDataCache::isAnotherProcessRebuilding()) {
+        const char* waitForOtherMsg = QT_TRANSLATE_NOOP("initNavCache", "Another copy of FlightGear is creating the navigation database. Waiting for it to finish.");
+        QString m = qApp->translate("initNavCache", waitForOtherMsg);
+
+        addSentryBreadcrumb("Launcher: showing wait for other process NavCache rebuild dialog", "info");
+        QProgressDialog waitForRebuild(m,
+                                       QString() /* cancel text */,
+                                       0, 0, Q_NULLPTR,
+                                       wflags);
+        waitForRebuild.setWindowModality(Qt::WindowModal);
+        waitForRebuild.setMinimumWidth(600);
+        waitForRebuild.setAutoReset(false);
+        waitForRebuild.setAutoClose(false);
+        waitForRebuild.show();
+
+        QTimer updateTimer;
+        updateTimer.setInterval(500);
+
+        QObject::connect(&updateTimer, &QTimer::timeout, [&waitForRebuild]() {
+            if (!NavDataCache::isAnotherProcessRebuilding()) {
+                waitForRebuild.done(0);
+                return;
+            }
+        });
+
+        updateTimer.start(); // timer won't actually run until we process events
+        waitForRebuild.exec();
+        updateTimer.stop();
+        addSentryBreadcrumb("Launcher: done waiting for other process NavCache rebuild dialog", "info");
+    }
+
     NavDataCache* cache = NavDataCache::createInstance();
     if (cache->isRebuildRequired()) {
         QProgressDialog rebuildProgress(baseLabel,
-                                       QString() /* cancel text */,
-                                       0, 100, Q_NULLPTR,
-                                       Qt::Dialog
-                                           | Qt::CustomizeWindowHint
-                                           | Qt::WindowTitleHint
-                                           | Qt::WindowSystemMenuHint
-                                           | Qt::MSWindowsFixedSizeDialogHint);
+                                        QString() /* cancel text */,
+                                        0, 100, Q_NULLPTR,
+                                        wflags);
         rebuildProgress.setWindowModality(Qt::WindowModal);
         rebuildProgress.setMinimumWidth(600);
         rebuildProgress.setAutoReset(false);
         rebuildProgress.setAutoClose(false);
         rebuildProgress.show();
-    
+
         QTimer updateTimer;
         updateTimer.setInterval(100);
+
         QObject::connect(&updateTimer, &QTimer::timeout, [&cache, &rebuildProgress, &baseLabel]() {
             auto phase = cache->rebuild();
             if (phase == NavDataCache::REBUILD_DONE) {
@@ -147,6 +183,7 @@ void initNavCache()
         updateTimer.start(); // timer won't actually run until we process events
         rebuildProgress.exec();
         updateTimer.stop();
+        flightgear::addSentryBreadcrumb("Launcher nav-cache rebuild complete", "info");
     }
 }
 
@@ -161,6 +198,10 @@ public:
         connect(this, &QThread::finished, this, &NaturalEarthDataLoaderThread::onFinished);
     }
 
+    void abandon()
+    {
+        m_abandoned = true;
+    }
 protected:
     void run() override
     {
@@ -173,6 +214,9 @@ protected:
 private:
     Q_SLOT void onFinished()
     {
+        if (m_abandoned)
+            return;
+
         flightgear::PolyLineList::const_iterator begin = m_parsedLines.begin() + m_lineInsertCount;
         unsigned int numToAdd = std::min<unsigned int>(1000U, static_cast<unsigned int>(m_parsedLines.size()) - m_lineInsertCount);
         flightgear::PolyLineList::const_iterator end = begin + numToAdd;
@@ -202,8 +246,38 @@ private:
 
     flightgear::PolyLineList m_parsedLines;
     unsigned int m_lineInsertCount;
+    bool m_abandoned = false;
 };
 
+bool checkForWorkingOpenGL()
+{
+    QSurfaceFormat fmt;
+    fmt.setProfile(QSurfaceFormat::CompatibilityProfile);
+    fmt.setMajorVersion(2);
+    fmt.setMinorVersion(1);
+
+    QOpenGLContext ctx;
+    ctx.setFormat(fmt);
+    if (!ctx.create()) {
+        return false;
+    }
+
+    QOffscreenSurface offSurface;
+    offSurface.setFormat(ctx.format()); // ensure it's compatible
+    offSurface.create();
+    if (!ctx.makeCurrent(&offSurface)) {
+        return false;
+    }
+
+    std::string renderer = (char*)glGetString(GL_RENDERER);
+    if (renderer == "GDI Generic") {
+        flightgear::addSentryBreadcrumb("Detected GDI generic renderer", "info");
+        return false;
+    }
+
+    return true;
+}
+
 } // of anonymous namespace
 
 static void initQtResources()
@@ -229,7 +303,9 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
 
     static const char* nullFile = "";
     const char* file = context.file ? context.file : nullFile;
-    sglog().log(SG_GUI, mappedPriority, file, context.line, msg.toStdString());
+    const auto s = msg.toStdString();
+    // important we copy the file name here, since QMessageLogContext doesn't
+    sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, s);
     if (type == QtFatalMsg) {
         abort();
     }
@@ -256,7 +332,11 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
     if (!qtInitDone) {
         qtInitDone = true;
 
-        sglog().setLogLevels( SG_ALL, SG_INFO );
+        // Disable Qt 5.15 warnings about obsolete Connections/onFoo: syntax
+        // we cannot use the new syntax
+        // as long as we have to support Qt 5.9
+        qputenv("QT_LOGGING_RULES", "qt.qml.connections.warning=false");
+        
         initQtResources(); // can't be called from a namespace
 
         s_argc = argc; // QApplication only stores a reference to argc,
@@ -302,7 +382,10 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
         QTranslator* translator = new QTranslator(static_qApp.get());
         // check for --langauge=xx option and prefer that over QLocale
         // detection of the locale if it exists
-        auto lang = Options::getArgValue(argc, argv, "--language");
+        auto lang = simgear::strutils::replace(
+            Options::getArgValue(argc, argv, "--language"),
+            "-",
+            "_");
         if (!lang.empty()) {
             QString localeFile = "FlightGear_" + QString::fromStdString(lang);
             if (translator->load(localeFile, QLatin1String(":/"))) {
@@ -321,10 +404,33 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
             delete translator;
         }
 
+        // temporary code: only enable getting-started tips for English, to give translators time to
+        // catch up
+        if (!lang.empty()) {
+            GettingStartedTip::setGlobalTipsEnabled(simgear::strutils::starts_with(lang, "en"));
+        } else {
+            QLocale currentLocale;
+            GettingStartedTip::setGlobalTipsEnabled(currentLocale.name().startsWith("en"));
+        }
+
         // reset numeric / collation locales as described at:
         // http://doc.qt.io/qt-5/qcoreapplication.html#details
         ::setlocale(LC_NUMERIC, "C");
         ::setlocale(LC_COLLATE, "C");
+
+
+#if defined(SG_MAC)
+        if (cocoaIsRunningTranslocated()) {
+            addSentryBreadcrumb("did show translocation warning", "info");
+            const char* titleKey = QT_TRANSLATE_NOOP("macTranslationWarning", "Application running from download location");
+            const char* key = QT_TRANSLATE_NOOP("macTranslationWarning", "FlightGear is running from the download image. For better performance and to avoid potential problems, "
+                                                "please copy FlightGear to some other location, such as your desktop or Applications folder.");
+            QString msg = qApp->translate("macTranslationWarning", key);
+            QString title = qApp->translate("macTranslationWarning", titleKey);
+
+            QMessageBox::warning(nullptr, title, msg);
+        }
+#endif
     }
 
     if (doInitQSettings) {
@@ -349,6 +455,7 @@ void initQSettings()
         qRegisterMetaType<QuantityValue>();
         qRegisterMetaTypeStreamOperators<QuantityValue>("QuantityValue");
 
+
         qSettingsInitDone = true;
         string fgHome = globals->get_fg_home().utf8Str();
 
@@ -439,6 +546,13 @@ void launcherSetSceneryPaths()
 
 bool runLauncherDialog()
 {
+    if (!checkForWorkingOpenGL()) {
+        QMessageBox::critical(nullptr, "Failed to find graphics drivers",
+                              "This computer is missing suitable graphics drivers (OpenGL) to run FlightGear. "
+                              "Please download and install drivers from your graphics card vendor.");
+        return false;
+    }
+
     // Used for NavDataCache initialization: needed to find the apt.dat files
     launcherSetSceneryPaths();
     // startup the nav-cache now. This pre-empts normal startup of
@@ -463,13 +577,16 @@ bool runLauncherDialog()
 
     // setup package language
     auto lang = options->valueForOption("language");
-    if (!lang.empty()) {
-        globals->packageRoot()->setLocale(lang);
-    } else {
+    if (lang.empty()) {
         const auto langName = QLocale::languageToString(QLocale{}.language());
-        globals->packageRoot()->setLocale(langName.toStdString());
+        lang = langName.toStdString();
     }
 
+    // we will re-do this later, but we want to access translated strings
+    // from within the launcher
+    globals->get_locale()->selectLanguage(lang);
+    globals->packageRoot()->setLocale(globals->get_locale()->getPreferredLanguage());
+
     // startup the HTTP system now since packages needs it
     FGHTTPClient* http = globals->add_new_subsystem<FGHTTPClient>();
 
@@ -477,14 +594,14 @@ bool runLauncherDialog()
     // will happen as normal
     http->init();
 
-    NaturalEarthDataLoaderThread* naturalEarthLoader = new NaturalEarthDataLoaderThread;
+    QPointer<NaturalEarthDataLoaderThread> naturalEarthLoader = new NaturalEarthDataLoaderThread;
     naturalEarthLoader->start();
 
     // avoid double Apple menu and other weirdness if both Qt and OSG
     // try to initialise various Cocoa structures.
     flightgear::WindowBuilder::setPoseAsStandaloneApp(false);
 
-    LauncherMainWindow dlg;
+    LauncherMainWindow dlg(false);
     if (options->isOptionSet("enable-fullscreen")) {
         dlg.showFullScreen();
     } else {
@@ -496,15 +613,26 @@ bool runLauncherDialog()
         return false; // quit
     }
 
+    // avoid crashes / NavCache races if the loader is still running after
+    // the launcher exits
+    if (naturalEarthLoader) {
+        naturalEarthLoader->abandon();
+    }
+
+    // avoid a race-y crash on the locale, if a scan thread is
+    // still running: this reset will cancel any running scan
+    LocalAircraftCache::reset();
+
     // don't set scenery paths twice
     globals->clear_fg_scenery();
+    globals->get_locale()->clear();
 
     return true;
 }
 
 bool runInAppLauncherDialog()
 {
-    LauncherMainWindow dlg;
+    LauncherMainWindow dlg(true);
     bool accepted = dlg.execInApp();
     if (!accepted) {
         return false;
@@ -513,6 +641,45 @@ bool runInAppLauncherDialog()
     return true;
 }
 
+static const char* static_lockFileDialog_Title =
+    QT_TRANSLATE_NOOP("LockFileDialog", "Multiple copies of FlightGear running");
+static const char* static_lockFileDialog_Text =
+    QT_TRANSLATE_NOOP("LockFileDialog",
+                      "FlightGear has detected another copy is already running. "
+                      "This copy will run in read-only mode, so downloads will not be possible, "
+                      "and settings will not be saved.");
+static const char* static_lockFileDialog_Info =
+    QT_TRANSLATE_NOOP("LockFileDialog",
+                      "If you are sure another copy is not running on this computer, "
+                      "you can choose to reset the lock file, and run this copy as normal. "
+                       "Alternatively, you can close this copy of the software.");
+
+LockFileDialogResult showLockFileDialog()
+{
+    flightgear::addSentryBreadcrumb("showing lock-file dialog", "info");
+
+    QString title = qApp->translate("LockFileDialog", static_lockFileDialog_Title);
+    QString text = qApp->translate("LockFileDialog", static_lockFileDialog_Text);
+    QString infoText = qApp->translate("LockFileDialog", static_lockFileDialog_Info);
+
+    QMessageBox mb;
+    mb.setIconPixmap(QPixmap(":/app-icon-large"));
+    mb.setWindowTitle(title);
+    mb.setText(text);
+    mb.setInformativeText(infoText);
+    mb.addButton(QMessageBox::Ok);
+    mb.setDefaultButton(QMessageBox::Ok);
+    mb.addButton(QMessageBox::Reset);
+    mb.addButton(QMessageBox::Close);
+
+    int r = mb.exec();
+    if (r == QMessageBox::Reset)
+        return LockFileReset;
+    if (r == QMessageBox::Close)
+        return LockFileQuit;
+    return LockFileContinue;
+}
+
 } // of namespace flightgear
 
 #include "QtLauncher.moc"
