@@ -272,7 +272,11 @@ public:
         FGMouseCursor::Cursor cur = FGMouseCursor::CURSOR_CLOSED_HAND;
 
         osg::Vec2d windowPos;
-        flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+        const bool ok = flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+        if (!ok) {
+            SG_LOG(SG_GUI, SG_WARN, "doMouseMoveWithCallbacks: ignoring mouse move with missing context/traits");
+            return;
+        }
 
         SGSceneryPicks pickList = globals->get_renderer()->pick(windowPos);
         if(pickList.empty())
@@ -551,14 +555,26 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
     return;
   }
 
+
   // Pass on to PUI and the panel if
   // requested, and return if one of
   // them consumes the event.
 
   osg::Vec2d windowPos;
-  flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+  bool ok = flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+  if (!ok) {
+      SG_LOG(SG_GUI, SG_WARN, "Ignoring mouse click with null context/traits");
+      return;
+  }
 
-  SGSceneryPicks pickList = globals->get_renderer()->pick(windowPos);
+  SGSceneryPicks pickList;
+
+  if (isRightDragLookActive() && (updown == MOUSE_BUTTON_DOWN)) {
+      // when spring-loaded mode is active, don't do scene selection for picks
+      // https://sourceforge.net/p/flightgear/codetickets/2108/
+  } else {
+      pickList = globals->get_renderer()->pick(windowPos);
+  }
 
   if( updown == MOUSE_BUTTON_UP )
   {
@@ -575,6 +591,12 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
 
       callbacks.pop_front();
     }
+
+    if (ea->getHandled()) {
+        // for https://sourceforge.net/p/flightgear/codetickets/2347/
+        // we cleared the active picks, but don't do further processing
+        return;
+    }
   }
 
   if (mode.pass_through) {
@@ -613,12 +635,10 @@ void FGMouseInput::processMotion(int x, int y, const osgGA::GUIEventAdapter* ea)
 
   mouse &m = d->mice[0];
   int modeIndex = m.current_mode;
-  // are we in spring-loaded look mode?
-  if (!d->rightClickModeCycle && m.nModes > 3) {
-    if (m.mouse_button_nodes[2]->getBoolValue()) {
+
+  if (isRightDragLookActive()) {
       // right mouse is down, force look mode
       modeIndex = 3;
-    }
   }
 
   if (modeIndex == 0) {
@@ -719,6 +739,20 @@ bool FGMouseInput::isActiveModePassThrough() const
     return m.modes[mode].pass_through;
 }
 
+bool FGMouseInput::isRightDragLookActive() const
+{
+    if (!d) {
+        return false;
+    }
+
+    const auto& m = d->mice[0];
+    if (!d->rightClickModeCycle && m.nModes > 3) {
+        return m.mouse_button_nodes[2]->getBoolValue();
+    }
+
+    return false;
+}
+
 
 // Register the subsystem.
 SGSubsystemMgr::Registrant<FGMouseInput> registrantFGMouseInput;
