@@ -35,7 +35,7 @@
 #include <simgear/props/props_io.hxx>
 #include <osg/GLU>
 
-#include <plib/fnt.h>
+#include "fnt.h"
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
@@ -649,6 +649,10 @@ void HUDText::draw()
 }
 
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<HUD> registrantHUD;
+
+
 void TextList::align(const char *s, int align, float *x, float *y,
         float *l, float *r, float *b, float *t) const
 {
