@@ -38,11 +38,9 @@
 #include <Main/fg_props.hxx>
 #include <Navaids/navlist.hxx>
 
-#if defined(ENABLE_FLITE)
 #include <Sound/soundmanager.hxx>
 #include <simgear/sound/sample_group.hxx>
 #include <Sound/VoiceSynthesizer.hxx>
-#endif
 
 #include "frequencyformatter.hxx"
 
@@ -51,7 +49,6 @@ namespace Instrumentation {
 using simgear::PropertyObject;
 using std::string;
 
-#if defined(ENABLE_FLITE)
 class AtisSpeaker: public SGPropertyChangeListener, SoundSampleReadyListener {
 public:
   AtisSpeaker();
@@ -136,7 +133,7 @@ void AtisSpeaker::valueChanged(SGPropertyNode * node)
   FGSoundManager * smgr = globals->get_subsystem<FGSoundManager>();
   assert(smgr != NULL);
 
-  SG_LOG(SG_INSTR,SG_INFO,"AtisSpeaker voice is " << voice );
+  SG_LOG(SG_INSTR, SG_DEBUG,"AtisSpeaker voice is " << voice );
   FLITEVoiceSynthesizer * synthesizer = dynamic_cast<FLITEVoiceSynthesizer*>(smgr->getSynthesizer(voice));
 
   synthesizer->synthesize(_synthesizeRequest);
@@ -147,7 +144,6 @@ void AtisSpeaker::SoundSampleReady(SGSharedPtr<SGSoundSample> sample)
   // we are now in the synthesizers worker thread!
   _spokenAtis.push(sample);
 }
-#endif
 
 SignalQualityComputer::~SignalQualityComputer()
 {
@@ -201,17 +197,22 @@ private:
 
 class OutputProperties: public OnExitHandler {
 public:
-  OutputProperties(SGPropertyNode_ptr rootNode)
-      : _rootNode(rootNode), _signalQuality_norm(0.0), _slantDistance_m(0.0), _trueBearingTo_deg(0.0), _trueBearingFrom_deg(0.0), _trackDistance_m(
-          0.0), _heightAboveStation_ft(0.0),
 
-      _PO_stationType(rootNode->getNode("station-type", true)), _PO_stationName(rootNode->getNode("station-name", true)), _PO_airportId(
-          rootNode->getNode("airport-id", true)), _PO_signalQuality_norm(rootNode->getNode("signal-quality-norm", true)), _PO_slantDistance_m(
-          rootNode->getNode("slant-distance-m", true)), _PO_trueBearingTo_deg(rootNode->getNode("true-bearing-to-deg", true)), _PO_trueBearingFrom_deg(
-          rootNode->getNode("true-bearing-from-deg", true)), _PO_trackDistance_m(rootNode->getNode("track-distance-m", true)), _PO_heightAboveStation_ft(
-          rootNode->getNode("height-above-station-ft", true))
+  void bind(SGPropertyNode* rn)
   {
+      _rootNode = rn;
+
+      _PO_stationType = PropertyObject<string>(_rootNode->getNode("station-type", true));
+      _PO_stationName = PropertyObject<string>(_rootNode->getNode("station-name", true));
+      _PO_airportId = PropertyObject<string>(_rootNode->getNode("airport-id", true));
+      _PO_signalQuality_norm = PropertyObject<double>(_rootNode->getNode("signal-quality-norm", true));
+      _PO_slantDistance_m = PropertyObject<double>(_rootNode->getNode("slant-distance-m", true));
+      _PO_trueBearingTo_deg = PropertyObject<double>(_rootNode->getNode("true-bearing-to-deg", true));
+      _PO_trueBearingFrom_deg = PropertyObject<double>(_rootNode->getNode("true-bearing-from-deg", true));
+      _PO_trackDistance_m = PropertyObject<double>(_rootNode->getNode("track-distance-m", true));
+      _PO_heightAboveStation_ft = PropertyObject<double>(_rootNode->getNode("height-above-station-ft", true));
   }
+
   virtual ~OutputProperties()
   {
   }
@@ -222,12 +223,12 @@ protected:
   std::string _stationType;
   std::string _stationName;
   std::string _airportId;
-  double _signalQuality_norm;
-  double _slantDistance_m;
-  double _trueBearingTo_deg;
-  double _trueBearingFrom_deg;
-  double _trackDistance_m;
-  double _heightAboveStation_ft;
+  double _signalQuality_norm = 0.0;
+  double _slantDistance_m = 0.0;
+  double _trueBearingTo_deg = 0.0;
+  double _trueBearingFrom_deg = 0.0;
+  double _trackDistance_m = 0.0;
+  double _heightAboveStation_ft = 0.0;
 
 private:
   PropertyObject<string> _PO_stationType;
@@ -258,8 +259,6 @@ private:
 
 class MetarBridge: public SGReferenced, public SGPropertyChangeListener {
 public:
-  MetarBridge();
-  ~MetarBridge();
 
   void bind();
   void unbind();
@@ -281,20 +280,12 @@ private:
   std::string _requestedId;
   SGPropertyNode_ptr _realWxEnabledNode;
   SGPropertyNode_ptr _metarPropertiesNode;
-  SGPropertyNode * _atisNode;
+    SGPropertyNode * _atisNode = nullptr;
   ATISEncoder _atisEncoder;
 };
+
 typedef SGSharedPtr<MetarBridge> MetarBridgeRef;
 
-MetarBridge::MetarBridge()
-    : _atisNode(0)
-{
-}
-
-MetarBridge::~MetarBridge()
-{
-}
-
 void MetarBridge::bind()
 {
   _realWxEnabledNode = fgGetNode("/environment/realwx/enabled", true);
@@ -318,7 +309,7 @@ void MetarBridge::requestMetarForId(std::string & id)
     _metarPropertiesNode->getNode("valid", true)->setBoolValue(false);
     _metarPropertiesNode->getNode("time-to-live", true)->setDoubleValue(0.0);
   } else {
-    // use the present weather to generate the ATIS. 
+    // use the present weather to generate the ATIS.
     if ( NULL != _atisNode && false == _requestedId.empty()) {
       CurrentWeatherATISInformationProvider provider(_requestedId);
       _atisNode->setStringValue(_atisEncoder.encodeATIS(&provider));
@@ -463,98 +454,53 @@ private:
 
 /* ------------- The CommRadio implementation ---------------------- */
 
-class CommRadioImpl: public CommRadio, OutputProperties {
-
+class CommRadioImpl: public CommRadio,
+                     OutputProperties
+{
 public:
-  CommRadioImpl(SGPropertyNode_ptr node);
-  virtual ~CommRadioImpl();
+    CommRadioImpl(SGPropertyNode_ptr node);
+    virtual ~CommRadioImpl();
 
-  virtual void update(double dt);
-  virtual void init();
-  void bind();
-  void unbind();
+    // Subsystem API.
+    void bind() override;
+    void init() override;
+    void unbind() override;
+    void update(double dt) override;
 
 private:
-  string getSampleGroupRefname() const
-  {
-    return _rootNode->getPath();
-  }
+    bool _useEightPointThree = false;
+    MetarBridgeRef _metarBridge;
+    AtisSpeaker _atisSpeaker;
+    SGSharedPtr<FrequencyFormatterBase> _useFrequencyFormatter;
+    SGSharedPtr<FrequencyFormatterBase> _stbyFrequencyFormatter;
+    const SignalQualityComputerRef _signalQualityComputer;
 
-  int _num;
-  MetarBridgeRef _metarBridge;
-  #if defined(ENABLE_FLITE)
-  AtisSpeaker _atisSpeaker;
-  #endif
-  SGSharedPtr<FrequencyFormatterBase> _useFrequencyFormatter;
-  SGSharedPtr<FrequencyFormatterBase> _stbyFrequencyFormatter;
-  const SignalQualityComputerRef _signalQualityComputer;
+    double _stationTTL = 0.0;
+    double _frequency = -1.0;
+    flightgear::CommStationRef _commStationForFrequency;
 
-  double _stationTTL;
-  double _frequency;
-  flightgear::CommStationRef _commStationForFrequency;
-  #if defined(ENABLE_FLITE)
-  SGSharedPtr<SGSampleGroup> _sampleGroup;
-  #endif
+    PropertyObject<double> _volume_norm;
+    PropertyObject<string> _atis;
+    PropertyObject<bool> _addNoise;
+    PropertyObject<double> _cutoffSignalQuality;
 
-  PropertyObject<bool> _serviceable;
-  PropertyObject<bool> _power_btn;
-  PropertyObject<bool> _power_good;
-  PropertyObject<double> _volume_norm;
-  PropertyObject<string> _atis;
-  PropertyObject<bool> _addNoise;
-  PropertyObject<double> _cutoffSignalQuality;
+    std::string _soundPrefix;
+    void stopAudio();
+    void updateAudio();
 
+    SGSampleGroup* _sampleGroup = nullptr;
 };
 
-CommRadioImpl::CommRadioImpl(SGPropertyNode_ptr node)
-    : OutputProperties(
-        fgGetNode("/instrumentation", true)->getNode(node->getStringValue("name", "comm"), node->getIntValue("number", 0), true)),
-        _num(node->getIntValue("number", 0)),
-        _metarBridge(new MetarBridge()),
-        _signalQualityComputer(new SimpleDistanceSquareSignalQualityComputer()),
-
-        _stationTTL(0.0),
-        _frequency(-1.0),
-        _commStationForFrequency(NULL),
-
-        _serviceable(_rootNode->getNode("serviceable", true)),
-        _power_btn(_rootNode->getNode("power-btn", true)),
-        _power_good(_rootNode->getNode("power-good", true)),
-        _volume_norm(_rootNode->getNode("volume", true)),
-        _atis(_rootNode->getNode("atis", true)),
-        _addNoise(_rootNode->getNode("add-noise", true)),
-        _cutoffSignalQuality(_rootNode->getNode("cutoff-signal-quality", true))
+CommRadioImpl::CommRadioImpl(SGPropertyNode_ptr node) :
+  _metarBridge(new MetarBridge),
+  _signalQualityComputer(new SimpleDistanceSquareSignalQualityComputer)
 {
-  if( node->getBoolValue("eight-point-three", false ) ) {
-	  _useFrequencyFormatter = new EightPointThreeFrequencyFormatter(
-		  _rootNode->getNode("frequencies", true),
-		  "selected-mhz",
-		  "selected-mhz-fmt",
-		  "selected-channel-width-khz",
-		  "selected-real-frequency-mhz",
-		  "selected-channel"
-		  );
-	  _stbyFrequencyFormatter = new EightPointThreeFrequencyFormatter(
-		  _rootNode->getNode("frequencies", true),
-		  "standby-mhz",
-		  "standby-mhz-fmt",
-		  "standby-channel-width-khz",
-		  "standby-real-frequency-mhz",
-		  "standby-channel"
-		  );
-  } else {
-      _useFrequencyFormatter = new FrequencyFormatter(
-    	  _rootNode->getNode("frequencies/selected-mhz", true),
-          _rootNode->getNode("frequencies/selected-mhz-fmt", true),
-		  0.025, 118.0, 137.0);
-
-      _stbyFrequencyFormatter = new FrequencyFormatter(
-    	  _rootNode->getNode("frequencies/standby-mhz", true),
-          _rootNode->getNode("frequencies/standby-mhz-fmt", true),
-		  0.025, 118.0, 137.0);
-
-
-  }
+  // set a special value to indicate we don't require a power supply node
+  // by default
+  setDefaultPowerSupplyPath("NO_DEFAULT");
+  readConfig(node, "comm");
+  _soundPrefix = name() + "_" + std::to_string(number()) + "_";
+  _useEightPointThree = node->getBoolValue("eight-point-three", false );
 }
 
 CommRadioImpl::~CommRadioImpl()
@@ -563,33 +509,69 @@ CommRadioImpl::~CommRadioImpl()
 
 void CommRadioImpl::bind()
 {
+  SGPropertyNode_ptr n = fgGetNode(nodePath(), true);
+  OutputProperties::bind(n);
+
+  _volume_norm = PropertyObject<double>(_rootNode->getNode("volume", true));
+  _atis = PropertyObject<string>(_rootNode->getNode("atis", true));
+  _addNoise = PropertyObject<bool>(_rootNode->getNode("add-noise", true));
+  _cutoffSignalQuality = PropertyObject<double>(_rootNode->getNode("cutoff-signal-quality", true));
+
   _metarBridge->setAtisNode(_atis.node());
-#if defined(ENABLE_FLITE)
   _atis.node()->addChangeListener(&_atisSpeaker);
-#endif
+
   // link the metar node. /environment/metar[3] is comm1 and /environment[4] is comm2.
   // see FGDATA/Environment/environment.xml
-  _metarBridge->setMetarPropertiesRoot(fgGetNode("/environment", true)->getNode("metar", _num + 3, true));
+  _metarBridge->setMetarPropertiesRoot(fgGetNode("/environment", true)->getNode("metar", number() + 3, true));
   _metarBridge->bind();
+
+  if (_useEightPointThree) {
+    _useFrequencyFormatter = new EightPointThreeFrequencyFormatter(
+                                                                   _rootNode->getNode("frequencies", true),
+                                                                   "selected-mhz",
+                                                                   "selected-mhz-fmt",
+                                                                   "selected-channel-width-khz",
+                                                                   "selected-real-frequency-mhz",
+                                                                   "selected-channel"
+                                                                   );
+    _stbyFrequencyFormatter = new EightPointThreeFrequencyFormatter(
+                                                                    _rootNode->getNode("frequencies", true),
+                                                                    "standby-mhz",
+                                                                    "standby-mhz-fmt",
+                                                                    "standby-channel-width-khz",
+                                                                    "standby-real-frequency-mhz",
+                                                                    "standby-channel"
+                                                                    );
+  } else {
+    _useFrequencyFormatter = new FrequencyFormatter(
+                                                    _rootNode->getNode("frequencies/selected-mhz", true),
+                                                    _rootNode->getNode("frequencies/selected-mhz-fmt", true),
+                                                    0.025, 118.0, 137.0);
+
+    _stbyFrequencyFormatter = new FrequencyFormatter(
+                                                     _rootNode->getNode("frequencies/standby-mhz", true),
+                                                     _rootNode->getNode("frequencies/standby-mhz-fmt", true),
+                                                     0.025, 118.0, 137.0);
+
+
+  }
 }
 
 void CommRadioImpl::unbind()
 {
-#if defined(ENABLE_FLITE)
   _atis.node()->removeChangeListener(&_atisSpeaker);
-  if (_sampleGroup.valid()) {
-      globals->get_subsystem<SGSoundMgr>()->remove(getSampleGroupRefname());
-  }
-#endif
+  
+  stopAudio();
+  
   _metarBridge->unbind();
+  AbstractInstrument::unbind();
 }
 
 void CommRadioImpl::init()
 {
-  // initialize power_btn to true if unset
-  string s = _power_btn.node()->getStringValue();
-  if (s.empty()) _power_btn = true;
+  initServicePowerProperties(_rootNode);
 
+  string s;
   // initialize squelch to a sane value if unset
   s = _cutoffSignalQuality.node()->getStringValue();
   if (s.empty()) _cutoffSignalQuality = 0.4;
@@ -597,6 +579,11 @@ void CommRadioImpl::init()
   // initialize add-noize to true if unset
   s = _addNoise.node()->getStringValue();
   if (s.empty()) _addNoise = true;
+  
+  auto soundManager = globals->get_subsystem<SGSoundMgr>();
+  if (soundManager) {
+    _sampleGroup = soundManager->find("atc", false);
+  }
 }
 
 void CommRadioImpl::update(double dt)
@@ -615,55 +602,11 @@ void CommRadioImpl::update(double dt)
     return;
   }
 
-#if defined(ENABLE_FLITE)
-  {
-    static const char * atisSampleRefName = "atis";
-    static const char * noiseSampleRefName = "noise";
-
-    if (_atisSpeaker.hasSpokenAtis()) {
-      // the speaker has created a new atis sample
-      if (!_sampleGroup.valid()) {
-        // create a sample group for our instrument on the fly
-          SGSoundMgr * smgr = globals->get_subsystem<SGSoundMgr>();
-
-        _sampleGroup = smgr->find(getSampleGroupRefname(), true);
-        _sampleGroup->tie_to_listener();
-        if (_addNoise) {
-          SGSharedPtr<SGSoundSample> noise = new SGSoundSample("Sounds/radionoise.wav", globals->get_fg_root());
-          _sampleGroup->add(noise, noiseSampleRefName);
-          _sampleGroup->play_looped(noiseSampleRefName);
-        }
-
-      }
-      // remove previous atis sample
-      _sampleGroup->remove(atisSampleRefName);
-      // add and play the new atis sample
-      SGSharedPtr<SGSoundSample> sample = _atisSpeaker.getSpokenAtis();
-      _sampleGroup->add(sample, atisSampleRefName);
-      _sampleGroup->play_looped(atisSampleRefName);
-    }
-
-    if (_sampleGroup.valid()) {
-      if (_addNoise) {
-        // if noise is configured and there is a noise sample
-        // scale noise and signal volume by signalQuality.
-        SGSoundSample * s = _sampleGroup->find(noiseSampleRefName);
-        if ( NULL != s) {
-          s->set_volume(1.0 - _signalQuality_norm);
-          s = _sampleGroup->find(atisSampleRefName);
-          s->set_volume(_signalQuality_norm);
-        }
-      }
-      // master volume for radio, mute on bad signal quality
-      _sampleGroup->set_volume(_signalQuality_norm >= _cutoffSignalQuality ? _volume_norm : 0.0);
-    }
-  }
-#endif
-
-  if (false == (_power_btn) || false == (_serviceable)) {
+  if (!isServiceableAndPowered()) {
     _metarBridge->clearMetar();
     _atis = "";
     _stationTTL = 0.0;
+    stopAudio();
     return;
   }
 
@@ -682,8 +625,11 @@ void CommRadioImpl::update(double dt)
 
   }
 
-  if (false == _commStationForFrequency.valid()) return;
-
+  if (false == _commStationForFrequency.valid()) {
+    stopAudio();
+    return;
+  }
+  
   _slantDistance_m = dist(_commStationForFrequency->cart(), SGVec3d::fromGeod(position));
 
   SGGeodesy::inverse(position, _commStationForFrequency->geod(), _trueBearingTo_deg, _trueBearingFrom_deg, _trackDistance_m);
@@ -695,9 +641,8 @@ void CommRadioImpl::update(double dt)
   _stationName = _commStationForFrequency->ident();
   _airportId = _commStationForFrequency->airport()->getId();
 
-#if defined(ENABLE_FLITE)
   _atisSpeaker.setStationId(_airportId);
-#endif
+
   switch (_commStationForFrequency->type()) {
     case FGPositioned::FREQ_ATIS:
       case FGPositioned::FREQ_AWOS: {
@@ -715,6 +660,59 @@ void CommRadioImpl::update(double dt)
       _atis = "";
       break;
   }
+  
+  updateAudio();
+}
+
+void CommRadioImpl::updateAudio()
+{
+  if (!_sampleGroup)
+    return;
+  
+  const string noiseRef = _soundPrefix + "_noise";
+  const string atisRef = _soundPrefix + "_atis";
+
+  SGSoundSample* noiseSample = _sampleGroup->find(noiseRef);
+  
+  // create noise sample if necessary, and play forever
+  if (_addNoise && !noiseSample) {
+    SGSharedPtr<SGSoundSample> noise = new SGSoundSample("Sounds/radionoise.wav", globals->get_fg_root());
+    _sampleGroup->add(noise, noiseRef);
+    _sampleGroup->play_looped(noiseRef);
+    noiseSample = noise;
+  }
+  
+  if (_atisSpeaker.hasSpokenAtis()) {
+    // the speaker has created a new atis sample
+    // remove previous atis sample
+    _sampleGroup->remove(atisRef);
+    
+    SGSharedPtr<SGSoundSample> sample = _atisSpeaker.getSpokenAtis();
+    _sampleGroup->add(sample, atisRef);
+    _sampleGroup->play_looped(atisRef);
+  }
+  
+  // adjust volumes
+  const bool doSquelch = (_signalQuality_norm < _cutoffSignalQuality);
+  double atisVolume = doSquelch ? 0.0 : _volume_norm;
+  if (_addNoise) {
+    const double noiseVol = (1.0 - _signalQuality_norm) * _volume_norm;
+    atisVolume = _signalQuality_norm * _volume_norm;
+    noiseSample->set_volume(doSquelch ? 0.0: noiseVol);
+  }
+  
+  SGSoundSample* s = _sampleGroup->find(atisRef);
+  if (s) {
+    s->set_volume(atisVolume);
+  }
+}
+  
+void CommRadioImpl::stopAudio()
+{
+  if (_sampleGroup) {
+    _sampleGroup->remove(_soundPrefix + "_noise");
+    _sampleGroup->remove(_soundPrefix + "_atis");
+  }
 }
 
 SGSubsystem * CommRadio::createInstance(SGPropertyNode_ptr rootNode)
@@ -722,5 +720,13 @@ SGSubsystem * CommRadio::createInstance(SGPropertyNode_ptr rootNode)
   return new CommRadioImpl(rootNode);
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<CommRadio> registrantCommRadio(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}});
+#endif
+
 } // namespace Instrumentation
 
