@@ -33,6 +33,8 @@ extern double pointsKnownDistanceFromGC(const SGGeoc& a, const SGGeoc&b, const S
 namespace flightgear
 {
 
+const auto turnComputeDist = SG_NM_TO_METER * 4.0;
+
 // declared in routePath.cxx
 bool geocRadialIntersection(const SGGeoc& a, double r1, const SGGeoc& b, double r2, SGGeoc& result);
 
@@ -62,15 +64,29 @@ double greatCircleCrossTrackError(double distanceOriginToPosition,double courseD
 	return crossTrackError * SG_RAD_TO_NM;
 }
 
+SGGeod computeTurnCenter(double turnRadiusM, const SGGeod& basePos, double inboundTrack, double turnAngle)
+{
+   
+   // this is the heading half way through the turn. Perpendicular to
+   // this is our turn center
+   const auto halfTurnHeading = inboundTrack + (turnAngle * 0.5);
+   double p = copysign(90.0, turnAngle);
+   double h = halfTurnHeading + p;
+   SG_NORMALIZE_RANGE(h, 0.0, 360.0);
+     
+   const double tcOffset = turnRadiusM / cos(turnAngle * 0.5 * SG_DEGREES_TO_RADIANS);
+   return SGGeodesy::direct(basePos, h, tcOffset);
+}
 
-////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////s
 
 WayptController::~WayptController()
 {
 }
 
-void WayptController::init()
+bool WayptController::init()
 {
+    return true;
 }
   
 bool WayptController::isDone() const
@@ -163,10 +179,11 @@ public:
       throw sg_exception("BasicWayptCtrl doesn't work with dynamic waypoints");
     }
   }
-  
-  virtual void init()
+
+  bool init() override
   {
     _targetTrack = SGGeodesy::courseDeg(_rnav->position(), _waypt->position());
+    return true;
   }
 
   virtual void update(double)
@@ -244,32 +261,144 @@ public:
     }
   }
 
-  virtual void init()
+  bool init() override
   {
-	bool isPreviousLegValid = false;
-
-	_waypointOrigin = _rnav->previousLegWaypointPosition(isPreviousLegValid);
-
-	_initialLegCourse 				= SGGeodesy::courseDeg(_waypointOrigin,_waypt->position());
-	_courseAircraftToTarget			= SGGeodesy::courseDeg(_rnav->position(),_waypt->position());
-	_distanceAircraftTargetMetre 	= SGGeodesy::distanceM(_rnav->position(),_waypt->position());
-
-
-	// check reach the leg in 45Deg or going direct
-	bool canReachLeg = (fabs(_initialLegCourse -_courseAircraftToTarget) < 45.0);
-
-	if (isPreviousLegValid && canReachLeg) {
+    auto previousLeg = _rnav->previousLegData();
+    if (previousLeg) {
+      _waypointOrigin = previousLeg.value().position;
+      _initialLegCourse = SGGeodesy::courseDeg(_waypointOrigin,_waypt->position());
+    } else {
+      // capture current position
+      _waypointOrigin = _rnav->position();
+    }
+    
+    _courseAircraftToTarget			= SGGeodesy::courseDeg(_rnav->position(),_waypt->position());
+    _distanceAircraftTargetMetre 	= SGGeodesy::distanceM(_rnav->position(),_waypt->position());
+    
+    
+    // check reach the leg in 45Deg or going direct
+    bool canReachLeg = (fabs(_initialLegCourse -_courseAircraftToTarget) < 45.0);
+    
+    if (previousLeg && canReachLeg) {
       _targetTrack = _initialLegCourse;
-	} else {
+    } else {
       // can't reach the leg with out a crazy turn, just go direct to the
       // destination waypt
       _targetTrack = _courseAircraftToTarget;
       _initialLegCourse = _courseAircraftToTarget;
       _waypointOrigin = _rnav->position();
-	}
+    }
+      
+  // turn angle depends on final leg course, not initial
+  // do this here so we have a chance of doing a fly-by at the end of the leg
+  _finalLegCourse = SGGeodesy::courseDeg(_waypt->position(), _waypointOrigin) + 180;
+  SG_NORMALIZE_RANGE(_finalLegCourse, 0.0, 360.0);
+    
+    // turn-in logic
+    if (previousLeg.has_value() && previousLeg.value().didFlyBy) {
+      _flyByTurnCenter = previousLeg.value().flyByTurnCenter;
+      _flyByTurnAngle = previousLeg.value().turnAngle;
+      _flyByTurnRadius = previousLeg.value().flyByRadius;
+      _entryFlyByActive = true;
+      SG_NORMALIZE_RANGE(_flyByTurnAngle, -180.0, 180.0);
+    }
+
+    return true;
+  }
+  
+  void computeTurnAnticipation()
+  {
+    _didComputeTurn = true;
+    
+    if (_waypt->flag(WPT_OVERFLIGHT)) {
+      return; // can't fly-by
+    }
+    
+    if (!_rnav->canFlyBy())
+      return;
+    
+    auto nextLegTrack = _rnav->nextLegTrack();
+    
+    if (!nextLegTrack.has_value()) {
+      return;
+    }
+    
+    _flyByTurnAngle = nextLegTrack.value() - _finalLegCourse;
+    SG_NORMALIZE_RANGE(_flyByTurnAngle, -180.0, 180.0);
+    
+    if (fabs(_flyByTurnAngle) > 120.0) {
+      // too sharp, don't anticipate
+      return;
+    }
+    
+    _flyByTurnRadius =  _rnav->turnRadiusNm() * SG_NM_TO_METER;
+      _flyByTurnCenter = computeTurnCenter(_flyByTurnRadius, _waypt->position(), _finalLegCourse, _flyByTurnAngle);
+    _doFlyBy = true;
+  }
+  
+  bool updateInTurn()
+  {
+    // find bearing to turn center
+    // when it hits 90 off our track
+    
+    auto bearingToTurnCenter = SGGeodesy::courseDeg(_rnav->position(), _flyByTurnCenter);
+    auto distToTurnCenter = SGGeodesy::distanceM(_rnav->position(), _flyByTurnCenter);
+
+    if (!_flyByStarted) {
+      // check for the turn center being 90 degrees off one wing (start the turn)
+      auto a = bearingToTurnCenter - _finalLegCourse;
+      SG_NORMALIZE_RANGE(a, -180.0, 180.0);
+        if (fabs(a) < 90.0) {
+        return false; // keep flying normal leg
+      }
+      
+      _flyByStarted = true;
+    }
+    
+    // check for us passing the half-way point; that's when we should
+    // sequence to the next WP
+    const auto halfPointAngle = (_finalLegCourse + (_flyByTurnAngle * 0.5));
+    auto b = bearingToTurnCenter - halfPointAngle;
+    SG_NORMALIZE_RANGE(b, -180.0, 180.0);
+            
+    if (fabs(b) >= 90.0) {
+      _toFlag = false;
+      setDone();
+    }
+    
+    // in the actual turn, our desired track is always pependicular to the
+    // bearing to the turn center we computed
+    _targetTrack = bearingToTurnCenter - copysign(90, _flyByTurnAngle);
+      
+    SG_NORMALIZE_RANGE(_targetTrack, 0.0, 360.0);
+
+    _crossTrackError = (distToTurnCenter - _flyByTurnRadius) * SG_METER_TO_NM;
+    _courseDev = _crossTrackError * 10.0; // arbitrary guess for now
+    
+    return true;
+  }
+  
+  void updateInEntryTurn()
+  {
+    auto bearingToTurnCenter = SGGeodesy::courseDeg(_rnav->position(), _flyByTurnCenter);
+    auto distToTurnCenter = SGGeodesy::distanceM(_rnav->position(), _flyByTurnCenter);
+  
+    auto b = bearingToTurnCenter - _initialLegCourse;
+
+      SG_NORMALIZE_RANGE(b, -180.0, 180.0);
+    if (fabs(b) >= 90.0) {
+      _entryFlyByActive = false;
+      return; // we're done with the entry turn
+    }
+    
+    _targetTrack = bearingToTurnCenter - copysign(90, _flyByTurnAngle);
+
+    SG_NORMALIZE_RANGE(_targetTrack, 0.0, 360.0);
+    _crossTrackError = (distToTurnCenter - _flyByTurnRadius) * SG_METER_TO_NM;
+    _courseDev = _crossTrackError * 10.0; // arbitrary guess for now
   }
 
-  virtual void update(double)
+  void update(double) override
   {
     _courseOriginToAircraft			= SGGeodesy::courseDeg(_waypointOrigin,_rnav->position());
     _distanceOriginAircraftMetre 	= SGGeodesy::distanceM(_waypointOrigin,_rnav->position());
@@ -277,39 +406,72 @@ public:
     _courseAircraftToTarget			= SGGeodesy::courseDeg(_rnav->position(),_waypt->position());
     _distanceAircraftTargetMetre 	= SGGeodesy::distanceM(_rnav->position(),_waypt->position());
 
+    if (_entryFlyByActive) {
+      updateInEntryTurn();
+      return;
+    }
+    
+    if (!_didComputeTurn && (_distanceAircraftTargetMetre < turnComputeDist)) {
+      computeTurnAnticipation();
+    }
+        
+    if (_didComputeTurn && _doFlyBy) {
+      bool ok = updateInTurn();
+      if (ok) {
+        return;
+      }
+      
+      // otherwise we fall through
+    }
+    
     // from the Aviation Formulary
 #if 0
-      Suppose you are proceeding on a great circle route from A to B (course =crs_AB) and end up at D,
-      perhaps off course. (We presume that A is ot a pole!) You can calculate the course from A to D
-      (crs_AD) and the distance from A to D (dist_AD) using the formulae above. In terms of these the
-      cross track error, XTD, (distance off course) is given by
-      
-      XTD =asin(sin(dist_AD)*sin(crs_AD-crs_AB))
-      (positive XTD means right of course, negative means left)
-      (If the point A is the N. or S. Pole replace crs_AD-crs_AB with
-       lon_D-lon_B or lon_B-lon_D, respectively.)
+    Suppose you are proceeding on a great circle route from A to B (course =crs_AB) and end up at D,
+    perhaps off course. (We presume that A is ot a pole!) You can calculate the course from A to D
+    (crs_AD) and the distance from A to D (dist_AD) using the formulae above. In terms of these the
+    cross track error, XTD, (distance off course) is given by
+    
+    XTD =asin(sin(dist_AD)*sin(crs_AD-crs_AB))
+    (positive XTD means right of course, negative means left)
+    (If the point A is the N. or S. Pole replace crs_AD-crs_AB with
+     lon_D-lon_B or lon_B-lon_D, respectively.)
 #endif
     // however, just for fun, our convention for polarity of the cross-track
     // sign is opposite, so we add a -ve to the computation below.
-      
+    
     double distOriginAircraftRad = _distanceOriginAircraftMetre * SG_METER_TO_NM * SG_NM_TO_RAD;
     double xtkRad = asin(sin(distOriginAircraftRad) * sin((_courseOriginToAircraft - _initialLegCourse) * SG_DEGREES_TO_RADIANS));
-      
+    
     // convert to NM and flip sign for consistency with existing code.
     // since we derive the abeam point and course-deviation from this, and
     // thus the GPS cdi-deflection, if we don't fix this here, the sign of
     // all of those comes out backwards.
     _crossTrackError = -(xtkRad * SG_RAD_TO_NM);
-
+    
     /*
      The "along track distance", ATD, the distance from A along the course towards B
      to the point abeam D is given by:
      ATD=acos(cos(dist_AD)/cos(XTD))
      */
     double atd = acos(cos(distOriginAircraftRad) / cos(xtkRad));
+
+    // fix sign of ATD, if we are 'behind' the waypoint origin, rather than
+    // in front of it. We need to set a -ve sign on atd, otherwise we compute
+    // an abeamPoint ahead of the waypoint origin, potentially so far ahead
+    // that the computed track is backwards.
+    // see test-case GPSTests::testCourseLegIntermediateWaypoint
+    // for this happening
+    {
+        double x = _courseOriginToAircraft - _initialLegCourse;
+        SG_NORMALIZE_RANGE(x, -180.0, 180.0);
+        if (fabs(x) > 90.0) {
+            atd = -atd;
+        }
+    }
+
     const double atdM = atd * SG_RAD_TO_NM * SG_NM_TO_METER;
     SGGeod abeamPoint = SGGeodesy::direct(_waypointOrigin, _initialLegCourse, atdM);
-    
+
     // if we're close to the target point, we enter something similar to the VOR zone
     // of confusion. Force target track to the final course from the origin.
     if (_distanceAircraftTargetMetre < (SG_NM_TO_METER * 2.0)) {
@@ -324,6 +486,7 @@ public:
     _courseDev = _courseAircraftToTarget - _targetTrack;
     SG_NORMALIZE_RANGE(_courseDev, -180.0, 180.0);
 
+    
     bool isMinimumOverFlightDistanceReached = _distanceAircraftTargetMetre < _rnav->overflightDistanceM();
     bool isOverFlightConeArmed 				= _distanceAircraftTargetMetre < ( _rnav->overflightArmDistanceM() + _rnav->overflightDistanceM() );
     bool leavingOverFlightCone 				= (fabs(_courseDev) > _rnav->overflightArmAngleDeg());
@@ -343,6 +506,22 @@ public:
     }
   }
 
+  simgear::optional<RNAV::LegData> legData() const override
+  {
+    RNAV::LegData r;
+    r.position = _waypt->position();
+    if (_doFlyBy) {
+      // copy all the fly by paramters, so the next controller can
+      // smoothly link up
+      r.didFlyBy = true;
+      r.flyByRadius = _flyByTurnRadius;
+      r.flyByTurnCenter = _flyByTurnCenter;
+      r.turnAngle = _flyByTurnAngle;
+    }
+    
+    return r;
+  }
+  
   virtual double distanceToWayptM() const
   {
     return _distanceAircraftTargetMetre;
@@ -381,6 +560,7 @@ private:
    */
   SGGeod _waypointOrigin;
   double _initialLegCourse;
+  double _finalLegCourse;
   double _distanceOriginAircraftMetre;
   double _distanceAircraftTargetMetre;
   double _courseOriginToAircraft;
@@ -388,7 +568,14 @@ private:
   double _courseDev;
   bool _toFlag;
   double _crossTrackError;
-
+  
+  bool _didComputeTurn = false;
+  bool _doFlyBy = false;
+  SGGeod _flyByTurnCenter;
+  double _flyByTurnAngle;
+  double _flyByTurnRadius;
+  bool _entryFlyByActive = false;
+  bool _flyByStarted = false;
 };
 
 /**
@@ -406,11 +593,15 @@ public:
     _courseDev(0.0)
   {
   }
-  
-  virtual void init()
+
+  bool init() override
   {
     _runway = static_cast<RunwayWaypt*>(_waypt.get())->runway();
+    if (!_runway) {
+        return false;
+    }
     _targetTrack = _runway->headingDeg();
+    return true;
   }
 
   virtual void update(double)
@@ -481,11 +672,12 @@ public:
     }
   }
 
-  virtual void init()
+  bool init() override
   {
     HeadingToAltitude* w = (HeadingToAltitude*) _waypt.get();
     _targetTrack = w->headingDegMagnetic() + _rnav->magvarDeg();
       _filteredFPM = _lastFPM = _rnav->vspeedFPM();
+      return true;
   }
   
   virtual void update(double dt)
@@ -566,14 +758,18 @@ public:
     }
   }
 
-  virtual void init()
+  bool init() override
   {
     RadialIntercept* w = (RadialIntercept*) _waypt.get();
       _trueRadial = w->radialDegMagnetic() + _rnav->magvarDeg();
     _targetTrack = w->courseDegMagnetic() + _rnav->magvarDeg();
+      
+      _canFlyBy = !_waypt->flag(WPT_OVERFLIGHT) && _rnav->canFlyBy();
+
+      return true;
   }
   
-  virtual void update(double)
+   void update(double) override
   {
       SGGeoc c,
         geocPos = SGGeoc::fromGeod(_rnav->position()),
@@ -585,9 +781,7 @@ public:
           // try with a backwards offset from the waypt pos, in case the
           // procedure waypt location is too close. (eg, KSFO OCEAN SID)
 
-          SGGeoc navidAdjusted;
-          SGGeodesy::advanceRadM(geocWayptPos, _trueRadial, SG_NM_TO_METER * -10, navidAdjusted);
-
+          SGGeoc navidAdjusted = SGGeodesy::advanceDegM(geocWayptPos, _trueRadial, -10 * SG_NM_TO_METER);
           ok = geocRadialIntersection(geocPos, _targetTrack,
                                       navidAdjusted, _trueRadial, c);
           if (!ok) {
@@ -598,8 +792,16 @@ public:
       }
 
       _projectedPosition = SGGeod::fromGeoc(c);
-
-
+      _distanceToProjectedInterceptM =  SGGeodesy::distanceM(_rnav->position(), _projectedPosition);
+      if (!_didComputeTurn && (_distanceToProjectedInterceptM < turnComputeDist)) {
+          computeTurn();
+      }
+      
+      if (_doFlyBy) {
+          updateFlyByTurn();
+      }
+      
+      
     // note we want the outbound radial from the waypt, hence the ordering
     // of arguments to courseDeg
     double r = SGGeodesy::courseDeg(_waypt->position(), _rnav->position());
@@ -609,19 +811,91 @@ public:
       setDone();
     }
   }
+          
+    bool updateFlyByTurn()
+    {
+        // find bearing to turn center
+        // when it hits 90 off our track
+        
+        auto bearingToTurnCenter = SGGeodesy::courseDeg(_rnav->position(), _flyByTurnCenter);
+        auto distToTurnCenter = SGGeodesy::distanceM(_rnav->position(), _flyByTurnCenter);
+
+        if (!_flyByStarted) {
+          // check for the turn center being 90 degrees off one wing (start the turn)
+          auto a = bearingToTurnCenter - _targetTrack;
+          SG_NORMALIZE_RANGE(a, -180.0, 180.0);
+            if (fabs(a) < 90.0) {
+                return false;
+          }
+          
+          _flyByStarted = true;
+        }
+        
+        
+        // in the actual turn, our desired track is always pependicular to the
+        // bearing to the turn center we computed
+        _targetTrack = bearingToTurnCenter - copysign(90, _flyByTurnAngle);
+          
+        SG_NORMALIZE_RANGE(_targetTrack, 0.0, 360.0);
+        _crossTrackError = (distToTurnCenter - _flyByTurnRadius) * SG_METER_TO_NM;
+        return true;
+    }
+    
+    void computeTurn()
+    {
+        _didComputeTurn = true;
+        if (!_canFlyBy)
+            return;
+        
+        double inverseRadial = _trueRadial + 180.0;
+        SG_NORMALIZE_RANGE(inverseRadial, 0.0, 360.0);
+        _flyByTurnAngle = inverseRadial - _targetTrack;
+        SG_NORMALIZE_RANGE(_flyByTurnAngle, -180.0, 180.0);
+        
+        if (fabs(_flyByTurnAngle) > 120.0) {
+          // too sharp, no fly-by
+          return;
+        }
+        
+        _flyByTurnRadius =  _rnav->turnRadiusNm() * SG_NM_TO_METER;
+        _flyByTurnCenter = computeTurnCenter(_flyByTurnRadius, _projectedPosition, _targetTrack, _flyByTurnAngle);
+        _doFlyBy = true;
+    }
   
-  virtual double distanceToWayptM() const
+double distanceToWayptM() const override
   {
-    return SGGeodesy::distanceM(_rnav->position(), position());
+      return _distanceToProjectedInterceptM;
   }
 
-    virtual SGGeod position() const
+     SGGeod position() const override
     {
         return _projectedPosition;
     }
+    
+     double xtrackErrorNm() const override
+     {
+         if (!_flyByStarted)
+             return 0.0; // unless we're doing the fly-by, we're always on course
+         return _crossTrackError;
+     }
+
+    double courseDeviationDeg() const override
+     {
+         // guessed scaling factor
+         return xtrackErrorNm() * 10.0;
+     }
 private:
   double _trueRadial;
     SGGeod _projectedPosition;
+    bool _canFlyBy = false;
+    bool _didComputeTurn = false;
+    bool _doFlyBy = false;
+    double _distanceToProjectedInterceptM = 0.0;
+    SGGeod _flyByTurnCenter;
+    double _flyByTurnAngle;
+    double _flyByTurnRadius;
+    bool _flyByStarted = false;
+    double _crossTrackError = 0.0;
 };
 
 class DMEInterceptCtl : public WayptController
@@ -637,10 +911,11 @@ public:
     }
   }
 
-  virtual void init()
+  bool init() override
   {
     _dme  = (DMEIntercept*) _waypt.get();
     _targetTrack = _dme->courseDegMagnetic() + _rnav->magvarDeg();
+    return true;
   }
   
   virtual void update(double)
@@ -664,9 +939,7 @@ public:
         double distRad = _dme->dmeDistanceNm() * SG_NM_TO_RAD;
 
         // compute radial GC course
-        SGGeoc bPt;
-        SGGeodesy::advanceRadM(geocPos, _targetTrack, 100 * SG_NM_TO_RAD, bPt);
-
+        SGGeoc bPt = SGGeodesy::advanceDegM(geocPos, _targetTrack, 1e5);
         double dNm = pointsKnownDistanceFromGC(geocPos, bPt, navid, distRad);
         if (dNm < 0.0) {
             SG_LOG(SG_AUTOPILOT, SG_WARN, "DMEInterceptCtl::position failed");
@@ -699,7 +972,7 @@ HoldCtl::HoldCtl(RNAV* aRNAV, const WayptRef& aWpt) :
     }
 }
 
-void HoldCtl::init()
+bool HoldCtl::init()
 {
   _segmentEnd = _waypt->position();
   _state = LEG_TO_HOLD;
@@ -707,6 +980,8 @@ void HoldCtl::init()
   // use leg controller to fly us to the hold point - this also gives
   // the normal legl behaviour if the hold is not enabled
   setSubController(new LegWayptCtl(_rnav, _waypt));
+
+  return true;
 }
   
 void HoldCtl::computeEntry()
@@ -1032,9 +1307,9 @@ public:
   {
   }
 
-  virtual void init()
+  bool init() override
   {
- 
+      return true;
   }
   
   virtual void update(double)
@@ -1066,13 +1341,15 @@ DirectToController::DirectToController(RNAV* aRNAV, const WayptRef& aWpt, const
 {
 }
 
-void DirectToController::init()
+bool DirectToController::init()
 {
   if (_waypt->flag(WPT_DYNAMIC)) {
-    throw sg_exception("can't direct-to a dynamic waypoint");
+      SG_LOG(SG_AUTOPILOT, SG_WARN, "can't use a dynamic waypoint for direct-to: " << _waypt->ident());
+      return false;
   }
 
   _targetTrack = SGGeodesy::courseDeg(_origin, _waypt->position());
+  return true;
 }
 
 void DirectToController::update(double)
@@ -1140,13 +1417,15 @@ OBSController::OBSController(RNAV* aRNAV, const WayptRef& aWpt) :
 {
 }
 
-void OBSController::init()
+bool OBSController::init()
 {
   if (_waypt->flag(WPT_DYNAMIC)) {
-    throw sg_exception("can't use a dynamic waypoint for OBS mode");
+      SG_LOG(SG_AUTOPILOT, SG_WARN, "can't use a dynamic waypoint for OBS mode" << _waypt->ident());
+      return false;
   }
   
   _targetTrack = _rnav->selectedMagCourse() + _rnav->magvarDeg();
+  return true;
 }
 
 void OBSController::update(double)
