@@ -233,6 +233,12 @@ int _bootstrap_OSInit;
 // Main entry point; catch any exceptions that have made it this far.
 int main ( int argc, char **argv )
 {
+    // we don't want to accidently show a GUI box and block startup in
+    // non_GUI setups, so check this value early here, before options are
+    // processed
+    const bool headless = flightgear::Options::checkForArg(argc, argv, "disable-gui");
+    flightgear::setHeadlessMode(headless);
+
 #ifdef ENABLE_SIMD
   if (!detectSIMD()) {
     flightgear::fatalMessageBoxThenExit(
@@ -260,9 +266,7 @@ int main ( int argc, char **argv )
   _bootstrap_OSInit = 0;
     
 #if defined(HAVE_SENTRY)
-    if (flightgear::Options::checkForArg(argc, argv, "enable-sentry")) {
-        flightgear::initSentry();
-    }
+  flightgear::initSentry();
 #endif
 
 // if we're not using the normal crash-reported, install our
@@ -320,10 +324,12 @@ int main ( int argc, char **argv )
         flightgear::fatalMessageBoxWithoutExit("Fatal exception", e.what());
     } catch (const std::string &s) {
         flightgear::fatalMessageBoxWithoutExit("Fatal exception", s);
+    } catch (const flightgear::FatalErrorException&) {
+        // we already showed the message box, just carry on to exit
     } catch (const char *s) {
         std::cerr << "Fatal error (const char*): " << s << std::endl;
-
     } catch (...) {
+        flightgear::sentryReportException("Unknown main loop exception");
         std::cerr << "Unknown exception in the main loop. Aborting..." << std::endl;
         if (errno)
             perror("Possible cause");
