@@ -33,6 +33,7 @@
 #include <Environment/presets.hxx>
 #include <Navaids/NavDataCache.hxx>
 #include <GUI/gui.h>
+#include <Main/sentryIntegration.hxx>
 
 #include "fg_init.hxx"
 #include "fg_io.hxx"
@@ -194,7 +195,13 @@ do_null (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_nasal (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    return ((FGNasalSys*)globals->get_subsystem("nasal"))->handleCommand(arg, root);
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
+    if (!nasalSys) {
+        SG_LOG(SG_GUI, SG_ALERT, "do_nasal command: Nasal subsystem not found");
+        return false;
+    }
+
+    return nasalSys->handleCommand(arg, root);
 }
 
 /**
@@ -735,6 +742,10 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
     if (file.extension() != "xml")
         file.concat(".xml");
 
+    // some Nasal uses loadxml to also speculatively probe for existence of
+    // files. This flag allows us not to be noisy in the logs, in that case.
+    const bool quiet = arg->getBoolValue("quiet", false);
+
     std::string icao = arg->getStringValue("icao");
     if (icao.empty()) {
         if (file.isRelative()) {
@@ -743,23 +754,32 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
               file = absPath;
           else
           {
-              SG_LOG(SG_IO, SG_ALERT, "loadxml: Cannot find XML property file '"
-                          << file << "'.");
+              if (!quiet) {
+                  SG_LOG(SG_IO, SG_ALERT, "loadxml: Cannot find XML property file '" << file << "'.");
+              }
               return false;
           }
         }
     } else {
         if (!XMLLoader::findAirportData(icao, file.utf8Str(), file)) {
-          SG_LOG(SG_IO, SG_INFO, "loadxml: failed to find airport data for "
-            << file << " at ICAO:" << icao);
+            if (!quiet) {
+                SG_LOG(SG_IO, SG_INFO, "loadxml: failed to find airport data for " << file << " at ICAO:" << icao);
+            }
           return false;
         }
     }
 
+    if (!file.exists()) {
+        if (!quiet) {
+            SG_LOG(SG_IO, SG_WARN, "loadxml: no such file:" << file);
+        }
+        return false;
+    }
+
     SGPath validated_path = fgValidatePath(file, false);
     if (validated_path.isNull()) {
-        SG_LOG(SG_IO, SG_ALERT, "loadxml: reading '" << file << "' denied "
-                "(unauthorized directory - authorization no longer follows symlinks; to authorize reading additional directories, pass them to --allow-nasal-read)");
+        SG_LOG(SG_IO, quiet ? SG_DEV_WARN : SG_ALERT, "loadxml: reading '" << file << "' denied "
+                                                                                      "(unauthorized directory - authorization no longer follows symlinks; to authorize reading additional directories, pass them to --allow-nasal-read)");
         return false;
     }
 
@@ -769,12 +789,17 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
     else
         targetnode = const_cast<SGPropertyNode *>(arg)->getNode("data", true);
 
+    // don't report Sentry errors for Nasal-loaded XML, since it makes
+    // for very noisy reports
+    flightgear::sentryThreadReportXMLErrors(false);
     try {
         readProperties(validated_path, targetnode, true);
     } catch (const sg_exception &e) {
-        SG_LOG(SG_IO, SG_WARN, "loadxml: " << e.getFormattedMessage());
+        SG_LOG(SG_IO, quiet ? SG_DEV_WARN : SG_WARN, "loadxml exception: " << e.getFormattedMessage());
+        flightgear::sentryThreadReportXMLErrors(true);
         return false;
     }
+    flightgear::sentryThreadReportXMLErrors(true);
 
     return true;
 }
