@@ -61,12 +61,13 @@
 #include <simgear/props/props_io.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
 
-#include <simgear/scene/model/modellib.hxx>
-#include <simgear/scene/material/matlib.hxx>
 #include <simgear/scene/material/Effect.hxx>
+#include <simgear/scene/material/matlib.hxx>
+#include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/model/particles.hxx>
-#include <simgear/scene/tsync/terrasync.hxx>
+#include <simgear/scene/tgdb/TreeBin.hxx>
 #include <simgear/scene/tgdb/userdata.hxx>
+#include <simgear/scene/tsync/terrasync.hxx>
 
 #include <simgear/package/Root.hxx>
 #include <simgear/package/Package.hxx>
@@ -114,6 +115,7 @@
 #include <Sound/voice.hxx>
 #include <Sound/soundmanager.hxx>
 #include <Systems/system_mgr.hxx>
+#include <Time/tide.hxx>
 #include <Time/light.hxx>
 #include <Time/TimeManager.hxx>
 
@@ -233,13 +235,14 @@ public:
         try {
           readProperties(setFile, globals->get_props());
         } catch ( const sg_exception &e ) {
-          SG_LOG(SG_INPUT, SG_ALERT,
-                 "Error reading aircraft: " << e.getFormattedMessage());
-          flightgear::fatalMessageBoxWithoutExit(
-            "Error reading aircraft",
-            "An error occured reading the requested aircraft (" + aircraft + ")",
-            e.getFormattedMessage());
-          return false;
+            SG_LOG(SG_IO, SG_ALERT,
+                   "Error reading aircraft: " << e.getFormattedMessage());
+            SG_LOG(SG_IO, SG_ALERT, "aircraft dir is:" << aircraftDir);
+            flightgear::fatalMessageBoxWithoutExit(
+                "Error reading aircraft",
+                "An error occured reading the requested aircraft (" + aircraft + ")",
+                e.getFormattedMessage());
+            return false;
         }
 
         checkAircraftMinVersion();
@@ -260,14 +263,16 @@ public:
     }
     
     if (!checkCache()) {
-      // prepare cache for re-scan
-      SGPropertyNode *n = _cache->getNode("fg-root", true);
-      n->setStringValue(globals->get_fg_root().utf8Str());
-      n->setAttribute(SGPropertyNode::USERARCHIVE, true);
-      n = _cache->getNode("fg-aircraft", true);
-      n->setStringValue(getAircraftPaths().c_str());
-      n->setAttribute(SGPropertyNode::USERARCHIVE, true);
-      _cache->removeChildren("aircraft");
+        flightgear::addSentryBreadcrumb("Scanning aircraft paths", "info");
+
+        // prepare cache for re-scan
+        SGPropertyNode* n = _cache->getNode("fg-root", true);
+        n->setStringValue(globals->get_fg_root().utf8Str());
+        n->setAttribute(SGPropertyNode::USERARCHIVE, true);
+        n = _cache->getNode("fg-aircraft", true);
+        n->setStringValue(getAircraftPaths().c_str());
+        n->setAttribute(SGPropertyNode::USERARCHIVE, true);
+        _cache->removeChildren("aircraft");
   
         visitAircraftPaths();
     }
@@ -275,6 +280,9 @@ public:
     if (_foundPath.isNull()) {
       SG_LOG(SG_GENERAL, SG_ALERT,
              "Cannot find the specified aircraft: '" << aircraft << "'");
+
+      SG_LOG(SG_GENERAL, SG_ALERT, "\tin paths:" << SGPath::join(globals->get_aircraft_paths(), ";"));
+
       flightgear::fatalMessageBoxWithoutExit(
         "Aircraft not found",
         "The requested aircraft (" + aircraft + ") could not be found "
@@ -335,8 +343,9 @@ private:
       SGPath xml(cache[i]->getStringValue("path", ""));
       xml.append(name);
       if (xml.exists()) {
-        _foundPath = xml;
-        return true;
+          flightgear::addSentryBreadcrumb("Found aircraft via cache", "info");
+          _foundPath = xml;
+          return true;
       } 
       
       return false;
@@ -456,7 +465,7 @@ SGPath fgHomePath()
     return SGPath::fromEnv("FG_HOME", platformDefaultDataPath());
 }
 
-bool fgInitHome()
+InitHomeResult fgInitHome()
 {
   SGPath dataPath = fgHomePath();
   globals->set_fg_home(dataPath);
@@ -471,16 +480,16 @@ bool fgInitHome()
           "Problem setting up user data",
           "Unable to create the user-data storage folder at '" +
           dataPath.utf8Str() + "'.");
-        return false;
+        return InitHomeAbort;
     }
     
     if (fgGetBool("/sim/fghome-readonly", false)) {
         // user / config forced us into readonly mode, fine
         SG_LOG(SG_GENERAL, SG_INFO, "Running with FG_HOME readonly");
-        return true;
+        return InitHomeExplicitReadOnly;
     }
     
-	bool result = false;
+    InitHomeResult result = InitHomeOkay;
 #if defined(SG_WINDOWS)
 	// don't use a PID file on Windows, because deleting on close is
 	// unreliable and causes false-positives. Instead, use a named
@@ -490,14 +499,14 @@ bool fgInitHome()
 	if (static_fgHomeWriteMutex == nullptr) {
 		printf("CreateMutex error: %d\n", GetLastError());
 		SG_LOG(SG_GENERAL, SG_POPUP, "Failed to create mutex for multi-app protection");
-		return false;
+        return InitHomeAbort;
 	} else if (GetLastError() == ERROR_ALREADY_EXISTS) {
 		SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
 		fgSetBool("/sim/fghome-readonly", true);
-		return true;
+        return InitHomeReadOnly;
 	} else {
 		SG_LOG(SG_GENERAL, SG_INFO, "Created multi-app mutex, we are in writeable mode");
-		result = true;
+        result = InitHomeOkay;
 	}
 #else
 // write our PID, and check writeability
@@ -509,26 +518,28 @@ bool fgInitHome()
         if (fd < 0) {
             SG_LOG(SG_GENERAL, SG_ALERT, "failed to open local file:" << pidPath
                    << "\n\tdue to:" << simgear::strutils::error_string(errno));
-            return false;
+            return InitHomeAbort;
         }
         
         int err = ::flock(fd, LOCK_EX | LOCK_NB);
         if (err < 0) {
             if ( errno ==  EWOULDBLOCK) {
-                SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only.");
+                SG_LOG(SG_GENERAL, SG_ALERT, "flightgear instance already running, switching to FG_HOME read-only. ");
+                SG_LOG(SG_GENERAL, SG_ALERT, "Couldn't flock() file at:" << pidPath);
+
                 // set a marker property so terrasync/navcache don't try to write
                 // from secondary instances
                 fgSetBool("/sim/fghome-readonly", true);
-                return true;
+                return InitHomeReadOnly;
             } else {
                 SG_LOG(SG_GENERAL, SG_ALERT, "failed to lock file:" << pidPath
                        << "\n\tdue to:" << simgear::strutils::error_string(errno));
-                return false;
+                return InitHomeAbort;
             }
         }
         
        // we locked it!
-        result = true;
+        result = InitHomeOkay;
     } else {
         char buf[16];
        std::string ps = pidPath.utf8Str();
@@ -538,24 +549,24 @@ bool fgInitHome()
         if (fd < 0) {
             SG_LOG(SG_GENERAL, SG_ALERT, "failed to open local file:" << pidPath
                << "\n\tdue to:" << simgear::strutils::error_string(errno));
-            return false;
+            return InitHomeAbort;
         }
             
         int err = write(fd, buf, len);
         if (err < 0) {
             SG_LOG(SG_GENERAL, SG_ALERT, "failed to write to lock file:" << pidPath
             << "\n\tdue to:" << simgear::strutils::error_string(errno));
-            return false;
+            return InitHomeAbort;
         }
 
         err = flock(fd, LOCK_EX);
         if (err != 0) {
             SG_LOG(SG_GENERAL, SG_ALERT, "failed to lock file:" << pidPath
             << "\n\tdue to:" << simgear::strutils::error_string(errno));
-            return false;
+            return InitHomeAbort;
         }
         
-        result = true;
+        result = InitHomeOkay;
     }
 #endif
     fgSetBool("/sim/fghome-readonly", false);
@@ -576,6 +587,16 @@ void fgShutdownHome()
 #endif
 }
 
+void fgDeleteLockFile()
+{
+#if defined(SG_WINDOWS)
+    // there's no file here, so we can't actually delete anything
+#else
+    SGPath pidPath = globals->get_fg_home() / "fgfs_lock.pid";
+    pidPath.remove();
+#endif
+}
+
 static void createBaseStorageDirForAddons(const SGPath& exportDir)
 {
     SGPath addonStorageBasePath = exportDir / "Addons";
@@ -654,12 +675,6 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
     
     static SimLogFileLine   simLogFileLine;
 
-#ifdef ENABLE_SWIFT
-    //Set standard settings for swift connection
-    fgSetString("/sim/swift/adress","127.0.0.1");
-    fgSetString("/sim/swift/port","45003");
-#endif
-
     // Read global defaults from $FG_ROOT/defaults
     SG_LOG(SG_GENERAL, SG_DEBUG, "Reading global defaults");
     SGPath defaultsXML = globals->get_fg_root() / "defaults.xml";
@@ -670,7 +685,13 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
             defaultsXML.utf8Str());
     }
 
-    fgLoadProps("defaults.xml", globals->get_props());
+    if(!fgLoadProps("defaults.xml", globals->get_props()))
+    {
+        flightgear::fatalMessageBoxThenExit(
+            "Corrupted file",
+            "Couldn't load an essential simulator data file as it is corrupted.",
+            defaultsXML.utf8Str());
+    }
     SG_LOG(SG_GENERAL, SG_DEBUG, "Finished Reading global defaults");
 
     // do not load user settings when reset to default is requested, or if
@@ -867,6 +888,7 @@ bool fgInitGeneral() {
     fgSetBool("/sim/startup/stderr-to-terminal", isatty(2) != 0 );
 
     sgUserDataInit( globals->get_props() );
+    flightgear::addSentryTag("have-reset", "no");
 
     return true;
 }
@@ -1087,8 +1109,13 @@ void fgCreateSubsystems(bool duringReset) {
 #ifdef ENABLE_IAX
     ////////////////////////////////////////////////////////////////////
     // Initialize the FGCom subsystem.
+    // very important this goes in the SOUND group, since IAXClient
+    // depends on OpenAL, which is shutdown when the SOUND group
+    // shutdown.
+    // Sentry: FLIGHTGEAR-66
+
     ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("fgcom", new FGCom);
+    globals->add_new_subsystem<FGCom>(SGSubsystemMgr::SOUND);
 #endif
 
     {
@@ -1105,6 +1132,7 @@ void fgCreateSubsystems(bool duringReset) {
     if (!duringReset) {
         globals->add_subsystem("lighting", new FGLight, SGSubsystemMgr::DISPLAY);
         globals->add_subsystem("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
+        globals->add_subsystem("tides", new FGTide );
     }
 
     globals->add_new_subsystem<FGAircraftModel>(SGSubsystemMgr::DISPLAY);
@@ -1190,6 +1218,12 @@ void fgStartReposition()
     envMgr->get_subsystem("realwx")->reinit();
   }
   
+    // needed for parking assignment to work after reposition
+    auto atcManager = globals->get_subsystem<FGATCManager>();
+    if (atcManager) {
+        atcManager->reposition();
+    }
+
   // need to bind FDMshell again
   fdm->bind();
 
@@ -1205,10 +1239,13 @@ void fgStartReposition()
     master_freeze->setBoolValue(false);
   }
   fgSetBool("/sim/sceneryloaded",false);
+  flightgear::addSentryBreadcrumb("end of reposition", "info");
 }
 
 void fgStartNewReset()
 {
+    flightgear::addSentryTag("have-reset", "yes");
+
     // save user settings now, so that USERARCIVE-d values changes since the
     // last init are recorded and hence re-loaded when we fgInitConfig down
     // later in this function. Otherwise all such settings are lost.
@@ -1229,6 +1266,11 @@ void fgStartNewReset()
     
     subsystemManger->shutdown();
     subsystemManger->unbind();
+
+    // hack fix for many reset crashes relating to the static instance
+    // of this class. Will be fixed better for future versions by making
+    // this a proper subsystem.
+    FGATCDialogNew::hackyReset();
     
     // remove most subsystems, with a few exceptions.
     for (int g=0; g<SGSubsystemMgr::MAX_GROUPS; ++g) {
@@ -1251,7 +1293,10 @@ void fgStartNewReset()
             // don't delete here, dropping the ref should be sufficient
         }
     } // of top-level groups iteration
-    
+
+    // drop any references to AI objects with TACAN
+    flightgear::NavDataCache::instance()->clearDynamicPositioneds();
+
     FGRenderer* render = globals->get_renderer();
     // needed or we crash in multi-threaded OSG mode
     render->getViewer()->stopThreading();
@@ -1275,20 +1320,39 @@ void fgStartNewReset()
     // preserve the event handler; re-creating it would entail fixing the
     // idle handler
     osg::ref_ptr<flightgear::FGEventHandler> eventHandler = render->getEventHandler();
-    
+    // tell the event handler to drop properties, etc
+    eventHandler->clear();
+
     globals->set_renderer(NULL);
     globals->set_matlib(NULL);
-    
+
+    flightgear::unregisterMainLoopProperties();
+    FGReplayData::resetStatisticsProperties();
+
+    simgear::clearSharedTreeGeometry();
     simgear::clearEffectCache();
     simgear::SGModelLib::resetPropertyRoot();
-    simgear::GlobalParticleCallback::setSwitch(NULL);
-    simgear::UniformFactory::instance()->reset();
-    
+    simgear::ParticlesGlobalManager::clear();
+    simgear::UniformFactory::instance()->reset();    
+
+    flightgear::addons::AddonManager::reset();
+
     globals->resetPropertyRoot();
     // otherwise channels are duplicated
     globals->get_channel_options_list()->clear();
 
-    flightgear::addons::AddonManager::reset();
+    // IMPORTANT
+    // this is the low-water mark of the reset process.
+    // Subsystems (except the special ones), properties, OSG nodes, Effects
+    // should all be gone at this, except for special things, such as the
+    // splash node.
+    // From here onwards we're recreating early parts of main/init, before
+    // we restart the main loop.
+    // This is the place to check that instances of classes have all be
+    // cleaned up correctly. (Also, the OSG threads are all paused, we're back
+    // in single threaded mode)
+    /////////////////////
+
     flightgear::addons::AddonManager::createInstance();
 
     fgInitConfig(0, NULL, true);
@@ -1350,7 +1414,7 @@ void fgStartNewReset()
     globals->set_matlib( new SGMaterialLib );
     
 // terra-sync needs the property tree root, pass it back in
-    auto terra_sync = static_cast<simgear::SGTerraSync*>(subsystemManger->get_subsystem("terrasync"));
+    auto terra_sync = subsystemManger->get_subsystem<simgear::SGTerraSync>();
     if (terra_sync) {
         terra_sync->setRoot(globals->get_props());
     }
@@ -1358,6 +1422,8 @@ void fgStartNewReset()
     fgSetBool("/sim/signals/reinit", false);
     fgSetBool("/sim/freeze/master", false);
     fgSetBool("/sim/sceneryloaded",false);
+
+    flightgear::addSentryBreadcrumb("end of reset", "info");
 }
 
 void fgInitPackageRoot()
