@@ -194,20 +194,18 @@ setLoggingPriority (const char * p)
 {
   if (p == 0)
       return;
+
   string priority = p;
-  if (priority == "bulk") {
-    sglog().set_log_priority(SG_BULK);
-  } else if (priority == "debug") {
-    sglog().set_log_priority(SG_DEBUG);
-  } else if (priority.empty() || priority == "info") { // default
-    sglog().set_log_priority(SG_INFO);
-  } else if (priority == "warn") {
-    sglog().set_log_priority(SG_WARN);
-  } else if (priority == "alert") {
-    sglog().set_log_priority(SG_ALERT);
+  if (priority.empty()) {
+      sglog().set_log_priority(SG_INFO);
   } else {
-    SG_LOG(SG_GENERAL, SG_WARN, "Unknown logging priority " << priority);
+      try {
+          sglog().set_log_priority(logstream::priorityFromString(priority));
+      } catch (std::exception& e) {
+          SG_LOG(SG_GENERAL, SG_WARN, "Unknown logging priority: " << priority);
+      }
   }
+
   SG_LOG(SG_GENERAL, SG_DEBUG, "Logging priority is " << getLoggingPriority());
 }
 
@@ -231,7 +229,8 @@ setFreeze (bool f)
     frozen = f;
 
     // Pause the particle system
-    simgear::Particles::setFrozen(f);
+    auto p = simgear::ParticlesGlobalManager::instance();
+    p->setFrozen(f);
 }
 
 
@@ -279,6 +278,7 @@ setDateString (const char * date_string)
 
 				// Scan for basic ISO format
 				// YYYY-MM-DDTHH:MM:SS
+  new_time.tm_isdst = 0;
   int ret = sscanf(date_string, "%d-%d-%dT%d:%d:%d",
 		   &(new_time.tm_year), &(new_time.tm_mon),
 		   &(new_time.tm_mday), &(new_time.tm_hour),
@@ -293,7 +293,6 @@ setDateString (const char * date_string)
 	   << " not in YYYY-MM-DDTHH:MM:SS format; skipped");
     return;
   }
-
 				// OK, it looks like we got six
 				// values, one way or another.
   new_time.tm_year -= 1900;
