@@ -39,6 +39,7 @@
 #include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/propertyObject.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/props/AtomicChangeListener.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/package/Root.hxx>
 
@@ -165,8 +166,7 @@ FGGlobals::FGGlobals() :
     channel_options_list( NULL ),
     initial_waypoints( NULL ),
     channellist( NULL ),
-    haveUserSettings(false),
-    _headless(false)
+    haveUserSettings(false)
 {
     SGPropertyNode* root = new SGPropertyNode;
     props = SGPropertyNode_ptr(root);
@@ -208,14 +208,19 @@ FGGlobals::~FGGlobals()
 
     // stop OSG threading first, to avoid thread races while we tear down
     // scene-graph pieces
-    osg::ref_ptr<osgViewer::Viewer> vw(renderer->getViewer());
-    if (vw) {
-        // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
-        // explicitly stop trheading before we delete the renderer or
-        // viewMgr (which ultimately holds refs to the CameraGroup, and
-        // GraphicsContext)
-        vw->stopThreading();
+    // there are some scenarios where renderer is already gone.
+    osg::ref_ptr<osgViewer::Viewer> vw;
+    if (renderer) {
+        vw = renderer->getViewer();
+        if (vw) {
+            // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
+            // explicitly stop trheading before we delete the renderer or
+            // viewMgr (which ultimately holds refs to the CameraGroup, and
+            // GraphicsContext)
+            vw->stopThreading();
+        }
     }
+
     subsystem_mgr->shutdown();
     subsystem_mgr->unbind();
 
@@ -227,17 +232,19 @@ FGGlobals::~FGGlobals()
     }
 
     osgDB::Registry::instance()->clearObjectCache();
-    subsystem_mgr->remove(FGScenery::staticSubsystemClassId());
+    if (subsystem_mgr->get_subsystem(FGScenery::staticSubsystemClassId())) {
+        subsystem_mgr->remove(FGScenery::staticSubsystemClassId());
+    }
 
     // renderer touches subsystems during its destruction
-    set_renderer(NULL);
+    set_renderer(nullptr);
 
     FGFontCache::shutdown();
     fgCancelSnapShot();
 
     delete subsystem_mgr;
-    subsystem_mgr = NULL; // important so ::get_subsystem returns NULL
-    vw = nullptr; // don't delete the viewer until now
+    subsystem_mgr = nullptr; // important so ::get_subsystem returns NULL
+    vw = nullptr;
     set_matlib(NULL);
 
     delete time_params;
@@ -245,6 +252,13 @@ FGGlobals::~FGGlobals()
     delete initial_waypoints;
     delete channellist;
 
+    // delete commands before we release the property root
+    // this avoids crash where commands might be storing a propery
+    // ref/pointer.
+    // see https://sentry.io/organizations/flightgear/issues/1890563449
+    delete commands;
+    commands = nullptr;
+    
     simgear::PropertyObjectBase::setDefaultRoot(NULL);
     simgear::SGModelLib::resetPropertyRoot();
     delete locale;
@@ -254,7 +268,6 @@ FGGlobals::~FGGlobals()
 
     props.clear();
 
-    delete commands;
     delete simgear::ResourceManager::instance();
 }
 
@@ -302,6 +315,7 @@ PathList FGGlobals::get_data_paths() const
 {
     PathList r(additional_data_paths);
     r.push_back(fg_root);
+    r.insert(r.end(), _dataPathsAfterFGRoot.begin(), _dataPathsAfterFGRoot.end());
     return r;
 }
 
@@ -318,16 +332,26 @@ PathList FGGlobals::get_data_paths(const std::string& suffix) const
     return r;
 }
 
-void FGGlobals::append_data_path(const SGPath& path)
+void FGGlobals::append_data_path(const SGPath& path, bool afterFGRoot)
 {
     if (!path.exists()) {
         SG_LOG(SG_GENERAL, SG_WARN, "adding non-existant data path:" << path);
     }
 
-    additional_data_paths.push_back(path);
+    using RM = simgear::ResourceManager;
+    auto resManager = RM::instance();
+    if (afterFGRoot) {
+        _dataPathsAfterFGRoot.push_back(path);
+        // after FG_ROOT
+        resManager->addBasePath(path, static_cast<RM::Priority>(RM::PRIORITY_DEFAULT - 10));
+    } else {
+        additional_data_paths.push_back(path);
+        // after NORMAL prioirty, but ahead of FG_ROOT
+        resManager->addBasePath(path, static_cast<RM::Priority>(RM::PRIORITY_DEFAULT + 10));
+    }
 }
 
-SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
+SGPath FGGlobals::findDataPath(const std::string& pathSuffix) const
 {
     for (SGPath p : additional_data_paths) {
         p.append(pathSuffix);
@@ -342,8 +366,15 @@ SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
         return rootPath;
     }
 
-    SG_LOG(SG_GENERAL, SG_WARN, "dir not found in any data path:" << pathSuffix);
-    return SGPath();
+    for (SGPath p : _dataPathsAfterFGRoot) {
+        p.append(pathSuffix);
+        if (p.exists()) {
+            return p;
+        }
+    }
+
+    SG_LOG(SG_IO, SG_WARN, "not found in any data path: '" << pathSuffix << "'");
+    return SGPath{};
 }
 
 void FGGlobals::append_fg_scenery (const PathList &paths)
@@ -437,7 +468,7 @@ void FGGlobals::set_terrasync_dir(const SGPath &path)
 {
   if (terrasync_dir.realpath() != SGPath(fgGetString("/sim/terrasync/scenery-dir")).realpath()) {
     // if they don't match, /sim/terrasync/scenery-dir has been set by something else
-    SG_LOG(SG_GENERAL, SG_ALERT, "/sim/terrasync/scenery-dir is no longer stored across runs: if you wish to keep using a non-standard Terrasync directory, use --terrasync-dir or the launcher's settings");
+    SG_LOG(SG_GENERAL, SG_WARN, "/sim/terrasync/scenery-dir is no longer stored across runs: if you wish to keep using a non-standard Terrasync directory, use --terrasync-dir or the launcher's settings");
   }
   SGPath abspath(path.realpath());
   terrasync_dir = abspath;
@@ -472,7 +503,7 @@ void FGGlobals::append_aircraft_path(const SGPath& path)
 {
   SGPath dirPath(path);
   if (!dirPath.exists()) {
-    SG_LOG(SG_GENERAL, SG_ALERT, "aircraft path not found:" << path);
+    SG_LOG(SG_GENERAL, SG_WARN, "aircraft path not found:" << path);
     return;
   }
 
@@ -626,6 +657,9 @@ FGGlobals::resetPropertyRoot()
 {
     delete locale;
 
+    // avoid a warning when we processOptions after reset
+    terrasync_dir = SGPath{};
+
     cleanupListeners();
 
     // we don't strictly need to clear these (they will be reset when we
@@ -787,15 +821,9 @@ static void tryAutosaveMigration(const SGPath& userDataPath, SGPropertyNode* pro
     // copy remaining props out
     copyProperties(&oldProps, props);
 
-    // inform the user
-    FGLocale *locale = globals->get_locale();
-    const auto title = locale->getLocalizedString("settings-migration-title", "sys", "Settings migrated");
-    const auto msg = locale->getLocalizedString("settings-migration-text", "sys",
-                                                "Saved settings were migrated from a previous version of FlightGear. "
-                                                "If you encounter any problems when using the system, try restoring "
-                                                "the default settings, before reporting a problem. "
-                                                "Saved settings can affect the appearance, performance and features of the simulator.");
-    flightgear::modalMessageBox(title, msg);
+    // we can't inform the user yet, becuase embedded resources and the locale
+    // are not done. So we set a flag and check it once those things are done.
+    fgSetBool("/sim/autosave-migration/did-migrate", true);
 }
 
 // Load user settings from the autosave file (normally in $FG_HOME)
@@ -925,6 +953,8 @@ void FGGlobals::cleanupListeners()
         delete *i;
     }
     _listeners_to_cleanup.clear();
+
+    simgear::AtomicChangeListener::clearPendingChanges();
 }
 
 simgear::pkg::Root* FGGlobals::packageRoot()
@@ -939,12 +969,12 @@ void FGGlobals::setPackageRoot(const SGSharedPtr<simgear::pkg::Root>& p)
 
 bool FGGlobals::is_headless()
 {
-    return _headless;
+    return flightgear::isHeadlessMode();
 }
 
 void FGGlobals::set_headless(bool mode)
 {
-    _headless = mode;
+    flightgear::setHeadlessMode(mode);
 }
 
 // end of globals.cxx
