@@ -173,28 +173,12 @@ static void initTerrasync()
     // add the terrasync root as a data path so data can be retrieved from it
     // (even if we are in read-only mode)
     SGPath terraSyncDir(globals->get_terrasync_dir());
-    globals->append_data_path(terraSyncDir);
+    globals->append_data_path(terraSyncDir, false /* = ahead of FG_ROOT */);
 
     if (fgGetBool("/sim/fghome-readonly", false)) {
         return;
     }
 
-    // start TerraSync up now, so it can be synchronizing shared models
-    // and airports data in parallel with a nav-cache rebuild.
-    SGPath tsyncCache(terraSyncDir);
-    tsyncCache.append("terrasync-cache.xml");
-
-    // wipe the cache file if requested
-    if (flightgear::Options::sharedInstance()->isOptionSet("restore-defaults")) {
-        SG_LOG(SG_GENERAL, SG_INFO, "restore-defaults requested, wiping terrasync update cache at " <<
-               tsyncCache);
-        if (tsyncCache.exists()) {
-            tsyncCache.remove();
-        }
-    }
-
-    fgSetString("/sim/terrasync/cache-path", tsyncCache.utf8Str());
-
     // make fg-root dir available so existing Scenery data can be copied, and
     // hence not downloaded again.
     fgSetString("/sim/terrasync/installation-dir", (globals->get_fg_root() / "Scenery").utf8Str());
@@ -283,7 +267,9 @@ static void checkOpenGLVersion()
     } // of three parts
 }
 
-static void registerMainLoop()
+namespace flightgear {
+
+void registerMainLoop()
 {
     // stash current frame signal property
     frame_signal = fgGetNode("/sim/signals/frame", true);
@@ -292,6 +278,15 @@ static void registerMainLoop()
     fgRegisterIdleHandler( fgMainLoop );
 }
 
+void unregisterMainLoopProperties()
+{
+    frame_signal.reset();
+    nasal_gc_threaded.reset();
+    nasal_gc_threaded_wait.reset();
+}
+
+} // namespace flightgear
+
 // This is the top level master main function that is registered as
 // our idle function
 
@@ -338,6 +333,9 @@ static void fgIdleFunction ( void ) {
             throw sg_exception("General initialization failed");
         }
 
+        // now we have commands up
+        flightgear::delayedSentryInit();
+
         ////////////////////////////////////////////////////////////////////
         // Initialize the property-based built-in commands
         ////////////////////////////////////////////////////////////////////
@@ -424,8 +422,8 @@ static void fgIdleFunction ( void ) {
         // We've finished all our initialization steps, from now on we
         // run the main loop.
         fgSetBool("sim/sceneryloaded", false);
-        registerMainLoop();
-        
+        flightgear::registerMainLoop();
+
         ngccn = new simgear::Notifications::NasalGarbageCollectionConfigurationNotification(nasal_gc_threaded->getBoolValue(), nasal_gc_threaded_wait->getBoolValue());
          simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(*ngccn);
          simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(mln_started);
@@ -450,7 +448,7 @@ void fgInitSecureMode()
 {
     bool secureMode = true;
     if (Options::sharedInstance()->isOptionSet("allow-nasal-from-sockets")) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "\n!! Network connections allowed to use Nasal !!\n"
+        SG_LOG(SG_GENERAL, SG_MANDATORY_INFO, "\n!! Network connections allowed to use Nasal !!\n"
        "Network connections will be allowed full access to the simulator \n"
        "including running arbitrary scripts. Ensure you have adequate security\n"
         "(such as a firewall which blocks external connections).\n");
@@ -506,42 +504,91 @@ extern "C" {
 }
 #endif
 
-static void logToHome()
+static void rotateOldLogFiles()
 {
+    const int maxLogCount = 10;
+    const auto homePath = globals->get_fg_home();
+
+    for (int i = maxLogCount; i > 0; --i) {
+        const auto name = "fgfs_" + std::to_string(i - 1) + ".log";
+        SGPath curLogFile = homePath / name;
+        if (curLogFile.exists()) {
+            auto newName = "fgfs_" + std::to_string(i) + ".log";
+            curLogFile.rename(homePath / newName);
+        }
+    }
+
+    SGPath p = homePath / "fgfs.log";
+    if (!p.exists())
+        return;
+    SGPath log0Path = homePath / "fgfs_0.log";
+    p.rename(log0Path);
+}
+
+static void logToHome(const std::string& pri)
+{
+    sgDebugPriority fileLogLevel = SG_INFO;
+    // https://sourceforge.net/p/flightgear/codetickets/2100/
+    if (!pri.empty()) {
+        try {
+            fileLogLevel = std::min(fileLogLevel, logstream::priorityFromString(pri));
+        } catch (std::exception& e) {
+            // let's not worry about this, and just log at INFO
+        }
+    }
+
     SGPath logPath = globals->get_fg_home();
     logPath.append("fgfs.log");
     if (logPath.exists()) {
-        SGPath prevLogPath = globals->get_fg_home();
-        prevLogPath.append("fgfs_0.log");
-        logPath.rename(prevLogPath);
-        // bit strange, we need to restore the correct value of logPath now
-        logPath = globals->get_fg_home();
-        logPath.append("fgfs.log");
+        rotateOldLogFiles();
     }
-    sglog().logToFile(logPath, SG_ALL, SG_INFO);
+
+    sglog().logToFile(logPath, SG_ALL, fileLogLevel);
 }
 
 // Main top level initialization
 int fgMainInit( int argc, char **argv )
 {
-    // set default log level to 'info' for startup, we will revert to a lower
-    // level once startup is done.
-    sglog().setLogLevels( SG_ALL, SG_INFO );
+    sglog().setLogLevels( SG_ALL, SG_WARN );
     sglog().setStartupLoggingEnabled(true);
     
     globals = new FGGlobals;
-    if (!fgInitHome()) {
-        return EXIT_FAILURE;
+    auto initHomeResult = fgInitHome();
+    if (initHomeResult == InitHomeAbort) {
+        flightgear::fatalMessageBoxThenExit("Unable to create lock file",
+                                "Flightgear was unable to create the lock file in FG_HOME");
     }
     
 #if defined(HAVE_QT)
     flightgear::initApp(argc, argv);
 #endif
 
+    // check if the launcher is requested, since it affects config file parsing
+    bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
+    // an Info.plist bundle can't define command line arguments, but it can set
+    // environment variables. This avoids needed a wrapper shell-script on OS-X.
+    showLauncher |= (::getenv("FG_LAUNCHER") != nullptr);
+
+#if defined(HAVE_QT)
+    if (showLauncher && (initHomeResult == InitHomeReadOnly)) {
+// show this message early, if we can
+        auto r = flightgear::showLockFileDialog();
+        if (r == flightgear::LockFileReset) {
+            SG_LOG( SG_GENERAL, SG_MANDATORY_INFO, "Deleting lock file at user request");
+            flightgear::addSentryBreadcrumb("deleting lock-file at user request", "info");
+            fgDeleteLockFile();
+            fgSetBool("/sim/fghome-readonly", false);
+        } else if (r == flightgear::LockFileQuit) {
+            return EXIT_SUCCESS;
+        }
+    }
+#endif
+    
     const bool readOnlyFGHome = fgGetBool("/sim/fghome-readonly");
     if (!readOnlyFGHome) {
         // now home is initialised, we can log to a file inside it
-        logToHome();
+        const auto level = flightgear::Options::getArgValue(argc, argv, "--log-level");
+        logToHome(level);
     }
 
     if (readOnlyFGHome) {
@@ -558,9 +605,6 @@ int fgMainInit( int argc, char **argv )
 #if OSG_VERSION_LESS_THAN(3,4,1)
     SG_LOG(SG_GENERAL, SG_ALERT, "Minimum supported OpenScenegraph is V3.4.1 - currently using " << osgGetVersion() << " This can cause fatal OSG 'final reference count' errors at runtime");
 #endif
-
-    flightgear::addSentryTag("jenkins-build-number", std::to_string(JENKINS_BUILD_NUMBER));
-    flightgear::addSentryTag("osg-version", osgGetVersion());
     
 #ifdef __OpenBSD__
     {
@@ -597,11 +641,6 @@ int fgMainInit( int argc, char **argv )
     upper_case_property("/sim/tower/airport-id");
     upper_case_property("/autopilot/route-manager/input");
 
-// check if the launcher is requested, since it affects config file parsing
-    bool showLauncher = flightgear::Options::checkForArg(argc, argv, "launcher");
-    // an Info.plist bundle can't define command line arguments, but it can set
-    // environment variables. This avoids needed a wrapper shell-script on OS-X.
-    showLauncher |= (::getenv("FG_LAUNCHER") != nullptr);
     if (showLauncher) {
         // to minimise strange interactions when launcher and config files
         // set overlaping options, we disable the default files. Users can
@@ -609,13 +648,6 @@ int fgMainInit( int argc, char **argv )
         flightgear::Options::sharedInstance()->setShouldLoadDefaultConfig(false);
     }
 
-    if (showLauncher && readOnlyFGHome) {
-        // this is perhaps not what the user wanted, let's inform them
-        flightgear::modalMessageBox("Multiple copies of FlightGear",
-                                    "Another copy of FlightGear is already running on this computer, "
-                                    "so this copy will run in read-only mode.");
-    }
-
     // Load the configuration parameters.  (Command line options
     // override config file options.  Config file options override
     // defaults.)
@@ -676,6 +708,19 @@ int fgMainInit( int argc, char **argv )
     SG_LOG(SG_GENERAL, SG_INFO,
            "EmbeddedResourceManager: selected locale '" << locale << "'");
 
+    if (fgGetBool("/sim/autosave-migration/did-migrate", false)) {
+        // inform the user we did migration. This is the earliest point
+        // we can do it, since now the locale is set
+        auto locale = globals->get_locale();
+        const auto title = locale->getLocalizedString("settings-migration-title", "sys", "Settings migrated");
+        const auto msg = locale->getLocalizedString("settings-migration-text", "sys",
+                                                    "Saved settings were migrated from a previous version of FlightGear. "
+                                                    "If you encounter any problems when using the system, try restoring "
+                                                    "the default settings, before reporting a problem. "
+                                                    "Saved settings can affect the appearance, performance and features of the simulator.");
+        flightgear::modalMessageBox(title, msg);
+    }
+
     // Copy the property nodes for the menus added by registered add-ons
     addons::AddonManager::instance()->addAddonMenusToFGMenubar();
 
@@ -695,6 +740,10 @@ int fgMainInit( int argc, char **argv )
     fntInit();
     globals->get_renderer()->preinit();
 
+#if defined(ENABLE_COMPOSITOR)
+    flightgear::addSentryTag("compositor", "yes");
+#endif
+
     if (fgGetBool("/sim/ati-viewport-hack", true)) {
         SG_LOG(SG_GENERAL, SG_WARN, "Enabling ATI/AMD viewport hack");
         flightgear::addSentryTag("ati-viewport-hack", "enabled");
@@ -708,7 +757,8 @@ int fgMainInit( int argc, char **argv )
 
     // pass control off to the master event handler
     int result = fgOSMainLoop();
-    frame_signal.clear();
+    flightgear::unregisterMainLoopProperties();
+
     fgOSCloseWindow();
     fgShutdownHome();
     
