@@ -48,6 +48,7 @@
 #include <Navaids/airways.hxx>
 #include <Autopilot/route_mgr.hxx>
 #include <Aircraft/AircraftPerformance.hxx>
+#include <Main/sentryIntegration.hxx>
 
 using std::string;
 using std::vector;
@@ -78,7 +79,7 @@ namespace flightgear {
 // implemented in route.cxx
 const char* restrictionToString(RouteRestriction aRestrict);
 
-typedef std::vector<FlightPlan::DelegateFactory*> FPDelegateFactoryVec;
+typedef std::vector<FlightPlan::DelegateFactoryRef> FPDelegateFactoryVec;
 static FPDelegateFactoryVec static_delegateFactories;
   
 FlightPlan::FlightPlan() :
@@ -98,7 +99,7 @@ FlightPlan::FlightPlan() :
   for (auto factory : static_delegateFactories) {
     Delegate* d = factory->createFlightPlanDelegate(this);
     if (d) { // factory might not always create a delegate
-      d->_deleteWithPlan = true;
+      d->_factory = factory; // record for clean-up purposes
       addDelegate(d);
     }
   }
@@ -106,10 +107,11 @@ FlightPlan::FlightPlan() :
   
 FlightPlan::~FlightPlan()
 {
-// delete all delegates which we own.
+// clean up delegates
     for (auto d : _delegates) {
-        if (d->_deleteWithPlan) {
-            delete d;
+        if (d->_factory) {
+            auto f = d->_factory;
+            f->destroyFlightPlanDelegate(this,  d);
         }
     }
 }
@@ -125,21 +127,36 @@ FlightPlan* FlightPlan::clone(const string& newIdent) const
   c->setDeparture(_departureRunway);
   
   if (_approach) {
-    c->setApproach(_approach);
+    c->setApproach(_approach, _approachTransition);
   } else if (_destinationRunway) {
     c->setDestination(_destinationRunway);
   } else if (_destination) {
     c->setDestination(_destination);
   }
-  
-  c->setSTAR(_star);
-  c->setSID(_sid);
-  
-// mark data as unchanged since this is a clean plan
+
+  c->setSTAR(_star, _starTransition);
+  c->setSID(_sid, _sidTransition);
+
+  // mark data as unchanged since this is a clean plan
   c->_arrivalChanged = false;
   c->_departureChanged = false;
-    
-// copy legs
+
+  // copy cruise data
+  if (_cruiseFlightLevel > 0) {
+      c->setCruiseFlightLevel(_cruiseFlightLevel);
+  } else if (_cruiseAltitudeFt > 0) {
+      c->setCruiseAltitudeFt(_cruiseAltitudeFt);
+  }
+
+  if (_cruiseAirspeedMach > 0) {
+      c->setCruiseSpeedMach(_cruiseAirspeedMach);
+  } else if (_cruiseAirspeedKnots > 0) {
+      c->setCruiseSpeedKnots(_cruiseAirspeedKnots);
+  }
+
+  c->_didLoadFP = true; // set the loaded flag to give delegates a chance
+
+  // copy legs
   c->_waypointsChanged = true;
   for (int l=0; l < numLegs(); ++l) {
     c->_legs.push_back(_legs[l]->cloneFor(c));
@@ -233,8 +250,32 @@ void FlightPlan::deleteIndex(int aIndex)
   
   unlockDelegates();
 }
-  
-void FlightPlan::clear()
+
+void FlightPlan::clearAll()
+{
+    lockDelegates();
+    _departure.clear();
+    _departureRunway = nullptr;
+    _destinationRunway = nullptr;
+    _destination.clear();
+    _sid.clear();
+    _sidTransition.clear();
+    _star.clear();
+    _starTransition.clear();
+    _approach.clear();
+    _approachTransition.clear();
+    _alternate.clear();
+
+    _cruiseAirspeedMach = 0.0;
+    _cruiseAirspeedKnots = 0;
+    _cruiseFlightLevel = 0;
+    _cruiseAltitudeFt = 0;
+
+    clearLegs();
+    unlockDelegates();
+}
+
+void FlightPlan::clearLegs()
 {
     // some badly behaved CDU implementations call clear on a Nasal timer
     // during startup.
@@ -377,7 +418,7 @@ int FlightPlan::findWayptIndex(const SGGeod& aPos) const
 int FlightPlan::findWayptIndex(const FGPositionedRef aPos) const
 {
   for (int i=0; i<numLegs(); ++i) {
-    if (_legs[i]->waypoint()->source() == aPos) {
+    if (_legs[i]->waypoint()->matches(aPos)) {
       return i;
     }
   }
@@ -473,7 +514,7 @@ void FlightPlan::clearDeparture()
   
 void FlightPlan::setSID(SID* sid, const std::string& transition)
 {
-  if (sid == _sid) {
+  if ((sid == _sid) && (_sidTransition == transition)) {
     return;
   }
   
@@ -526,7 +567,7 @@ void FlightPlan::setDestination(FGAirport* apt)
   _destination = apt;
   _destinationRunway = nullptr;
   clearSTAR();
-  setApproach(nullptr);
+  setApproach(static_cast<Approach*>(nullptr));
   unlockDelegates();
 }
     
@@ -554,7 +595,7 @@ void FlightPlan::clearDestination()
   _destination = nullptr;
   _destinationRunway = nullptr;
   clearSTAR();
-  setApproach(nullptr);
+  setApproach(static_cast<Approach*>(nullptr));
   unlockDelegates();
 }
 
@@ -573,7 +614,7 @@ void FlightPlan::setAlternate(FGAirportRef alt)
 
 void FlightPlan::setSTAR(STAR* star, const std::string& transition)
 {
-  if (_star == star) {
+  if ((_star == star) && (_starTransition == transition)) {
     return;
   }
   
@@ -635,16 +676,17 @@ Transition* FlightPlan::starTransition() const
   
   return _star->findTransitionByName(_starTransition);
 }
-  
-void FlightPlan::setApproach(flightgear::Approach *app)
+
+void FlightPlan::setApproach(flightgear::Approach* app, const std::string& trans)
 {
-  if (_approach == app) {
-    return;
-  }
-  
+    if ((_approach == app) && (trans == _approachTransition)) {
+        return;
+    }
+
   lockDelegates();
   _arrivalChanged = true;
   _approach = app;
+  _approachTransition = trans;
   if (app) {
     // keep runway + airport in sync
     if (_destinationRunway != _approach->runway()) {
@@ -658,6 +700,29 @@ void FlightPlan::setApproach(flightgear::Approach *app)
   unlockDelegates();
 }
 
+void FlightPlan::setApproach(Transition* approachWithTrans)
+{
+    if (!approachWithTrans) {
+        setApproach((Approach*)nullptr);
+        return;
+    }
+
+    if (!Approach::isApproach(approachWithTrans->parent()->type()))
+        throw sg_exception("FlightPlan::setApproach: transition does not belong to an approach");
+
+    setApproach(static_cast<Approach*>(approachWithTrans->parent()),
+                approachWithTrans->ident());
+}
+
+Transition* FlightPlan::approachTransition() const
+{
+    if (!_approach || _approachTransition.empty()) {
+        return nullptr;
+    }
+
+    return _approach->findTransitionByName(_approachTransition);
+}
+
 bool FlightPlan::save(std::ostream& stream) const
 {
     try {
@@ -705,11 +770,9 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
     if (_departure) {
         d->setStringValue("departure/airport", _departure->ident());
         if (_sid) {
-            if (!_sidTransition.empty()) {
-                d->setStringValue("departure/sid", _sidTransition);
-            } else {
-                d->setStringValue("departure/sid", _sid->ident());
-            }
+            d->setStringValue("departure/sid", _sid->ident());
+            if (!_sidTransition.empty())
+                d->setStringValue("departure/sid_trans", _sidTransition);
         }
         
         if (_departureRunway) {
@@ -720,15 +783,15 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
     if (_destination) {
         d->setStringValue("destination/airport", _destination->ident());
         if (_star) {
-            if (!_starTransition.empty()) {
-                d->setStringValue("destination/star", _starTransition);
-            } else {
-                d->setStringValue("destination/star", _star->ident());
-            }
+             d->setStringValue("destination/star", _star->ident());
+             if (!_starTransition.empty())
+                 d->setStringValue("destination/star_trans", _starTransition);
         }
         
         if (_approach) {
             d->setStringValue("destination/approach", _approach->ident());
+            if (!_approachTransition.empty())
+                d->setStringValue("destination/approach_trans", _approachTransition);
         }
         
         if (_destinationRunway) {
@@ -764,6 +827,18 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
     } // of waypoint iteration
 }
 
+static bool anyWaypointsWithFlag(FlightPlan* plan, WayptFlag flag)
+{
+    bool r = false;
+    plan->forEachLeg([&r, flag](FlightPlan::Leg* l) {
+        if (l->waypoint()->flags() & flag) {
+            r = true;
+        }
+    });
+    
+    return r;
+}
+
 bool FlightPlan::load(const SGPath& path)
 {
     if (!path.exists()) {
@@ -787,8 +862,13 @@ bool FlightPlan::load(const SGPath& path)
         // a load, since we assume the flight-plan had it specified already
         // especially, the XML might have a SID/STAR embedded, which we don't
         // want to lose
-        _arrivalChanged = false;
-        _departureChanged = false;
+        
+        // however, we do want to run the normal delegate if no procedure was
+        // defined. We'll use the presence of waypoints tagged to decide
+        const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL);
+        const bool hasDeparture = anyWaypointsWithFlag(this, WPT_DEPARTURE);
+        _arrivalChanged = !hasArrival;
+        _departureChanged = !hasDeparture;
         Status = true;
     } else if (loadPlainTextFormat(path)) { // simple textual list of waypoints
         _arrivalChanged = true;
@@ -802,6 +882,8 @@ bool FlightPlan::load(const SGPath& path)
     
     _cruiseDataChanged = true;
     _waypointsChanged = true;
+    _didLoadFP = true;
+
     unlockDelegates();
     
     return Status;
@@ -837,10 +919,22 @@ bool FlightPlan::load(std::istream &stream)
         Status = false;
     }
     
-    _arrivalChanged = false;
-    _departureChanged = false;
+    // we don't want to re-compute the arrival / departure after
+     // a load, since we assume the flight-plan had it specified already
+     // especially, the XML might have a SID/STAR embedded, which we don't
+     // want to lose
+    
+     // however, we do want to run the normal delegate if no procedure was
+     // defined. We'll use the presence of waypoints tagged to decide
+     const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL);
+     const bool hasDeparture = anyWaypointsWithFlag(this, WPT_DEPARTURE);
+     _arrivalChanged = !hasArrival;
+     _departureChanged = !hasDeparture;
+    
     _cruiseDataChanged = true;
     _waypointsChanged = true;
+    _didLoadFP = true;
+
     unlockDelegates();
     
     return Status;
@@ -946,7 +1040,7 @@ bool FlightPlan::loadGpxFormat(const SGPath& path)
         return false;
     }
 
-    clear();
+    clearAll();
 
     // copy in case we need to modify
     WayptVec wps = gpxVisitor.waypoints();
@@ -975,27 +1069,35 @@ bool FlightPlan::loadGpxFormat(const SGPath& path)
 bool FlightPlan::loadXmlFormat(const SGPath& path)
 {
   SGPropertyNode_ptr routeData(new SGPropertyNode);
+
+  // avoid error reports on user flight-plans
+  flightgear::sentryThreadReportXMLErrors(false);
   try {
     readProperties(path, routeData);
   } catch (sg_exception& e) {
      SG_LOG(SG_NAVAID, SG_ALERT, "Failed to load flight-plan '" << e.getOrigin()
              << "'. " << e.getMessage());
     // XML parsing fails => not a property XML file
+    flightgear::sentryThreadReportXMLErrors(true);
     return false;
   }
 
+  flightgear::sentryThreadReportXMLErrors(true);
+
   if (routeData.valid())
   {
     try {
       int version = routeData->getIntValue("version", 1);
+      bool ok = false;
       if (version == 1) {
-        loadVersion1XMLRoute(routeData);
+        ok = loadVersion1XMLRoute(routeData);
       } else if (version == 2) {
-        loadVersion2XMLRoute(routeData);
+        ok = loadVersion2XMLRoute(routeData);
       } else {
-        throw sg_io_exception("unsupported XML route version");
+        SG_LOG(SG_NAVAID, SG_POPUP, "Unsupported flight plan version " << version << " loading " << path);
       }
-      return true;
+
+      return ok;
     } catch (sg_exception& e) {
       SG_LOG(SG_NAVAID, SG_ALERT, "Failed to load flight-plan '" << e.getOrigin()
              << "'. " << e.getMessage());
@@ -1036,13 +1138,13 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
       }
     
       if (dep->hasChild("sid")) {
+          // previously, we would write a transition id for 'SID' if set,
+          // but this is ambigous. Starting with 2020.2, we only every try
+          // to parse this value as a SID, and look for a seperate sid_trans
+          // value
+          const string trans = dep->getStringValue("sid_trans");
           const auto sid = dep->getStringValue("sid");
-          auto trans = _departure->selectSIDByTransition(sid);
-          if (trans) {
-              setSID(trans);
-          } else {
-              setSID(_departure->findSIDWithIdent(sid));
-          }
+          setSID(_departure->findSIDWithIdent(sid), trans);
       }
     }
   }
@@ -1058,17 +1160,17 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
       }
       
       if (dst->hasChild("star")) {
+          // prior to 2020.2 we would attempt to treat 'star' as a
+          // transiiton ID, but this is ambiguous. Look for a seperate value now
           const auto star = dst->getStringValue("star");
-          auto trans = _destination->selectSTARByTransition(star);
-          if (trans) {
-              setSTAR(trans);
-          } else {
-              setSTAR(_destination->findSTARWithIdent(star));
-          }
-      }
+          const string trans = dst->getStringValue("star_trans");
+          setSTAR(_destination->findSTARWithIdent(star), trans);
+      } // of STAR processing
       
       if (dst->hasChild("approach")) {
-        setApproach(_destination->findApproachWithIdent(dst->getStringValue("approach")));
+          auto app = _destination->findApproachWithIdent(dst->getStringValue("approach"));
+          const auto trans = dst->getStringValue("approach_trans");
+          setApproach(app, trans);
       }
     }
   }
@@ -1095,8 +1197,11 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
   } // of cruise data loading
 }
 
-void FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
+bool FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
 {
+    if (!routeData->hasChild("route"))
+        return false;
+
   loadXMLRouteHeader(routeData);
   
   // route nodes
@@ -1105,6 +1210,10 @@ void FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
   if (routeNode.valid()) {
     for (auto wpNode : routeNode->getChildren("wp")) {
         auto wp = Waypt::createFromProperties(this, wpNode);
+        if (!wp) {
+            continue;
+        }
+
       LegRef l = new Leg{this, wp};
       // sync leg restrictions with waypoint ones
         if (wp->speedRestriction() != RESTRICT_NONE) {
@@ -1122,10 +1231,14 @@ void FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
     } // of route iteration
   }
   _waypointsChanged = true;
+  return true;
 }
 
-void FlightPlan::loadVersion1XMLRoute(SGPropertyNode_ptr routeData)
+bool FlightPlan::loadVersion1XMLRoute(SGPropertyNode_ptr routeData)
 {
+    if (!routeData->hasChild("route"))
+        return false;
+
   loadXMLRouteHeader(routeData);
   
   // _legs nodes
@@ -1137,6 +1250,7 @@ void FlightPlan::loadVersion1XMLRoute(SGPropertyNode_ptr routeData)
     _legs.push_back(l);
   } // of route iteration
   _waypointsChanged = true;
+  return true;
 }
 
 WayptRef FlightPlan::parseVersion1XMLWaypt(SGPropertyNode* aWP)
@@ -1211,9 +1325,17 @@ bool FlightPlan::loadPlainTextFormat(const SGPath& path)
         continue; // ignore empty/comment lines
       }
       
+      // prevent Sentry error 'FLIGHTGEAR-J6', when we try loading XML
+      // data here
+      if (simgear::strutils::starts_with(line, "<?xml")) {
+          return false;
+      }
+
       WayptRef w = waypointFromString(line);
       if (!w) {
-        throw sg_io_exception("Failed to create waypoint from line '" + line + "'.");
+          SG_LOG(SG_NAVAID, SG_ALERT, "Failed to create waypoint from '" << line << "' in " << path);
+          _legs.clear();
+          return false;
       }
       
       _legs.push_back(LegRef{new Leg(this, w)});
@@ -1674,7 +1796,31 @@ SGGeod FlightPlan::pointAlongRoute(int aIndex, double aOffsetNm) const
     RoutePath rp(this);
     return rp.positionForDistanceFrom(aIndex, aOffsetNm * SG_NM_TO_METER);
 }
-    
+
+SGGeod FlightPlan::pointAlongRouteNorm(int aIndex, double aOffsetNorm) const
+{
+    RoutePath rp(this);
+    if (fabs(aOffsetNorm) > 1.0) {
+        SG_LOG(SG_AUTOPILOT, SG_ALERT, "FlightPlan::pointAlongRouteNorm: called with invalid arg:" << aOffsetNorm);
+        return rp.positionForIndex(aIndex);
+    }
+
+    const bool forwards = (aOffsetNorm >= 0.0);
+    double d = 0.0;
+    if (forwards) {
+        d = rp.distanceForIndex(aIndex + 1);
+    } else {
+        d = rp.distanceForIndex(aIndex);
+    }
+
+    // in degenerate cases, just use basic position of index
+    if (d <= 0.0) {
+        return rp.positionForIndex(aIndex);
+    }
+
+    return rp.positionForDistanceFrom(aIndex, d * aOffsetNorm);
+}
+
 void FlightPlan::lockDelegates()
 {
   if (_delegateLock == 0) {
@@ -1696,6 +1842,13 @@ void FlightPlan::unlockDelegates()
     return;
   }
   
+    if (_didLoadFP) {
+        _didLoadFP = false;
+        for (auto d : _delegates) {
+          d->loaded();
+        }
+    }
+    
   if (_departureChanged) {
     _departureChanged = false;
     for (auto d : _delegates) {
@@ -1735,7 +1888,7 @@ void FlightPlan::unlockDelegates()
   --_delegateLock;
 }
   
-void FlightPlan::registerDelegateFactory(DelegateFactory* df)
+void FlightPlan::registerDelegateFactory(DelegateFactoryRef df)
 {
   auto it = std::find(static_delegateFactories.begin(), static_delegateFactories.end(), df);
   if (it != static_delegateFactories.end()) {
@@ -1745,7 +1898,7 @@ void FlightPlan::registerDelegateFactory(DelegateFactory* df)
   static_delegateFactories.push_back(df);
 }
   
-void FlightPlan::unregisterDelegateFactory(DelegateFactory* df)
+void FlightPlan::unregisterDelegateFactory(DelegateFactoryRef df)
 {
   auto it = std::find(static_delegateFactories.begin(), static_delegateFactories.end(), df);
   if (it == static_delegateFactories.end()) {
@@ -2114,5 +2267,71 @@ int FlightPlan::cruiseAltitudeFt() const
 {
     return _cruiseAltitudeFt;
 }
+
+void FlightPlan::forEachLeg(const LegVisitor& lv)
+{
+    std::for_each(_legs.begin(), _legs.end(), lv);
+}
+
+
+int FlightPlan::indexOfFirstNonDepartureWaypoint() const
+{
+    const auto numLegs = _legs.size();
+    for (int i = 0; i < numLegs; ++i) {
+        if (!(_legs.at(i)->waypoint()->flags() & WPT_DEPARTURE))
+            return i ;
+    }
     
+    // all waypoints are marked as departure
+    return -1;
+}
+
+int FlightPlan::indexOfFirstArrivalWaypoint() const
+{
+    const auto numLegs = _legs.size();
+    for (int i = 0; i < numLegs; ++i) {
+        if (_legs.at(i)->waypoint()->flags() & WPT_ARRIVAL)
+            return i;
+    }
+    
+    // no waypoints are marked as arrival
+    return -1;
+}
+
+int FlightPlan::indexOfFirstApproachWaypoint() const
+{
+    const auto numLegs = _legs.size();
+    for (int i = 0; i < numLegs; ++i) {
+        if (_legs.at(i)->waypoint()->flags() & WPT_APPROACH)
+            return i;
+    }
+    
+    // no waypoints are marked as arrival
+    return -1;
+}
+
+int FlightPlan::indexOfDestinationRunwayWaypoint() const
+{
+    if (!_destinationRunway)
+        return -1;
+    
+    // work backwards in case the departure and destination match
+    // this way we'll find the one we want
+    for (int i = numLegs() - 1; i >= 0; i--) {
+        if (_legs.at(i)->waypoint()->source() == _destinationRunway) {
+            return i;
+        }
+    }
+    
+    return -1;
+}
+
+void FlightPlan::DelegateFactory::destroyFlightPlanDelegate(FlightPlan* fp, Delegate* d)
+{
+    // mimic old behaviour before destroyFlightPlanDelegate was added
+    SG_UNUSED(fp);
+    delete d;
+}
+
+
 } // of namespace flightgear
