@@ -46,6 +46,14 @@
   #include "fg_sqlite3.h"
 #endif
 
+#if defined(SG_WINDOWS)
+#define WIN32_LEAN_AND_MEAN // less crap :)
+#include <Windows.h>
+#else
+#  include <sys/file.h>
+#include <unistd.h>
+#endif
+
 // SimGear
 #include <simgear/sg_inlines.h>
 #include <simgear/structure/exception.hxx>
@@ -56,26 +64,26 @@
 #include <simgear/misc/strutils.hxx>
 #include <simgear/threads/SGThread.hxx>
 
-#include <Main/globals.hxx>
-#include <Main/fg_props.hxx>
-#include <Main/options.hxx>
+#include "CacheSchema.h"
+#include "PositionedOctree.hxx"
+#include "fix.hxx"
 #include "markerbeacon.hxx"
 #include "navrecord.hxx"
-#include <Airports/airport.hxx>
-#include <Airports/runways.hxx>
+#include "poidb.hxx"
 #include <ATC/CommStation.hxx>
-#include "fix.hxx"
+#include <Airports/airport.hxx>
+#include <Airports/apt_loader.hxx>
+#include <Airports/gnnode.hxx>
+#include <Airports/parking.hxx>
+#include <Airports/runways.hxx>
+#include <GUI/MessageBox.hxx>
+#include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/options.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Navaids/airways.hxx>
 #include <Navaids/fixlist.hxx>
 #include <Navaids/navdb.hxx>
-#include "PositionedOctree.hxx"
-#include <Airports/apt_loader.hxx>
-#include <Navaids/airways.hxx>
-#include "poidb.hxx"
-#include <Airports/parking.hxx>
-#include <Airports/gnnode.hxx>
-#include "CacheSchema.h"
-#include <GUI/MessageBox.hxx>
-#include <Navaids/airways.hxx>
 
 using std::string;
 
@@ -230,9 +238,27 @@ public:
                const string& ident, const SGGeod& pos) :
     FGPositioned(guid, FGPositioned::TOWER, ident, pos)
   {
+      SG_UNUSED(airport);
   }
 };
 
+// useful for debugging 'hanging' queries: look for a statement
+// which starts but never completes
+#if 0
+int traceCallback(unsigned int traceCode, void* ctx, void* p, void* x)
+{
+    if (traceCode == SQLITE_TRACE_STMT) {
+        SG_LOG(SG_NAVCACHE, SG_WARN, "step:" << p << " text=" << (char*)x);
+    }
+    else if (traceCode == SQLITE_TRACE_PROFILE) {
+        int64_t* nanoSecs = (int64_t*)x;
+        SG_LOG(SG_NAVCACHE, SG_WARN, "profile:" << p << " took " << *nanoSecs);
+    }
+
+    return 0;
+}
+#endif
+
 class NavDataCache::NavDataCachePrivate
 {
 public:
@@ -264,8 +290,12 @@ public:
           throw sg_exception("Nav-cache file is not writeable");
       }
 
+      if (outer->isAnotherProcessRebuilding()) {
+          SG_LOG(SG_NAVCACHE, SG_ALERT, "NavDataCache: init() while another processing is rebuilding the DB");
+      }
+
       int openFlags = readOnly ? SQLITE_OPEN_READONLY :
-        SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE;
+        (SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE);
       std::string pathUtf8 = path.utf8Str();
       int result = sqlite3_open_v2(pathUtf8.c_str(), &db, openFlags, NULL);
       if (result == SQLITE_MISUSE) {
@@ -282,6 +312,9 @@ public:
           throw sg_exception("Nav-cache file opened but is not writeable");
       }
 
+  // enable tracing for debugging stuck queries
+   //   sqlite3_trace_v2(db, SQLITE_TRACE_STMT | SQLITE_TRACE_PROFILE, traceCallback, this);
+
     sqlite3_stmt_ptr checkTables =
       prepare("SELECT count(*) FROM sqlite_master WHERE name='properties'");
 
@@ -296,6 +329,8 @@ public:
       didCreate = true;
     }
 
+    reset(checkTables);
+
     readPropertyQuery = prepare("SELECT value FROM properties WHERE key=?");
     writePropertyQuery = prepare("INSERT INTO properties (key, value) VALUES (?,?)");
     clearProperty = prepare("DELETE FROM properties WHERE key=?1");
@@ -463,6 +498,7 @@ public:
   {
     if (!execSelect(stmt)) {
       SG_LOG(SG_NAVCACHE, SG_WARN, "empty SELECT running:\n\t" << sqlite3_sql(stmt));
+      flightgear::sentryReportException("empty SELECT running:" + std::string{sqlite3_sql(stmt)});
       throw sg_exception("no results returned for select", sqlite3_sql(stmt));
     }
   }
@@ -697,16 +733,18 @@ public:
                         const SGGeod& pos,
                         PositionedID airport)
   {
-    sqlite3_bind_int64(loadCommStation, 1, rowId);
-    execSelect1(loadCommStation);
+      SG_UNUSED(id);
 
-    int range = sqlite3_column_int(loadCommStation, 0);
-    int freqKhz = sqlite3_column_int(loadCommStation, 1);
-    reset(loadCommStation);
+      sqlite3_bind_int64(loadCommStation, 1, rowId);
+      execSelect1(loadCommStation);
 
-    CommStation* c = new CommStation(rowId, name, ty, pos, freqKhz, range);
-    c->setAirport(airport);
-    return c;
+      int range = sqlite3_column_int(loadCommStation, 0);
+      int freqKhz = sqlite3_column_int(loadCommStation, 1);
+      reset(loadCommStation);
+
+      CommStation* c = new CommStation(rowId, name, ty, pos, freqKhz, range);
+      c->setAirport(airport);
+      return c;
   }
 
   FGPositioned* loadNav(sqlite3_int64 rowId,
@@ -1045,27 +1083,31 @@ void NavDataCache::NavDataCachePrivate::findDatFiles(
   NavDataCache::DatFileType datFileType)
 {
   NavDataCache::DatFilesGroupInfo result;
-  SGPath visitedPath;           // to avoid duplicates and time wasting
-  const string datFilesSubDir = "NavData/" +
-                               NavDataCache::datTypeStr[datFileType];
-  const PathList& sceneryPaths = globals->get_fg_scenery();
   result.datFileType = datFileType;
   result.totalSize = 0;
 
-  for (PathList::const_iterator dirsIt = sceneryPaths.begin();
-       dirsIt != sceneryPaths.end(); dirsIt++) {
-    if (dirsIt->isNull() || *dirsIt == visitedPath) {
-      continue;                 // duplicate or empty path
-    } else if (! dirsIt->isDir()) {
-      SG_LOG(SG_NAVCACHE, SG_WARN, *dirsIt <<
+  // we don't always add FG_ROOT/Scenery to the path list. But if it exists,
+  // we want to pick up NavData files from it, since we have shipped
+  // scenery (for BIKF) which uses newer data than the default files
+  // in Airports/
+  auto paths = globals->get_fg_scenery();
+  const auto fgrootScenery = globals->get_fg_root() / "Scenery";
+  if (fgrootScenery.exists()) {
+    auto it = std::find(paths.begin(), paths.end(), fgrootScenery);
+    if (it == paths.end()) {
+       paths.push_back(fgrootScenery);
+    }
+  }
+
+  for (const auto& path: paths) {
+    if (! path.isDir()) {
+      SG_LOG(SG_NAVCACHE, SG_WARN, path <<
              ": given as a scenery path, but is not a directory");
-      visitedPath = *dirsIt;    // to avoid duplicate log messages
       continue;
     }
 
-    visitedPath = *dirsIt;
-    SGPath datFilesDir = visitedPath;
-    datFilesDir.append(datFilesSubDir);
+    const SGPath datFilesDir =
+      path / "NavData" / NavDataCache::datTypeStr[datFileType];
 
     if (datFilesDir.isDir()) {
       // Deterministic because the return value of simgear::Dir::children() is
@@ -1073,7 +1115,7 @@ void NavDataCache::NavDataCachePrivate::findDatFiles(
       const PathList files = simgear::Dir(datFilesDir).children(
         simgear::Dir::TYPE_FILE | simgear::Dir::NO_DOT_OR_DOTDOT);
 
-      for (auto f : files) {
+      for (const auto& f : files) {
         const std::string name = f.file();
         if (simgear::strutils::ends_with(name, ".dat") ||
             simgear::strutils::ends_with(name, ".dat.gz")) {
@@ -1082,7 +1124,7 @@ void NavDataCache::NavDataCachePrivate::findDatFiles(
         }
       }
     }
-  } // of loop over 'sceneryPaths'
+  } // of loop over the list of scenery paths
 
   // Add the default file (e.g., $FG_ROOT/Airports/apt.dat.gz), at least for
   // now
@@ -1212,14 +1254,13 @@ NavDataCache::NavDataCache()
         os << "navdata_" << versionParts[0] << "_" << versionParts[1] << ".cache";
     }
 
-    homePath.append(os.str());
-
     // permit additional DB connections from the same process
     sqlite3_config(SQLITE_CONFIG_MULTITHREAD);
 
     for (int t=0; t < MAX_TRIES; ++t) {
+        SGPath cachePath = homePath / os.str();
         try {
-            d.reset(new NavDataCachePrivate(homePath, this));
+            d.reset(new NavDataCachePrivate(cachePath, this));
             d->init();
             //d->checkCacheFile();
             // reached this point with no exception, success
@@ -1239,14 +1280,14 @@ NavDataCache::NavDataCache()
             d.reset();
 
             // only wipe the existing if not readonly
-            if (!fgGetBool("/sim/fghome-readonly", false)) {
-                bool ok = homePath.remove();
+            if (cachePath.exists() && !fgGetBool("/sim/fghome-readonly", false)) {
+                bool ok = cachePath.remove();
                 if (!ok) {
                     SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache: failed to remove previous cache file");
                     flightgear::fatalMessageBoxThenExit(
                       "Unable to re-create navigation cache",
                       "Attempting to remove the old cache failed.",
-                      "Location: "  + homePath.utf8Str());
+                      "Location: "  + cachePath.utf8Str());
                 }
             }
         }
@@ -1354,9 +1395,94 @@ bool NavDataCache::isRebuildRequired()
   return false;
 }
 
+
+#if defined(SG_WINDOWS)
+static HANDLE static_fgNavCacheRebuildMutex = nullptr;
+#else
+const std::string static_rebuildLockFile = "fgfs_cache_rebuild.lock";
+static int static_rebuildLockFileFd = -1;
+#endif
+
+enum RebuildLockStatus {
+    RebuildLockFailed = 0,
+    RebuildLockAlreadyLocked,
+    RebuildLockOk
+};
+
+RebuildLockStatus accquireRebuildLock()
+{
+#if defined(SG_WINDOWS)
+    if (static_fgNavCacheRebuildMutex == nullptr) {
+        // avoid multiple copies racing on the nav-cache build
+        static_fgNavCacheRebuildMutex = CreateMutexA(nullptr, FALSE, "org.flightgear.fgfs.rebuild-navcache");
+        if (static_fgNavCacheRebuildMutex == nullptr) {
+            SG_LOG(SG_IO, SG_ALERT, "Failed to create NavCache rebuild mutex");
+            return RebuildLockFailed;
+        }
+
+        if (GetLastError() == ERROR_ALREADY_EXISTS) {
+            return RebuildLockAlreadyLocked;
+        }
+
+        // accquire the mutex, so that other processes can check the status.
+        const int result = WaitForSingleObject(static_fgNavCacheRebuildMutex, 100);
+        if (result != WAIT_OBJECT_0) {
+            SG_LOG(SG_IO, SG_ALERT, "Failed to lock NavCache rebuild mutex:" << GetLastError());
+            return RebuildLockFailed;
+        }
+    }
+#else
+    SGPath lockPath(globals->get_fg_home(), static_rebuildLockFile);
+    std::string ps = lockPath.utf8Str();
+    static_rebuildLockFileFd = ::open(ps.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0644);
+    if (static_rebuildLockFileFd < 0) {
+        SG_LOG(SG_IO, SG_ALERT, "Failed to create rebuild lock file (" << lockPath << "):" << simgear::strutils::error_string(errno));
+        return RebuildLockFailed;
+    }
+    int err = ::flock(static_rebuildLockFileFd, LOCK_EX | LOCK_NB);
+    if (err < 0) {
+        if (errno == EWOULDBLOCK) {
+            return RebuildLockAlreadyLocked;
+        }
+
+        SG_LOG(SG_IO, SG_ALERT, "Failed to lock file (" << lockPath << "):" << simgear::strutils::error_string(errno));
+        return RebuildLockFailed;
+    }
+
+#endif
+    return RebuildLockOk;
+}
+
+static void releaseRebuildLock()
+{
+#if defined(SG_WINDOWS)
+    ReleaseMutex(static_fgNavCacheRebuildMutex);
+#else
+    int err = ::flock(static_rebuildLockFileFd, LOCK_UN);
+    if (err < 0) {
+        SG_LOG(SG_IO, SG_ALERT, "Failed to unlock rebuild file:" << simgear::strutils::error_string(errno));
+    }
+
+    ::close(static_rebuildLockFileFd);
+    SGPath lockPath(globals->get_fg_home(), static_rebuildLockFile);
+    lockPath.remove();
+#endif
+}
+
+
 NavDataCache::RebuildPhase NavDataCache::rebuild()
 {
     if (!d->rebuilder.get()) {
+        auto r = accquireRebuildLock();
+        if (r == RebuildLockAlreadyLocked) {
+            flightgear::fatalMessageBoxThenExit("Multiple copies of Flightgear initializing",
+                                                "Multiple copies of FlightGear are trying to initialise the same navigation database. "
+                                                "This means something has gone badly wrong: please report this error.");
+        } else if (r == RebuildLockFailed) {
+            flightgear::fatalMessageBoxThenExit("Multiple copies of Flightgear initializing",
+                                                "Failed to initialise NavCache rebuild protection");
+        }
+
         d->rebuilder.reset(new RebuildThread(this));
         d->rebuilder->start();
     }
@@ -1365,10 +1491,71 @@ NavDataCache::RebuildPhase NavDataCache::rebuild()
     RebuildPhase phase = d->rebuilder->currentPhase();
     if (phase == REBUILD_DONE) {
         d->rebuilder.reset(); // all done!
+        releaseRebuildLock();
     }
     return phase;
 }
 
+bool NavDataCache::isAnotherProcessRebuilding()
+{
+#if defined(SG_WINDOWS)
+    if (!static_fgNavCacheRebuildMutex) {
+        static_fgNavCacheRebuildMutex = OpenMutexA(SYNCHRONIZE, FALSE, "org.flightgear.fgfs.rebuild-navcache");
+        if (!static_fgNavCacheRebuildMutex) {
+            // this is the common case: no other fgfs.exe is doing a rebuild, so
+            // the mutex does not exist. Simple, we are done
+            if (GetLastError() == ERROR_FILE_NOT_FOUND) {
+                return false;
+            }
+
+            flightgear::fatalMessageBoxThenExit("Multiple copies of Flightgear initializing",
+                                                "Unable to check if other copies of FlightGear are initializing. "
+                                                "Please report this error.");
+        }
+    }
+
+    // poll the named mutex
+    auto result = WaitForSingleObject(static_fgNavCacheRebuildMutex, 0);
+    if (result == WAIT_OBJECT_0) {
+        // we accquired it, release it and we're done
+        // (there could be multiple read-only copies in this situation)
+        ReleaseMutex(static_fgNavCacheRebuildMutex);
+        CloseHandle(static_fgNavCacheRebuildMutex);
+        return false;
+    }
+
+    // failed to accquire the mutex, so assume another FGFS.exe is rebuilding,
+    // the GU should wait.
+    return true;
+#else
+    SGPath lockPath(globals->get_fg_home(), static_rebuildLockFile);
+    std::string ps = lockPath.utf8Str();
+    static_rebuildLockFileFd = ::open(ps.c_str(), O_RDONLY, 0644);
+    if (static_rebuildLockFileFd < 0) {
+        if (errno == ENOENT) {
+            return false; // no such file, easy
+        }
+
+        SG_LOG(SG_IO, SG_ALERT, "Error opening lock file:" << simgear::strutils::error_string(errno));
+        return false;
+    }
+
+    int err = ::flock(static_rebuildLockFileFd, LOCK_EX | LOCK_NB);
+    if (err < 0) {
+        if (errno == EWOULDBLOCK) {
+            return true;
+        }
+
+        SG_LOG(SG_IO, SG_ALERT, "Error querying lock file:" << simgear::strutils::error_string(errno));
+        return false;
+    }
+
+    // release it again, so any *other* waiting copies can also succeed
+    ::flock(static_rebuildLockFileFd, LOCK_UN);
+    return false;
+#endif
+}
+
 unsigned int NavDataCache::rebuildPhaseCompletionPercentage() const
 {
     if (!d->rebuilder.get()) {
@@ -1466,7 +1653,7 @@ void NavDataCache::doRebuild()
     }
 
 #ifdef SG_WINDOWS
-      SG_LOG(SG_NAVCACHE, SG_ALERT, "SKIPPING POI load on Windows");
+      SG_LOG(SG_NAVCACHE, SG_MANDATORY_INFO, "SKIPPING POI load on Windows");
 #else
       {
           Transaction txn(this);
@@ -1689,6 +1876,7 @@ void NavDataCache::commitTransaction()
       // it's active, the DB was rolled-back
       if (sqlite3_get_autocommit(d->db)) {
         SG_LOG(SG_NAVCACHE, SG_ALERT, "commit: was rolled back!" << retries);
+        flightgear::sentryReportException("DB commit was rolled back");
         d->transactionAborted = true;
         break;
       }
@@ -1697,13 +1885,14 @@ void NavDataCache::commitTransaction()
         break;
       }
 
-      SGTimeStamp::sleepForMSec(++retries * 10);
+      SGTimeStamp::sleepForMSec(++retries * 100);
       SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache contention on commit, will retry:" << retries);
     } // of retry loop for DB busy
 
     string errMsg;
     if (result != SQLITE_DONE) {
       errMsg = sqlite3_errmsg(d->db);
+      flightgear::sentryReportException("DB error:" + errMsg + " running " + std::string{sqlite3_sql(q)});
       SG_LOG(SG_NAVCACHE, SG_ALERT, "Sqlite error:" << errMsg << " for  " << result
              << " while running:\n\t" << sqlite3_sql(q));
     }
@@ -1715,6 +1904,7 @@ void NavDataCache::commitTransaction()
 void NavDataCache::abortTransaction()
 {
   SG_LOG(SG_NAVCACHE, SG_WARN, "NavCache: aborting transaction");
+  flightgear::sentryReportException("DB aborting transactino");
 
   assert(d->transactionLevel > 0);
   if (--d->transactionLevel == 0) {
@@ -1725,6 +1915,16 @@ void NavDataCache::abortTransaction()
   d->transactionAborted = true;
 }
 
+void NavDataCache::clearDynamicPositioneds()
+{
+    std::for_each(d->cache.begin(), d->cache.end(), [](PositionedCache::value_type& v) {
+        if (v.second->type() == FGPositioned::MOBILE_TACAN) {
+            auto mobile = fgpositioned_cast<FGMobileNavRecord>(v.second);
+            mobile->clearVehicle();
+        }
+    });
+}
+
 FGPositionedRef NavDataCache::loadById(PositionedID rowid)
 {
   if (rowid < 1) {
@@ -1987,8 +2187,15 @@ FGPositionedRef NavDataCache::findClosestWithIdent( const string& aIdent,
 
 int NavDataCache::getOctreeBranchChildren(int64_t octreeNodeId)
 {
-  sqlite3_bind_int64(d->getOctreeChildren, 1, octreeNodeId);
-  d->execSelect1(d->getOctreeChildren);
+    sqlite3_bind_int64(d->getOctreeChildren, 1, octreeNodeId);
+    if (!d->execSelect(d->getOctreeChildren)) {
+        // this can occur when in read-only mode: we don't add
+        // new Octree nodes to the real DB (only in memory),
+        // but will still call this code speculatively.
+        // see the early-return just below in defineOctreeNode
+        return 0;
+    }   
+
   int children = sqlite3_column_int(d->getOctreeChildren, 0);
   d->reset(d->getOctreeChildren);
   return children;
@@ -1996,6 +2203,10 @@ int NavDataCache::getOctreeBranchChildren(int64_t octreeNodeId)
 
 void NavDataCache::defineOctreeNode(Octree::Branch* pr, Octree::Node* nd)
 {
+  if (isReadOnly()) {
+    return;
+  }
+  
   sqlite3_bind_int64(d->insertOctree, 1, nd->guid());
   d->execInsert(d->insertOctree);
 
@@ -2219,7 +2430,8 @@ NavDataCache::findAirportRunway(const std::string& aName)
 
   AirportRunwayPair result;
   sqlite_bind_stdstring(d->findAirportRunway, 1, parts[0]);
-  sqlite_bind_stdstring(d->findAirportRunway, 2, cleanRunwayNo(parts[1]));
+  const auto cleanedRunway = cleanRunwayNo(parts[1]);
+  sqlite_bind_stdstring(d->findAirportRunway, 2, cleanedRunway);
 
   if (d->execSelect(d->findAirportRunway)) {
     result = AirportRunwayPair(sqlite3_column_int64(d->findAirportRunway, 0),
@@ -2252,13 +2464,15 @@ NavDataCache::findILS(PositionedID airport, const string& aRunway, const string&
 
 int NavDataCache::findAirway(int network, const string& aName, bool create)
 {
-  sqlite3_bind_int(d->findAirwayNet, 1, network);
-  sqlite_bind_stdstring(d->findAirwayNet, 2, aName);
+    assert((network == 1) || (network == 2));
 
-  int airway = 0;
-  if (d->execSelect(d->findAirwayNet)) {
-    // already exists
-    airway = sqlite3_column_int(d->findAirwayNet, 0);
+    sqlite3_bind_int(d->findAirwayNet, 1, network);
+    sqlite_bind_stdstring(d->findAirwayNet, 2, aName);
+
+    int airway = 0;
+    if (d->execSelect(d->findAirwayNet)) {
+        // already exists
+        airway = sqlite3_column_int(d->findAirwayNet, 0);
   } else if (create) {
     d->reset(d->insertAirway);
     sqlite_bind_stdstring(d->insertAirway, 1, aName);
@@ -2497,11 +2711,17 @@ NavDataCache::Transaction::Transaction(NavDataCache* cache) :
     _committed(false)
 {
     assert(cache);
-    _instance->beginTransaction();
+    if (!cache->isReadOnly()) {
+      _instance->beginTransaction();
+    }
 }
 
 NavDataCache::Transaction::~Transaction()
 {
+    if (_instance->isReadOnly()) {
+        return;
+    }
+
     if (!_committed) {
         SG_LOG(SG_NAVCACHE, SG_INFO, "aborting cache transaction!");
         _instance->abortTransaction();
@@ -2510,6 +2730,10 @@ NavDataCache::Transaction::~Transaction()
 
 void NavDataCache::Transaction::commit()
 {
+    if (_instance->isReadOnly()) {
+        return;
+    }
+        
     assert(!_committed);
     _committed = true;
     _instance->commitTransaction();
