@@ -166,73 +166,55 @@ public:
  *
  * Mongoose API is documented here: http://cesanta.com/docs/API.shtml
  */
-class MongooseHttpd: public FGHttpd {
+class MongooseHttpd : public FGHttpd
+{
 public:
+    /**
+     * Construct a MongooseHttpd object from options in a PropertyNode
+     */
+    MongooseHttpd(SGPropertyNode_ptr);
 
-  /**
-   * Construct a MongooseHttpd object from options in a PropertyNode
-   */
-  MongooseHttpd(SGPropertyNode_ptr);
+    /**
+     * Cleanup et.al.
+     */
+    ~MongooseHttpd();
 
-  /**
-   * Cleanup et.al.
-   */
-  ~MongooseHttpd();
+    // Subsystem API.
+    void bind() override;            // Currently a noop
+    void init() override;            // Reads the configuration PropertyNode, installs URIHandlers and configures mongoose
+    void unbind() override;          // shutdown of mongoose, clear connections, unregister URIHandlers
+    void update(double dt) override; // poll connections, check for changed properties
 
-  /**
-   * override SGSubsystem::init()
-   *
-   * Reads the configuration PropertyNode, installs URIHandlers and configures mongoose
-   */
-  void init();
+    // Subsystem identification.
+    static const char* staticSubsystemClassId() { return "mongoose-httpd"; }
 
-  /**
-   * override SGSubsystem::bind()
-   *
-   * Currently a noop
-   */
-  void bind();
+    /**
+     * Returns a URIHandler for the given uri
+     *
+     * @see URIHandlerMap::findHandler( const std::string & uri )
+     */
+    SGSharedPtr<URIHandler> findHandler(const std::string & uri) {
+        return _uriHandler.findHandler(uri);
+    }
 
-  /**
-   * override SGSubsystem::unbind()
-   * shutdown of mongoose, clear connections, unregister URIHandlers
-   */
-  void unbind();
-
-  /**
-   * overrride SGSubsystem::update()
-   * poll connections, check for changed properties
-   */
-  void update(double dt);
-
-  /**
-   * Returns a URIHandler for the given uri
-   *
-   * @see URIHandlerMap::findHandler( const std::string & uri )
-   */
-  SGSharedPtr<URIHandler> findHandler(const std::string & uri)
-  {
-    return _uriHandler.findHandler(uri);
-  }
-
-  Websocket * newWebsocket(const string & uri);
+    Websocket * newWebsocket(const string & uri);
 
 private:
-  int poll(struct mg_connection * connection);
-  int auth(struct mg_connection * connection);
-  int request(struct mg_connection * connection);
-  int onConnect(struct mg_connection * connection);
-  void close(struct mg_connection * connection);
+    int poll(struct mg_connection * connection);
+    int auth(struct mg_connection * connection);
+    int request(struct mg_connection * connection);
+    int onConnect(struct mg_connection * connection);
+    void close(struct mg_connection * connection);
 
-  static int staticRequestHandler(struct mg_connection *, mg_event event);
+    static int staticRequestHandler(struct mg_connection *, mg_event event);
 
-  struct mg_server *_server;
-  SGPropertyNode_ptr _configNode;
+    struct mg_server *_server;
+    SGPropertyNode_ptr _configNode;
 
-  typedef int (MongooseHttpd::*handler_t)(struct mg_connection *);
-  URIHandlerMap _uriHandler;
+    typedef int (MongooseHttpd::*handler_t)(struct mg_connection *);
+    URIHandlerMap _uriHandler;
 
-  PropertyChangeObserver _propertyChangeObserver;
+    PropertyChangeObserver _propertyChangeObserver;
 };
 
 class MongooseConnection: public Connection {
@@ -426,8 +408,13 @@ int WebsocketConnection::onConnect(struct mg_connection * connection)
 int WebsocketConnection::request(struct mg_connection * connection)
 {
   setConnection(connection);
+  if ((connection->wsbits & 0x0f) >= 0x8) {
+    // control opcode (close/ping/pong)
+    return MG_MORE;
+  }
+
   MongooseHTTPRequest request(connection);
-  SG_LOG(SG_NETWORK, SG_INFO, "WebsocketConnection::request for " << request.Uri);
+  SG_LOG(SG_NETWORK, SG_DEBUG, "WebsocketConnection::request for " << request.Uri);
 
   if ( NULL == _websocket) {
     SG_LOG(SG_NETWORK, SG_ALERT, "httpd: unhandled websocket uri: " << request.Uri);
@@ -655,5 +642,11 @@ FGHttpd * FGHttpd::createInstance(SGPropertyNode_ptr configNode)
   return new MongooseHttpd(configNode);
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<MongooseHttpd> registrantMongooseHttpd;
+#endif
+
 } // namespace http
 } // namespace flightgear
