@@ -1,7 +1,20 @@
+// Copyright (C) 2013  James Turner
+//
+// This program is free software; you can redistribute it and/or
+// modify it under the terms of the GNU General Public License as
+// published by the Free Software Foundation; either version 2 of the
+// License, or (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
 
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
@@ -41,17 +54,19 @@
 #include <simgear/timing/timestamp.hxx>
 
 #include "NasalAddons.hxx"
-#include "NasalSGPath.hxx"
-#include "NasalSys.hxx"
-#include "NasalSys_private.hxx"
 #include "NasalAircraft.hxx"
-#include "NasalModelData.hxx"
-#include "NasalPositioned.hxx"
 #include "NasalCanvas.hxx"
 #include "NasalClipboard.hxx"
 #include "NasalCondition.hxx"
+#include "NasalFlightPlan.hxx"
 #include "NasalHTTP.hxx"
+#include "NasalModelData.hxx"
+#include "NasalPositioned.hxx"
+#include "NasalSGPath.hxx"
 #include "NasalString.hxx"
+#include "NasalSys.hxx"
+#include "NasalSys_private.hxx"
+#include "NasalUnitTesting.hxx"
 
 #include <Main/globals.hxx>
 #include <Main/util.hxx>
@@ -64,7 +79,12 @@ using std::vector;
 
 void postinitNasalGUI(naRef globals, naContext c);
 
-static FGNasalSys* nasalSys = 0;
+static FGNasalSys* nasalSys = nullptr;
+
+// this is used by the test-suite to simplify
+// how much Nasal modules we load by default
+bool global_nasalMinimalInit = false;
+
 
 // Listener class for loading Nasal modules on demand
 class FGNasalModuleListener : public SGPropertyChangeListener
@@ -252,7 +272,7 @@ typedef nasal::Ghost<TimeStampObjRef> NasalTimeStampObj;
 FGNasalSys::FGNasalSys() :
     _inited(false)
 {
-    nasalSys = this;
+     nasalSys = this;
     _context = 0;
     _globals = naNil();
     _string = naNil();
@@ -484,8 +504,9 @@ static naRef f_setprop(naContext c, naRef me, int argc, naRef* args)
 }
 
 // print() extension function.  Concatenates and prints its arguments
-// to the FlightGear log.  Uses the highest log level (SG_ALERT), to
-// make sure it appears.  Is there better way to do this?
+// to the FlightGear log.  Uses the mandatory log level (SG_MANDATORY_INFO), to
+// make sure it appears.  Users should be converted to use logprint() instead,
+// and specify a level explicitly.
 static naRef f_print(naContext c, naRef me, int argc, naRef* args)
 {
     string buf;
@@ -495,7 +516,24 @@ static naRef f_print(naContext c, naRef me, int argc, naRef* args)
         if(naIsNil(s)) continue;
         buf += naStr_data(s);
     }
-    SG_LOG(SG_NASAL, SG_ALERT, buf);
+ 
+    /* Copy what SG_LOG() does, but use nasal file:line instead of
+    our own __FILE__ and __LINE__. */
+    if (sglog().would_log(SG_NASAL, SG_MANDATORY_INFO)) {
+        int frame = 0;
+        const char* file = naStr_data(naGetSourceFile(c, 0));
+        if (simgear::strutils::ends_with( file, "/globals.nas")) {
+            /* This generally means have been called by globals.nas's
+            printf function; go one step up the stack so we give the
+            file:line of the caller of printf, which is generally more
+            useful. */
+            frame += 1;
+            file = naStr_data(naGetSourceFile(c, frame));
+        }
+        int line = naGetLine(c, frame);
+        sglog().logCopyingFilename(SG_NASAL, SG_MANDATORY_INFO, file, line, buf);
+    }
+
     return naNum(buf.length());
 }
 
@@ -517,7 +555,7 @@ static naRef f_logprint(naContext c, naRef me, int argc, naRef* args)
   }
 // use the nasal source file and line for the message location, since
 // that's more useful than the location here!
-  sglog().log(SG_NASAL, (sgDebugPriority)(int) priority.num,
+  sglog().logCopyingFilename(SG_NASAL, (sgDebugPriority)(int) priority.num,
                naStr_data(naGetSourceFile(c, 0)),
                naGetLine(c, 0), buf);
   return naNum(buf.length());
@@ -570,6 +608,28 @@ static naRef f_makeTimer(naContext c, naRef me, int argc, naRef* args)
   return nasal::to_nasal(c, timerObj);
 }
 
+static naRef f_makeSingleShot(naContext c, naRef me, int argc, naRef* args)
+{
+    if (!naIsNum(args[0])) {
+        naRuntimeError(c, "bad interval argument to makesingleshot");
+    }
+
+    naRef func, self = naNil();
+    if (naIsFunc(args[1])) {
+        func = args[1];
+    } else if ((argc == 3) && naIsFunc(args[2])) {
+        self = args[1];
+        func = args[2];
+    } else {
+        naRuntimeError(c, "bad function/self arguments to makesingleshot");
+    }
+
+    auto timerObj = new TimerObj(c, nasalSys, func, self, args[0].num);
+    timerObj->setSingleShot(true);
+    timerObj->start();
+    return nasal::to_nasal(c, timerObj);
+}
+
 static naRef f_maketimeStamp(naContext c, naRef me, int argc, naRef* args)
 {
     TimeStampObj* timeStampObj = new TimeStampObj(c);
@@ -697,7 +757,7 @@ static naRef f_findDataDir(naContext c, naRef me, int argc, naRef* args)
     if(argc != 1 || !naIsString(args[0]))
         naRuntimeError(c, "bad arguments to findDataDir()");
 
-    SGPath p = globals->find_data_dir(naStr_data(args[0]));
+    SGPath p = globals->findDataPath(naStr_data(args[0]));
     std::string pdata = p.utf8Str();
     return naStr_fromdata(naNewString(c), const_cast<char*>(pdata.c_str()), pdata.length());
 }
@@ -751,8 +811,12 @@ static naRef f_removeCommand(naContext c, naRef me, int argc, naRef* args)
     if ((argc < 1) || !naIsString(args[0]))
         naRuntimeError(c, "bad argument to removecommand()");
 
-    globals->get_commands()->removeCommand(naStr_data(args[0]));
-    return naNil();
+    const string commandName(naStr_data(args[0]));
+    bool ok = nasalSys->removeCommand(commandName);
+    if (!ok) {
+        return naNum(0);
+    }
+    return naNum(1);
 }
 
 static naRef f_open(naContext c, naRef me, int argc, naRef* args)
@@ -794,18 +858,18 @@ static naRef ftype(naContext ctx, const SGPath& f)
 }
 
 // io.stat with UTF-8 path support, replaces the default one in
-// Nasal iolib.c which does not hsupport UTF-8 paths
+// Nasal iolib.c which does not support UTF-8 paths
 static naRef f_custom_stat(naContext ctx, naRef me, int argc, naRef* args)
 {
     naRef pathArg = argc > 0 ? naStringValue(ctx, args[0]) : naNil();
     if (!naIsString(pathArg))
         naRuntimeError(ctx, "bad argument to stat()");
-    
+
     const auto path = SGPath::fromUtf8(naStr_data(pathArg));
     if (!path.exists()) {
         return naNil();
     }
-    
+
     const SGPath filename = fgValidatePath(path, false );
     if (filename.isNull()) {
         SG_LOG(SG_NASAL, SG_ALERT, "stat(): reading '" <<
@@ -815,10 +879,10 @@ static naRef f_custom_stat(naContext ctx, naRef me, int argc, naRef* args)
         naRuntimeError(ctx, "stat(): access denied (unauthorized directory)");
         return naNil();
     }
-   
+
     naRef result = naNewVector(ctx);
     naVec_setsize(ctx, result, 12);
-    
+
     // every use in fgdata/Nasal only uses stat() to test existence of
     // files, not to check any of this information.
     int n = 0;
@@ -926,33 +990,34 @@ static naRef f_systime(naContext c, naRef me, int argc, naRef* args)
 }
 
 // Table of extension functions.  Terminate with zeros.
-static struct { const char* name; naCFunction func; } funcs[] = {
-    { "getprop",   f_getprop },
-    { "setprop",   f_setprop },
-    { "print",     f_print },
-    { "logprint",  f_logprint },
-    { "_fgcommand", f_fgcommand },
-    { "settimer",  f_settimer },
-    { "maketimer", f_makeTimer },
-    { "_setlistener", f_setlistener },
-    { "removelistener", f_removelistener },
-    { "addcommand", f_addCommand },
-    { "removecommand", f_removeCommand },
-    { "_cmdarg",  f_cmdarg },
-    { "_interpolate",  f_interpolate },
-    { "rand",  f_rand },
-    { "srand",  f_srand },
-    { "abort", f_abort },
-    { "directory", f_directory },
-    { "resolvepath", f_resolveDataPath },
-    { "finddata", f_findDataDir },
-    { "parsexml", f_parsexml },
-    { "parse_markdown", f_parse_markdown },
-    { "md5", f_md5 },
-    { "systime", f_systime },
-    { "maketimestamp", f_maketimeStamp},
-    { 0, 0 }
-};
+static struct { const char* name; naCFunction func;
+} funcs[] = {
+    {"getprop", f_getprop},
+    {"setprop", f_setprop},
+    {"print", f_print},
+    {"logprint", f_logprint},
+    {"_fgcommand", f_fgcommand},
+    {"settimer", f_settimer},
+    {"maketimer", f_makeTimer},
+    {"makesingleshot", f_makeSingleShot},
+    {"_setlistener", f_setlistener},
+    {"removelistener", f_removelistener},
+    {"addcommand", f_addCommand},
+    {"removecommand", f_removeCommand},
+    {"_cmdarg", f_cmdarg},
+    {"_interpolate", f_interpolate},
+    {"rand", f_rand},
+    {"srand", f_srand},
+    {"abort", f_abort},
+    {"directory", f_directory},
+    {"resolvepath", f_resolveDataPath},
+    {"finddata", f_findDataDir},
+    {"parsexml", f_parsexml},
+    {"parse_markdown", f_parse_markdown},
+    {"md5", f_md5},
+    {"systime", f_systime},
+    {"maketimestamp", f_maketimeStamp},
+    {0, 0}};
 
 naRef FGNasalSys::cmdArgGhost()
 {
@@ -968,6 +1033,7 @@ void FGNasalSys::initLogLevelConstants()
     hashset(_globals, "LOG_ALERT", naNum(SG_ALERT));
     hashset(_globals, "DEV_WARN", naNum(SG_DEV_WARN));
     hashset(_globals, "DEV_ALERT", naNum(SG_DEV_ALERT));
+    hashset(_globals, "MANDATORY_INFO", naNum(SG_MANDATORY_INFO));
 }
 
 void FGNasalSys::setCmdArg(SGPropertyNode* aNode)
@@ -999,7 +1065,7 @@ void FGNasalSys::init()
     hashset(_globals, "utf8", naInit_utf8(_context));
 
     initLogLevelConstants();
-    
+
     // Add our custom extension functions:
     for(i=0; funcs[i].name; i++)
         hashset(_globals, funcs[i].name,
@@ -1016,14 +1082,23 @@ void FGNasalSys::init()
     naSave(_context, _string);
     initNasalString(_globals, _string, _context);
 
-    initNasalPositioned(_globals, _context);
-    initNasalPositioned_cppbind(_globals, _context);
-    initNasalAircraft(_globals, _context);
-    NasalClipboard::init(this);
-    initNasalCanvas(_globals, _context);
-    initNasalCondition(_globals, _context);
-    initNasalHTTP(_globals, _context);
-    initNasalSGPath(_globals, _context);
+#if defined (BUILDING_TESTSUITE)
+    initNasalUnitTestCppUnit(_globals, _context);
+#else
+    initNasalUnitTestInSim(_globals, _context);
+#endif
+
+    if (!global_nasalMinimalInit) {
+        initNasalPositioned(_globals, _context);
+        initNasalFlightPlan(_globals, _context);
+        initNasalPositioned_cppbind(_globals, _context);
+        initNasalAircraft(_globals, _context);
+        NasalClipboard::init(this);
+        initNasalCanvas(_globals, _context);
+        initNasalCondition(_globals, _context);
+        initNasalHTTP(_globals, _context);
+        initNasalSGPath(_globals, _context);
+    }
 
     NasalTimerObj::init("Timer")
       .method("start", &TimerObj::start)
@@ -1039,11 +1114,18 @@ void FGNasalSys::init()
         .method("elapsedUSec", &TimeStampObj::elapsedUSec)
         ;
 
+    // everything after here, skip if we're doing minimal init, so
+    // we don'tload FG_DATA/Nasal or add-ons
+    if (global_nasalMinimalInit) {
+        _inited = true;
+        return;
+    }
+
     flightgear::addons::initAddonClassesForNasal(_globals, _context);
 
     // Now load the various source files in the Nasal directory
     simgear::Dir nasalDir(SGPath(globals->get_fg_root(), "Nasal"));
-    loadScriptDirectory(nasalDir);
+    loadScriptDirectory(nasalDir, globals->get_props()->getNode("/sim/nasal-load-priority"));
 
     // Add modules in Nasal subdirectories to property tree
     simgear::PathList directories = nasalDir.children(simgear::Dir::TYPE_DIR+
@@ -1077,6 +1159,8 @@ void FGNasalSys::shutdown()
     }
 
     shutdownNasalPositioned();
+    shutdownNasalFlightPlan();
+    shutdownNasalUnitTestInSim();
 
     for (auto l : _listener)
         delete l.second;
@@ -1143,11 +1227,10 @@ void FGNasalSys::update(double)
 {
     if( NasalClipboard::getInstance() )
         NasalClipboard::getInstance()->update();
-    if(!_dead_listener.empty()) {
-        vector<FGNasalListener *>::iterator it, end = _dead_listener.end();
-        for(it = _dead_listener.begin(); it != end; ++it) delete *it;
-        _dead_listener.clear();
-    }
+
+    std::for_each(_dead_listener.begin(), _dead_listener.end(),
+                  []( FGNasalListener* l) { delete l; });
+    _dead_listener.clear();
 
     if (!_loadList.empty())
     {
@@ -1186,17 +1269,33 @@ bool pathSortPredicate(const SGPath& p1, const SGPath& p2)
   return p1.file() < p2.file();
 }
 
-// Loads all scripts in given directory
-void FGNasalSys::loadScriptDirectory(simgear::Dir nasalDir)
+// Loads all scripts in given directory, with an optional partial ordering of
+// files defined in loadorder.
+void FGNasalSys::loadScriptDirectory(simgear::Dir nasalDir, SGPropertyNode* loadorder)
 {
     simgear::PathList scripts = nasalDir.children(simgear::Dir::TYPE_FILE, ".nas");
+
+    if (loadorder != nullptr && loadorder->hasChild("file")) {
+      // Load any scripts defined in the loadorder in order, removing them from
+      // the list so they don't get loaded twice.
+      simgear::PropertyList files = loadorder->getChildren("file");
+
+      auto loadAndErase = [ &scripts, &nasalDir, this ] (SGPropertyNode_ptr n) {
+        SGPath p = SGPath(nasalDir.path(), n->getStringValue());
+        auto script = std::find(scripts.begin(), scripts.end(), p);
+        if (script != scripts.end()) {
+          this->loadModule(p, p.file_base().c_str());
+          scripts.erase(script);
+        }
+      };
+
+      std::for_each(files.begin(), files.end(), loadAndErase);
+    }
+
+    // Load any remaining scripts.
     // Note: simgear::Dir already reports file entries in a deterministic order,
     // so a fixed loading sequence is guaranteed (same for every user)
-    for (unsigned int i=0; i<scripts.size(); ++i) {
-      SGPath fullpath(scripts[i]);
-      SGPath file = fullpath.file();
-      loadModule(fullpath, file.base().c_str());
-    }
+    std::for_each(scripts.begin(), scripts.end(), [this](SGPath p) { this->loadModule(p, p.file_base().c_str()); });
 }
 
 // Create module with list of scripts
@@ -1311,7 +1410,7 @@ void FGNasalSys::logError(naContext context)
 {
     string errorMessage = naGetError(context);
     SG_LOG(SG_NASAL, SG_ALERT, "Nasal runtime error: " << errorMessage);
-    
+
     string_list nasalStack;
     logNasalStack(context, nasalStack);
     flightgear::sentryReportNasalError(errorMessage, nasalStack);
@@ -1325,16 +1424,16 @@ void FGNasalSys::logNasalStack(naContext context, string_list& stack)
 
     stack.push_back(string{naStr_data(naGetSourceFile(context, 0))} +
                     ", line " + std::to_string(naGetLine(context, 0)));
-    
+
     SG_LOG(SG_NASAL, SG_ALERT,
            "  at " << naStr_data(naGetSourceFile(context, 0)) <<
            ", line " << naGetLine(context, 0));
-    
+
     for(int i=1; i<stack_depth; i++) {
         SG_LOG(SG_NASAL, SG_ALERT,
                "  called from: " << naStr_data(naGetSourceFile(context, i)) <<
                ", line " << naGetLine(context, i));
-        
+
         stack.push_back(string{naStr_data(naGetSourceFile(context, i))} +
                         ", line " + std::to_string(naGetLine(context, i)));
     }
@@ -1629,8 +1728,7 @@ naRef FGNasalSys::setListener(naContext c, int argc, naRef* args)
 naRef FGNasalSys::removeListener(naContext c, int argc, naRef* args)
 {
     naRef id = argc > 0 ? args[0] : naNil();
-    map<int, FGNasalListener *>::iterator it = _listener.find(int(id.num));
-
+    auto it = _listener.find(int(id.num));
     if(!naIsNum(id) || it == _listener.end() || it->second->_dead) {
         naRuntimeError(c, "removelistener() with invalid listener id");
         return naNil();
@@ -1665,17 +1763,18 @@ void FGNasalSys::addCommand(naRef func, const std::string& name)
     _commands[name] = cmd;
 }
 
-void FGNasalSys::removeCommand(const std::string& name)
+bool FGNasalSys::removeCommand(const std::string& name)
 {
-    NasalCommandDict::iterator it = _commands.find(name);
+    auto it = _commands.find(name);
     if (it == _commands.end()) {
         SG_LOG(SG_NASAL, SG_WARN, "remove of unknwon command:" << name);
-        return;
+        return false;
     }
 
     // will delete the NasalCommand instance
-    globals->get_commands()->removeCommand(name);
+    bool ok = globals->get_commands()->removeCommand(name);
     _commands.erase(it);
+    return ok;
 }
 
 void FGNasalSys::addPersistentTimer(TimerObj* pto)
