@@ -125,6 +125,41 @@ static naRef f_getType(naContext c, naRef me, int argc, naRef* args)
     return NASTR(t);
 }
 
+// Check if type of a property is numeric (returns 0 or 1).
+// Forms:
+//    props.Node.isNumeric(string relative_path);
+//    props.Node.isNumeric();
+static naRef f_isNumeric(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    switch(node->getType()) {
+    case props::INT:
+    case props::LONG:
+    case props::FLOAT:
+    case props::DOUBLE: return naNum(true);
+    default:
+        break;
+    }
+    return naNum(false);
+}
+
+// Check if type of a property is integer (returns 0 or 1).
+// Forms:
+//    props.Node.isInt(string relative_path);
+//    props.Node.isInt();
+static naRef f_isInt(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    if ((node->getType() == props::INT) || (node->getType() == props::LONG)) {
+        return naNum(true);
+    }
+    return naNum(false);
+}
+
 
 // Get an attribute of a property by name (returns true/false).
 // Forms:
@@ -380,6 +415,20 @@ static naRef f_setBoolValue(naContext c, naRef me, int argc, naRef* args)
     return naNum(node->setBoolValue(naTrue(val) ? true : false));
 }
 
+static naRef f_toggleBoolValue(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    if (node->getType() != props::BOOL) {
+        naRuntimeError(c, "props.toggleBoolValue() on non-bool prop");
+    }
+
+    const auto val = node->getBoolValue();
+    return naNum(node->setBoolValue(val ? false : true));
+}
+
 static naRef f_setDoubleValue(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
@@ -395,6 +444,38 @@ static naRef f_setDoubleValue(naContext c, naRef me, int argc, naRef* args)
     return naNum(node->setDoubleValue(r.num));
 }
 
+static naRef f_adjustValue(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 1, false);
+    naRef r = naNumValue(naVec_get(argv, 0));
+    if (naIsNil(r))
+        naRuntimeError(c, "props.adjustValue() with non-number");
+
+    if (SGMisc<double>::isNaN(r.num)) {
+        naRuntimeError(c, "props.adjustValue() passed a NaN");
+    }
+
+    switch(node->getType()) {
+    case props::BOOL:
+    case props::INT:
+    case props::LONG:
+    case props::FLOAT:
+    case props::DOUBLE:
+        // fall through
+        break;
+
+    default:
+        naRuntimeError(c, "props.adjustValue() called on non-numeric type");
+        return naNil();
+    }
+
+    const auto dv = node->getDoubleValue();
+    return naNum(node->setDoubleValue(dv + r.num));
+}
+
 // Forward declaration
 static naRef f_setChildrenHelper(naContext c, SGPropertyNode_ptr node, char* name, naRef val);
 
@@ -810,6 +891,7 @@ static struct {
     { f_setValues,          "_setValues"          },
     { f_setIntValue,        "_setIntValue"        },
     { f_setBoolValue,       "_setBoolValue"       },
+    { f_toggleBoolValue,    "_toggleBoolValue"    },
     { f_setDoubleValue,     "_setDoubleValue"     },
     { f_getParent,          "_getParent"          },
     { f_getChild,           "_getChild"           },
@@ -826,6 +908,9 @@ static struct {
     { f_getNode,            "_getNode"            },
     { f_new,                "_new"                },
     { f_globals,            "_globals"            },
+    { f_isNumeric,          "_isNumeric"          },
+    { f_isInt,              "_isInt"              },
+    { f_adjustValue,        "_adjustValue"        },
     { 0, 0 }
 };
 
