@@ -25,7 +25,6 @@
 #include "TimeManager.hxx"
 
 #include <simgear/timing/sg_time.hxx>
-#include <simgear/structure/event_mgr.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/timing/lowleveltime.h>
 #include <simgear/structure/commands.hxx>
@@ -84,7 +83,7 @@ void TimeManager::init()
   _warp = fgGetNode("/sim/time/warp", true);
   _warp->addChangeListener(this);
   _maxFrameRate  = fgGetNode("/sim/frame-rate-throttle-hz", true);
-
+  _localTimeStringNode = fgGetNode("/sim/time/local-time-string", true);
   _warpDelta = fgGetNode("/sim/time/warp-delta", true);
   
   SGPath zone(globals->get_fg_root());
@@ -93,9 +92,6 @@ void TimeManager::init()
   _impl = new SGTime(globals->get_aircraft_position(), zone, _timeOverride->getLongValue());
   
   _warpDelta->setDoubleValue(0.0);
-  
-  globals->get_event_mgr()->addTask("updateLocalTime", this,
-                            &TimeManager::updateLocalTime, 30*60 );
   updateLocalTime();
   
   _impl->update(globals->get_aircraft_position(), _timeOverride->getLongValue(),
@@ -141,7 +137,9 @@ void TimeManager::unbind()
     _frameRate.clear();
     _frameLatency.clear();
     _frameRateWorst.clear();
-    
+    _frameWait.clear();
+    _maxFrameRate.clear();
+
     _sceneryLoaded.clear();
     _modelHz.clear();
     _timeDelta.clear();
@@ -174,7 +172,6 @@ void TimeManager::shutdown()
   delete _impl;
   _impl = NULL;
   _inited = false;
-  globals->get_event_mgr()->removeTask("updateLocalTime");
 }
 
 void TimeManager::valueChanged(SGPropertyNode* aProp)
@@ -342,11 +339,18 @@ void TimeManager::update(double dt)
         _warp->setDoubleValue(_warp->getDoubleValue() + warpOffset);
     }
 
+    const auto d2 = distSqr(_lastTimeZoneCheckPosition, globals->get_aircraft_position_cart());
+    const auto oneNmSqr = SG_NM_TO_METER * SG_NM_TO_METER;
+    if (d2 > oneNmSqr) {
+        updateLocalTime();
+    }
+
   _lastClockFreeze = freeze;
   _impl->update(globals->get_aircraft_position(),
                _timeOverride->getLongValue(),
                _warp->getIntValue());
 
+  updateLocalTimeString();
   computeFrameRate();
 }
 
@@ -386,7 +390,26 @@ void TimeManager::throttleUpdateRate()
 // periodic time updater wrapper
 void TimeManager::updateLocalTime() 
 {
-  _impl->updateLocal(globals->get_aircraft_position(), globals->get_fg_root() / "Timezone");
+    _lastTimeZoneCheckPosition = globals->get_aircraft_position_cart();
+    _impl->updateLocal(globals->get_aircraft_position(), globals->get_fg_root() / "Timezone");
+    // synchronous update, since somebody might need that
+    updateLocalTimeString();
+}
+
+void TimeManager::updateLocalTimeString()
+{
+    time_t cur_time = _impl->get_cur_time();
+    struct tm* aircraftLocalTime = fgLocaltime(&cur_time, _impl->get_zonename());
+    static char buf[16];
+    snprintf(buf, 16, "%.2d:%.2d:%.2d",
+             aircraftLocalTime->tm_hour,
+             aircraftLocalTime->tm_min, aircraftLocalTime->tm_sec);
+
+    // check against current string to avoid changes all the time
+    const char* s = _localTimeStringNode->getStringValue();
+    if (strcmp(s, buf) != 0) {
+        _localTimeStringNode->setStringValue(buf);
+    }
 }
 
 void TimeManager::initTimeOffset()
@@ -414,19 +437,19 @@ void TimeManager::setTimeOffset(const std::string& offset_type, long int offset)
   if ( offset_type == "real" ) {
       warp = 0;
   } else if ( offset_type == "dawn" ) {
-      warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 90.0, true, "sun" );
+      warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 90.0, true, true );
   } else if ( offset_type == "morning" ) {
-     warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 75.0, true, "sun" );
+     warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 75.0, true, true );
   } else if ( offset_type == "noon" ) {
-     warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 0.0, true, "sun" );
+     warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 0.0, true, true );
   } else if ( offset_type == "afternoon" ) {
-    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 75.0, false, "sun" );
+    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 75.0, false, true );
   } else if ( offset_type == "dusk" ) {
-    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 90.0, false, "sun" );
+    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 90.0, false, true );
   } else if ( offset_type == "evening" ) {
-    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 100.0, false, "sun" );
+    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 100.0, false, true );
   } else if ( offset_type == "midnight" ) {
-    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 180.0, false, "sun" );
+    warp = fgTimeSecondsUntilBodyAngle( cur_time, loc, 180.0, false, true );
   } else if ( offset_type == "system-offset" ) {
     warp = offset;
     orig_warp = 0;
@@ -462,6 +485,10 @@ void TimeManager::setTimeOffset(const std::string& offset_type, long int offset)
             << _warp->getIntValue() );
 }
 
+double TimeManager::getSimSpeedUpFactor() const
+{
+    return _simTimeFactor->getDoubleValue();
+}
 
 // Register the subsystem.
 SGSubsystemMgr::Registrant<TimeManager> registrantTimeManager(
