@@ -74,6 +74,7 @@
 #include <Main/fg_init.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/sentryIntegration.hxx>
 
 #include "TrafficMgr.hxx"
 
@@ -127,8 +128,9 @@ public:
     return _isFinished;
   }
 
-  virtual void run()
+  void run() override
   {
+      flightgear::sentryThreadReportXMLErrors(false);
       for (const auto& p : _trafficDirPaths) {
           parseTrafficDir(p);
           if (_cancelThread) {
@@ -246,7 +248,7 @@ public:
                 snprintf(buffer, 16, "%d", acCounter);
                 requiredAircraft = buffer;
             }
-            SG_LOG(SG_AI, SG_DEBUG, "Adding flight: " << callsign << " "
+            SG_LOG(SG_AI, SG_BULK, "Adding flight: " << callsign << " "
                    << fltrules << " "
                    << departurePort << " "
                    << arrivalPort << " "
@@ -382,9 +384,13 @@ private:
             SG_LOG(SG_AI, SG_DEBUG, "parsing traffic in:" << p);
             simgear::PathList trafficFiles = d2.children(simgear::Dir::TYPE_FILE, ".xml");
             for (const auto& xml : trafficFiles) {
-                readXML(xml, *this);
-                if (_cancelThread) {
-                    return;
+                try {
+                    readXML(xml, *this);
+                    if (_cancelThread) {
+                        return;
+                    }
+                } catch (sg_exception& e) {
+                    SG_LOG(SG_AI, SG_WARN, "XML error parsing traffic file:" << xml << "\n\t" << e.getFormattedMessage());
                 }
             }
         } // of sub-directories iteration
@@ -493,6 +499,11 @@ void FGTrafficManager::shutdown()
         cachefile.close();
     }
     scheduledAircraft.clear();
+
+    for (auto flight : flights) {
+        for (auto scheduled : flight.second)
+            delete scheduled;
+    }
     flights.clear();
 
     currAircraft = scheduledAircraft.begin();
