@@ -80,15 +80,45 @@ public:
         // insert waypt for the dpearture runway
         auto dr = new RunwayWaypt(thePlan->departureRunway(), thePlan);
         dr->setFlag(WPT_DEPARTURE);
+        dr->setFlag(WPT_GENERATED);
         thePlan->insertWayptAtIndex(dr, 0);
         
         if (thePlan->sid()) {
             WayptVec sidRoute;
-            bool ok = thePlan->sid()->route(thePlan->departureRunway(), nullptr, sidRoute);
+            bool ok = thePlan->sid()->route(thePlan->departureRunway(), thePlan->sidTransition(), sidRoute);
             if (!ok)
                 throw sg_exception("failed to route via SID");
             int insertIndex = 1;
             for (auto w : sidRoute) {
+                w->setFlag(WPT_DEPARTURE);
+                w->setFlag(WPT_GENERATED);
+                thePlan->insertWayptAtIndex(w, insertIndex++);
+            }
+        }
+    }
+    
+    void arrivalChanged() override
+    {
+        // mimic the default delegate, inserting the STAR waypoints
+        
+        // clear anything existing
+        thePlan->clearWayptsWithFlag(WPT_ARRIVAL);
+        
+        // insert waypt for the destination runway
+        auto dr = new RunwayWaypt(thePlan->destinationRunway(), thePlan);
+        dr->setFlag(WPT_ARRIVAL);
+        dr->setFlag(WPT_GENERATED);
+        auto leg = thePlan->insertWayptAtIndex(dr, -1);
+        
+        if (thePlan->star()) {
+            WayptVec starRoute;
+            bool ok = thePlan->star()->route(thePlan->destinationRunway(), thePlan->starTransition(), starRoute);
+            if (!ok)
+                throw sg_exception("failed to route via STAR");
+            int insertIndex = leg->index();
+            for (auto w : starRoute) {
+                w->setFlag(WPT_ARRIVAL);
+                w->setFlag(WPT_GENERATED);
                 thePlan->insertWayptAtIndex(w, insertIndex++);
             }
         }
@@ -177,7 +207,7 @@ void RNAVProcedureTests::testBasic()
 void RNAVProcedureTests::testHeadingToAlt()
 {
     auto vhhh = FGAirport::findByIdent("VHHH");
-   // FGTestApi::setUp::logPositionToKML("heading_to_alt");
+//    FGTestApi::setUp::logPositionToKML("heading_to_alt");
 
     auto rm = globals->get_subsystem<FGRouteMgr>();
     auto fp = new FlightPlan;
@@ -195,7 +225,7 @@ void RNAVProcedureTests::testHeadingToAlt()
     wp->setAltitude(4000, RESTRICT_ABOVE);
     fp->insertWayptAtIndex(wp, 1); // between the runway WP and HAZEL
 
-   // FGTestApi::writeFlightPlanToKML(fp);
+  //  FGTestApi::writeFlightPlanToKML(fp);
 
     auto depRwy = fp->departureRunway();
     
@@ -213,8 +243,10 @@ void RNAVProcedureTests::testHeadingToAlt()
     pilot->setSpeedKts(200);
     pilot->flyGPSCourse(m_gps);
     
+    CPPUNIT_ASSERT_EQUAL(fp->currentIndex(), 0);
+
     // check we sequence to the heading-to-alt wp
-    bool ok = FGTestApi::runForTimeWithCheck(240.0, [fp] () {
+    bool ok = FGTestApi::runForTimeWithCheck(300.0, [fp] () {
       if (fp->currentIndex() == 1) {
           return true;
       }
@@ -386,7 +418,11 @@ void RNAVProcedureTests::testEGPH_TLA6C()
     CPPUNIT_ASSERT_EQUAL(std::string{"D242H"}, d242Wpt->ident());
     CPPUNIT_ASSERT_EQUAL(true, d242Wpt->flag(WPT_OVERFLIGHT));
     
-    CPPUNIT_ASSERT_EQUAL(std::string{"D346T"}, fp->legAtIndex(3)->waypoint()->ident());
+    const auto wp3Ident = fp->legAtIndex(3)->waypoint()->ident();
+    
+    // depeding which versino fo the procedures we loaded, we can find
+    // one ID or the other
+    CPPUNIT_ASSERT((wp3Ident == "D346T") || (wp3Ident == "D345T"));
     
   //  FGTestApi::writeFlightPlanToKML(fp);
 
@@ -519,3 +555,145 @@ void RNAVProcedureTests::testLFKC_AJO1R()
     });
     CPPUNIT_ASSERT(ok);
 }
+
+void RNAVProcedureTests::testTransitionsSID()
+{
+    auto kjfk = FGAirport::findByIdent("kjfk");
+    auto runway = kjfk->getRunwayByIdent("13L");
+    
+
+    auto sid = kjfk->selectSIDByTransition(runway, "CANDR");
+    // procedures not loaded, abandon test
+    if (!sid)
+        return;
+
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = new FlightPlan;
+    
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    fp->addDelegate(testDelegate);
+    
+    rm->setFlightPlan(fp);
+    FGTestApi::setUp::populateFPWithNasal(fp, "KJFK", "13L", "KCLE", "24R", "");
+    
+    fp->setSID(sid);
+    CPPUNIT_ASSERT_EQUAL(8, fp->numLegs());
+    auto wp = fp->legAtIndex(6);
+    CPPUNIT_ASSERT_EQUAL(std::string{"CANDR"}, wp->waypoint()->ident());
+    CPPUNIT_ASSERT(rm->activate());
+}
+
+void RNAVProcedureTests::testTransitionsSTAR()
+{
+    auto kjfk = FGAirport::findByIdent("kjfk");
+    auto runway = kjfk->getRunwayByIdent("22L");
+    auto star = kjfk->selectSTARByTransition(runway, "SEY");
+    // procedures not loaded, abandon test
+    if (!star)
+        return;
+
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = new FlightPlan;
+    
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    fp->addDelegate(testDelegate);
+    
+    rm->setFlightPlan(fp);
+    FGTestApi::setUp::populateFPWithNasal(fp, "KBOS", "22R", "KJFK", "22L", "");
+    
+    fp->setSTAR(star);
+    CPPUNIT_ASSERT_EQUAL(9, fp->numLegs());
+    auto wp = fp->legAtIndex(1);
+    CPPUNIT_ASSERT_EQUAL(std::string{"SEY"}, wp->waypoint()->ident());
+    CPPUNIT_ASSERT(rm->activate());
+}
+
+void RNAVProcedureTests::testLEBL_LARP2F()
+{
+    auto lebl = FGAirport::findByIdent("LEBL");
+    auto sid = lebl->findSIDWithIdent("LARP1F.25L");
+    // procedures not loaded, abandon test
+    if (!sid)
+        return;
+
+    FGTestApi::setUp::logPositionToKML("procedure_LEBL-LARP2F");
+
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = new FlightPlan;
+    
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    fp->addDelegate(testDelegate);
+    
+    rm->setFlightPlan(fp);
+    FGTestApi::setUp::populateFPWithNasal(fp, "LEBL", "25L", "LEIB", "06", "");
+    
+    fp->setSID(sid);
+    // I don't know if this should pass or not! 
+    // If its a bug, wrap next line in CPPUNIT_ASSERT_ASSERTION_FAIL
+    CPPUNIT_ASSERT_EQUAL(fp->legAtIndex(1)->waypoint()->position(), SGGeod::fromDeg(0, 0)); 
+    CPPUNIT_ASSERT_EQUAL(std::string{"LARPA"}, fp->legAtIndex(8)->waypoint()->ident());
+    double d = fp->legAtIndex(8)->distanceAlongRoute();
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(46.6, d, 1.0); // ensure the route didn't blow up
+    
+    FGTestApi::writeFlightPlanToKML(fp);
+
+    CPPUNIT_ASSERT(rm->activate());
+    
+    FGRunwayRef departureRunway = fp->departureRunway();
+    
+    setupStandardGPS();
+    
+    FGTestApi::setPositionAndStabilise(departureRunway->threshold());
+    m_gpsNode->setStringValue("command", "leg");
+    
+    auto pilot = SGSharedPtr<FGTestApi::TestPilot>(new FGTestApi::TestPilot);
+    pilot->resetAtPosition(globals->get_aircraft_position());
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(departureRunway->headingDeg(), m_gpsNode->getDoubleValue("wp/leg-true-course-deg"), 0.5);
+    pilot->setCourseTrue(m_gpsNode->getDoubleValue("wp/leg-true-course-deg"));
+    pilot->setSpeedKts(220);
+    pilot->flyGPSCourse(m_gps);
+    pilot->setTargetAltitudeFtMSL(8000);
+    pilot->setVerticalFPM(1800);
+    FGTestApi::runForTime(20.0);
+    bool ok = FGTestApi::runForTimeWithCheck(180.0, [fp] () {
+        if (fp->currentIndex() == 1) {
+            return true;
+        }
+        return false;
+    });
+    CPPUNIT_ASSERT(ok);
+    
+    FGTestApi::runForTime(180.0);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(199, m_gpsNode->getDoubleValue("wp/leg-true-course-deg"), 0.5);
+}
+
+// This could probably be in a better place but this allows it to access TestDelegate.
+// Also it does relate to procedures as its a bug that only occurs with procedures
+void RNAVProcedureTests::testIndexOf()
+{
+	auto egkk = FGAirport::findByIdent("EGKK");
+    auto sid = egkk->findSIDWithIdent("SAM3P");
+    // procedures not loaded, abandon test
+    if (!sid)
+        return;
+
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = new FlightPlan;
+    
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    fp->addDelegate(testDelegate);
+    
+    rm->setFlightPlan(fp);
+    FGTestApi::setUp::populateFPWithNasal(fp, "EGKK", "08R", "EGJJ", "27", "LELNA");
+    
+    fp->setSID(sid);
+	FGPositioned::TypeFilter f{FGPositioned::VOR};
+    auto southamptonVOR = fgpositioned_cast<FGNavRecord>(FGPositioned::findClosestWithIdent("SAM", SGGeod::fromDeg(-1.25, 51.0), &f));
+	auto SAM = fp->legAtIndex(6)->waypoint();
+	CPPUNIT_ASSERT_EQUAL(southamptonVOR->ident(), SAM->ident());
+	CPPUNIT_ASSERT_EQUAL(6, fp->findWayptIndex(southamptonVOR));
+}
