@@ -20,12 +20,14 @@
 #include <QQuickView>
 #include <QQmlContext>
 #include <QCommandLineParser>
+#include <QScreen>
 
 #include "canvasitem.h"
 #include "applicationcontroller.h"
 #include "canvasdisplay.h"
 #include "canvasconnection.h"
 #include "canvaspainteddisplay.h"
+#include "WindowData.h"
 
 int main(int argc, char *argv[])
 {
@@ -37,9 +39,6 @@ int main(int argc, char *argv[])
 
     QCommandLineParser parser;
     parser.addPositionalArgument("config", QCoreApplication::translate("main", "JSON configuration to load"));
-    QCommandLineOption framelessOption(QStringList() << "frameless",
-                                   QCoreApplication::translate("main", "Use a frameless window"));
-    parser.addOption(framelessOption);
     parser.process(a);
 
     ApplicationController appController;
@@ -48,33 +47,19 @@ int main(int argc, char *argv[])
     qmlRegisterType<CanvasDisplay>("FlightGear", 1, 0, "CanvasDisplay");
     qmlRegisterType<CanvasPaintedDisplay>("FlightGear", 1, 0, "PaintedCanvasDisplay");
 
+    qmlRegisterUncreatableType<WindowData>("FlightGear", 1, 0, "WindowData", "Don't create me");
     qmlRegisterUncreatableType<CanvasConnection>("FlightGear", 1, 0, "CanvasConnection", "Don't create me");
     qmlRegisterUncreatableType<ApplicationController>("FlightGear", 1, 0, "Application", "Can't create");
 
-    QQuickView quickView;
-    appController.setWindow(&quickView);
-    if (parser.isSet(framelessOption)) {
-        quickView.setFlag(Qt::FramelessWindowHint, true);
-    }
-    quickView.rootContext()->setContextProperty("_application", &appController);
-
     const QStringList args = parser.positionalArguments();
 
     if (!args.empty()) {
         appController.setDaemonMode();
         appController.loadFromFile(args.front());
-    } else {
-        quickView.setWidth(1024);
-        quickView.setHeight(768);
     }
 
-    quickView.setSource(QUrl("qrc:///qml/mainMenu.qml"));
-    quickView.setResizeMode(QQuickView::SizeRootObjectToView);
-    quickView.show();
-
-    appController.restoreWindowState();
+    appController.createWindows();
 
     int result = a.exec();
-
     return result;
 }
